//*********************************************************************
// Copyright (C) 2005,  Lucian Constantin <struct_bylighting@yahoo.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//*********************************************************************

#include "aliasdialog.h"
#ifdef USING_QT3
#include <qlayout.h>
#include <qlabel.h>
#include <qhbox.h>
#include <qaccel.h>
#include <qgroupbox.h>
#include <qbuttongroup.h>
#endif
#ifdef USING_QT4
#include <Q3VBoxLayout>
#include <QLabel>
#include <q3hbox.h>
#include <q3accel.h>
#include <Q3GroupBox>
#include <Q3ButtonGroup>
#endif

#include <qlineedit.h>
#include <qpushbutton.h>
#include <algorithm>

//*****************************************************************************/
// alias dialog
//*****************************************************************************/
AliasDialog::AliasDialog(): QDialog(0, "alias", true) {
	Q3VBoxLayout *vly = new Q3VBoxLayout(this, 10);
	vly->setAutoAdd(true);

	// file1
	Q3HBox *hbox1 = new Q3HBox(this);
	hbox1->setSpacing(10);
	label1_ = new QLabel(tr("Alias: "), hbox1);
	alias_ = new QLineEdit(hbox1);

	// file2
	Q3HBox *hbox2 = new Q3HBox(this);
	hbox2->setSpacing(10);
	label2_ = new QLabel(tr("Replacement: "), hbox2);
	replacement_ = new QLineEdit(hbox2);


#ifdef USING_QT3
	Q3ButtonGroup *bgrp2 = new Q3ButtonGroup( 1, Q3GroupBox::Horizontal, tr("Usage"), this );
#else
	Q3ButtonGroup *bgrp2 = new Q3ButtonGroup( 1, Qt::Horizontal, tr("Usage"), this );
#endif
	QLabel *lb = new QLabel(tr("\"Alias\" text followed by an empty space will be dinamically replaced with\n    \"Replacement\" text specified above."), bgrp2);
	(void) lb;

	// accept/reject
	Q3HBox *hbox3 = new Q3HBox(this);
	hbox3->setSpacing(10);
	button_next_ = new QPushButton(tr("Next"), hbox3);
	button_previous_ = new QPushButton(tr("Previous"), hbox3);
	// some empty space
	new QWidget(hbox3);
	new QWidget(hbox3);
	button_delete_ = new QPushButton(tr("Delete"), hbox3);
	button_ok_ = new QPushButton(tr("OK"), hbox3);
	button_cancel_ = new QPushButton(tr("Cancel"), hbox3);

	connect(button_ok_, SIGNAL(clicked()), SLOT(clicked_ok()));
	connect(button_cancel_, SIGNAL(clicked()), SLOT(reject()));
	connect(button_next_, SIGNAL(clicked()), this, SLOT(clicked_next()));
	connect(button_previous_, SIGNAL(clicked()), this, SLOT(clicked_previous()));
	connect(button_delete_, SIGNAL(clicked()), this, SLOT(clicked_delete()));

	alist_ = Global::instance()->alist;
	it_ = alist_.begin();
	if (it_ != alist_.end()) {
		alias_->setText((*it_).alias);
		replacement_->setText((*it_).replace);
	}
	else {
		alist_.push_back(Global::Alias(QString(""), QString("")));
		it_ = alist_.begin();
	}

}

void AliasDialog::clicked_next() {
	if (!alist_.empty()) {
		(*it_).alias = alias_->text();
		(*it_).replace = replacement_->text();
		(*it_).update();
	}
	else {
		alist_.push_back(Global::Alias(alias_->text(), replacement_->text()));
		it_ = alist_.begin();
	}

	++it_;
	if (it_ != alist_.end()) {
		alias_->setText((*it_).alias);
		replacement_->setText((*it_).replace);
	}
	else {
		alias_->setText("");
		replacement_->setText("");
		alist_.push_back(Global::Alias(QString(""), QString("")));
		--it_;
	}
}


void AliasDialog::clicked_previous() {
	(*it_).alias = alias_->text();
	(*it_).replace = replacement_->text();
	(*it_).update();

	if (it_ != alist_.begin())
		--it_;

	alias_->setText((*it_).alias);
	replacement_->setText((*it_).replace);
}

bool isEmptyAlias(Global::Alias& alias) {
	if (alias.alias.isEmpty() || alias.replace.isEmpty())
		return true;
	return false;
}

void updateAlias(Global::Alias& alias) {
	alias.update();
}

void AliasDialog::clicked_ok() {
	if (!alist_.empty()) {
		// save current alias
		(*it_).alias = alias_->text();
		(*it_).replace = replacement_->text();
		(*it_).update();
	}


	// remove empty aliases
	list<Global::Alias>::iterator new_end;
	new_end = remove_if(alist_.begin(), alist_.end(), isEmptyAlias);
	alist_.erase(new_end, alist_.end());

	// update reversed field
	for_each(alist_.begin(), alist_.end(), updateAlias);

	Global::instance()->alist = alist_;
	accept();
}

void AliasDialog::clicked_delete() {
	if (!alist_.empty())
		alist_.erase(it_);

	alias_->setText("");
	replacement_->setText("");

	// go to begining of the list
	it_ = alist_.begin();
	if (it_ != alist_.end()) {
		alias_->setText((*it_).alias);
		replacement_->setText((*it_).replace);
	}
}

