//****************************************************************************
// Copyright (C) 2005,  Lucian Constantin <struct_bylighting@yahoo.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//****************************************************************************

#include "briefedit.h"
#include "qwerty.h"
#include <qclipboard.h>
#include <qapplication.h>
#include "qtutils.h"

static const char *full_line_err = "Full line selection required!";

int BriefEdit::getAccLines() {
	if (!Global::instance()->acc_)
		return 0;
	return Global::instance()->acclines_;
}


void BriefEdit::cut() {
	if (Global::instance()->acc_) {
		QString str = selectedText();
		Global::instance()->accstr_ += str;
		Global::instance()->acclines_++;
	}
	Q3MultiLineEdit::cut();
}
void BriefEdit::copy() {
	if (Global::instance()->acc_) {
		QString str = selectedText();
		Global::instance()->accstr_ += str;
		Global::instance()->acclines_++;
	}
	Q3MultiLineEdit::copy();
}

void BriefEdit::paste() {
	Global *glb = Global::instance();
	QClipboard *clp = QApplication::clipboard();

	// in clipboard accumulation mode just dump whatevere is in the buffer and turn off the accumulation mode
	if (Global::instance()->acc_) {
		Global::instance()->acc_ = false;
		insert(Global::instance()->accstr_);
		Global::instance()->accstr_ = "";
		Global::instance()->acclines_ = 0;
		return;
	}

#ifndef ROSPELL_CYGWIN
	if (glb->lang == LANG_RO_COMMABELOW) { // translation to romanian commabelow
		{
			QString cliptext = clp->text(QClipboard::Selection);
			if (QTUTILS::hasBookmarkers(cliptext))
				cliptext = QTUTILS::cleanBookmarkers(cliptext);
			const QString converted = QTUTILS::cedilla2commabelow(cliptext);
			clp->setText(converted, QClipboard::Selection);
		}
		{
			QString cliptext = clp->text(QClipboard::Clipboard);
			if (QTUTILS::hasBookmarkers(cliptext))
				cliptext = QTUTILS::cleanBookmarkers(cliptext);
			const QString converted = QTUTILS::cedilla2commabelow(cliptext);
			clp->setText(converted, QClipboard::Clipboard);
		}

	}
	else {
#endif
		// only bookmarkers cleaning
		{
			QString cliptext = clp->text(QClipboard::Selection);
			if (QTUTILS::hasBookmarkers(cliptext)) {
				cliptext = QTUTILS::cleanBookmarkers(cliptext);
				clp->setText(cliptext, QClipboard::Selection);
			}
		}
		{
			QString cliptext = clp->text(QClipboard::Clipboard);
			if (QTUTILS::hasBookmarkers(cliptext)) {
				cliptext = QTUTILS::cleanBookmarkers(cliptext);
				clp->setText(cliptext, QClipboard::Clipboard);
			}
		}
#ifndef ROSPELL_CYGWIN
	}
#endif

	Q3MultiLineEdit::paste();
}

BriefEdit::BriefEdit(Editor *editor, QWidget * parent, const char * name): Q3MultiLineEdit(parent, name) {
	editor_ = editor;
	normal_insert_ = true;
	recording_ = 0;
	alias_buf_ = "";
	alist_ = Global::instance()->alist;
	setTextFormat(Qt::PlainText);
}


void BriefEdit::selectLine(int para, int index) {
	getCursorPosition(&para, &index);
	QString line = textLine(para);
	int len = line.length();
	int lines = numLines();
	if (para == (lines - 1)) {
		setSelection(para, 0, para, len);
	}
	else {
		setSelection(para, 0, para + 1, 0);
	}

	normal_insert_ = false;
}

void BriefEdit::keyPressEvent(QKeyEvent *e) {
	LOG("BriefEdit::keyPressEvent: %x/%x", e->state(), e->key());
	if (recording_) {
		list_.push_back(*e);
	}
	int para;
	int index;
	getCursorPosition(&para, &index);
	int key = e->key();

	// preserve bookmark if necessary by jumping to the next char on the line
	if (index == 0) {
		QString txt = textLine(para);
		if (QTUTILS::startsWithBookmarker(txt)) {
			// when selecting by pressing shift and moving the cursor
			// do not do the jump since it will mess up the selection
//			if (e->state() & Qt::ShiftButton &&
			if ( (key == Qt::Key_Up ||
			     key == Qt::Key_Down ||
			     key == Qt::Key_Left ||
			     key == Qt::Key_Right)) ;
			else {
				setCursorPosition(para, 1);
				index = 1;
			}
		}
	}

	// numeric pad + and - keys
	if ((unsigned) e->state() & Qt::Keypad) {
		normal_insert_ = true;

		if (key == Qt::Key_Plus) {
			LOG("key numpad plus");
			if (!hasMarkedText())
				selectLine(para, index);
			copy();
			deselect();
			setCursorPosition(para, index, false);
			return;
		}
		else if (key == Qt::Key_Minus) {
			LOG("key numpad minus");
			if (!hasMarkedText())
				selectLine(para, index);
			cut();
			// force an update of cursor position and accumulator on status bar
			getCursorPosition(&para, &index);
			setCursorPosition(para, index, false);
			return;
		}
	}

	// insert key
	if (key == Qt::Key_Insert) {
		if (!normal_insert_) {
			setCursorPosition(para, 0, false);
		}
		paste();
		return;
	}

	// enter key
	if (key == Qt::Key_Return) {
		insert("\n");
		if (para != 0) {
			QString tmp = textLine(para);
			// if necessary get rid of the bookmark
			if (QTUTILS::startsWithBookmarker(tmp))
				tmp = tmp.mid(1);
			int len = tmp.length();
			QString toinsert = "";
			int i;
			for (i = 0; i < len; i++) {
				if (tmp.at(i) == ' ' || tmp.at(i) == '\t')
					toinsert += tmp.at(i);
				else
					break;
			}
			QString n = tmp.mid(i);
			if (n.startsWith("if") || n.startsWith("else") || n.startsWith("for") || n.startsWith("while"))
				toinsert += '\t';
			insert(toinsert);
		}
		
		// force update of cursor postition
		getCursorPosition(&para, &index);
		setCursorPosition(para, index);
		
		return;
	}

	// romanian special characters
	Global *glb = Global::instance();
	if ((glb->lang == LANG_RO_COMMABELOW || glb->lang == LANG_RO_CEDILLA) &&
	    e->state() & Qt::AltButton) {
		QString str;
		bool done = false;
		if (key == Qt::Key_A) {
			if (e->state() & Qt::ShiftButton)
				str = QString::fromUtf8("Ă");
			else
				str = QString::fromUtf8("ă");
			done = true;
		}
		else if (key == Qt::Key_S && glb->lang == LANG_RO_COMMABELOW) {
			if (e->state() & Qt::ShiftButton)
				str = QString::fromUtf8("Ș");
			else
				str = QString::fromUtf8("ș");
			done = true;
		}
		else if (key == Qt::Key_S) {
			if (e->state() & Qt::ShiftButton)
				str = QString::fromUtf8("Ş");
			else
				str = QString::fromUtf8("ş");
			done = true;
		}
		else if (key == Qt::Key_Q) {
			if (e->state() & Qt::ShiftButton)
				str = QString::fromUtf8("Â");
			else
				str = QString::fromUtf8("â");
			done = true;
		}
		else if (key == Qt::Key_T && glb->lang == LANG_RO_COMMABELOW) {
			if (e->state() & Qt::ShiftButton)
				str = QString::fromUtf8("Ț");
			else
				str = QString::fromUtf8("ț");
			done = true;
		}
		else if (key == Qt::Key_T) {
			if (e->state() & Qt::ShiftButton)
				str = QString::fromUtf8("Ţ");
			else
				str = QString::fromUtf8("ţ");
			done = true;
		}
		else if (key == Qt::Key_I) {
			if (e->state() & Qt::ShiftButton)
				str = QString::fromUtf8("Î");
			else
				str = QString::fromUtf8("î");
			done = true;
		}

		if (done) {
			insertAt(str, para, index, false);
			setCursorPosition(para, index + 1);
			return;
		}
	}

	// check aliases
	if (key == Qt::Key_Shift); // this could be a shift key pressed!!!
	else if (key >= 0x80)
		alias_buf_ = "";
	else
		alias_buf_.prepend((char) key);
	alias_buf_.truncate(8);
	LOG("aslias_buf #%s#", (const char *) alias_buf_);

	if (alias_buf_.startsWith(" ")) {
		list<Global::Alias>::iterator it;
		for (it = alist_.begin(); it != alist_.end(); ++it) {
			Global::Alias al = *it;
			QString s = alias_buf_.mid(1);
			if (s.startsWith(al.reverse)) {
				// delete
				setSelection(para, index - al.alias.length(), para, index);
				del();
				// and insert replacement here
				getCursorPosition(&para, &index);
				insertAt(al.replace, para, index, false);
				setCursorPosition(para, index + al.replace.length());
				alias_buf_ = "";
				return;
			}
		}
	}

	// check del operation
	bool del_op = false;
	if (key == Qt::Key_Delete)
		del_op = true;

//	LOG("alias_buf_: %s", (const char *) alias_buf_);
	Q3MultiLineEdit::keyPressEvent(e);
	
	// make sure the bookmarkers are not in the middle of the line for del operations
	if (del_op) {
		getCursorPosition(&para, &index);
		QString txt = textLine(para);
		if (QTUTILS::hasBadBookmarkers(txt)) {
			if (QTUTILS::startsWithBookmarker(txt))
				index = (--index < 0)? 0: index;
			// bookmarker in the middle of the line -> clean it!
			QString newtxt = QTUTILS::cleanBookmarkers(txt);
			// delete current line
			setSelection(para, 0, para, txt.length());
			del();
			// and insert replacement here
			insertAt(newtxt, para, 0, false);
			setCursorPosition(para, index);
		}
	}
}

bool BriefEdit::recordAction() {
	if (recording_ == 0) {
		//clear the list here
		while (!list_.empty())
			list_.erase(list_.begin());
		recording_ = 1;
		return true;
	}

	recording_ = 0;
	return false;
}

void BriefEdit::playbackAction() {
	if (recording_)
		recording_ = 0;

	list<QKeyEvent>::iterator it;
	for (it = list_.begin(); it != list_.end(); ++it) {
		QKeyEvent ev = *it;
		keyPressEvent(&ev);
	}
}

void BriefEdit::reloadAlias() {
	alist_ = Global::instance()->alist;
	LOG("reload alias");
}

Q3PopupMenu *BriefEdit::createPopupMenu(const QPoint& pos) {
	Q3PopupMenu *menu = Q3MultiLineEdit::createPopupMenu( pos );
	int id;
	if (Global::instance()->acc_ == false)
		id = menu->insertItem(tr("Accumulate Clipboard"));
	else
		id = menu->insertItem(tr("Stop Clipboard Accumulation"));
	menu->connectItem(id, this, SLOT(clipboardAccumulate(int)));
	menu->insertSeparator();
	id = menu->insertItem(tr("Shift Left"));
	menu->connectItem(id, this, SLOT(shiftLeftClicked(int)));
	id = menu->insertItem(tr("Shift Right"));
	menu->connectItem(id, this, SLOT(shiftRightClicked(int)));
	id = menu->insertItem(tr("Space to Tab"));
	menu->connectItem(id, this, SLOT(space2tabClicked(int)));
	id = menu->insertItem(tr("Tab to Space"));
	menu->connectItem(id, this, SLOT(tab2spaceClicked(int)));
	id = menu->insertItem(tr("Sort Lines"));
	menu->connectItem(id, this, SLOT(sortLinesClicked(int)));
	menu->insertSeparator();
	id = menu->insertItem(tr("Quick Search"));
	menu->connectItem(id, this, SLOT(quickSearchClicked(int)));
	id = menu->insertItem(tr("Dictionary"));
	menu->connectItem(id, this, SLOT(dictionaryClicked(int)));

	return menu;
}

void BriefEdit::clipboardAccumulate(int id) {
	(void) id;
	if (Global::instance()->acc_ == false) {
		// start with a brand new accumulator
		Global::instance()->accstr_ = "";
		Global::instance()->acclines_ = 0;
	}

	Global::instance()->acc_ = !Global::instance()->acc_;
}

void BriefEdit::dictionaryClicked(int id) {
	(void) id;
	editor_->dexonline();
}	

void BriefEdit::quickSearchClicked(int id) {
	(void) id;
	int para;
	int pos;
	getCursorPosition(&para, &pos);
	// lucian: some code here is duplicated from qwerty.cpp void Editor::cursorPosChanged(int para, int pos) - to fix!

	QString line = textLine(para);
	QChar ch = line.at(pos);
	if (ch.isLetterOrNumber() || ch == '_') {
		// extract word
		int xstart = pos;
		while (xstart >= 0 && (line.at(xstart).isLetterOrNumber() || line.at(xstart) == '_'))
			xstart--;
		if (xstart != 0)
			xstart++;
		
		int xend = pos;
		while (line.at(xend).isLetterOrNumber() || line.at(xend) == '_')
			xend++;

		QString ctagword = line.mid(xstart, xend - xstart);
		if (ctagword.startsWith("\t")) // lucian: some problems with the first letter when is at the beginning of the line! Below is just a hack!
			ctagword = ctagword.mid(1);
		if (ctagword.startsWith("\\"))
			ctagword = ctagword.mid(1);
		// eliminate the bookmark character
		if (QTUTILS::hasBookmarkers(ctagword))
			ctagword = ctagword.mid(1);
			
		if (editor_->advanceSearch(ctagword, false))
			editor_->advanceSearchList(ctagword, false);
	}
}

void BriefEdit::shiftLeftClicked(int id) {
	(void) id;
	int para_from;
	int index_from;
	int para_to;
	int index_to;
	getSelection(&para_from, &index_from, &para_to, &index_to);
	LOG("BriefEdit::shiftLeftClicked:selection %d/%d, %d/%d\n", para_from, index_from, para_to, index_to);
	int ifrom = 0;
	QString txt = textLine(para_from);
	if (QTUTILS::startsWithBookmarker(txt))
		ifrom = 1;

	if (index_from != ifrom || index_to != 0) {
		editor_->msgScreen(full_line_err, 5000);
		return;
	}

	QString newstr = "";
	for (int i = para_from; i < para_to; i++) {
		QString line = textLine(i);
		bool bfound = QTUTILS::startsWithBookmarker(line);
		QChar c = ' ';
		if (bfound) {
			newstr += line.at(0);
			line = line.mid(1);
		}
		const char *ptr = (const char *) line;
		if (*ptr == ' ' || *ptr == '\t')
			newstr += ptr + 1; // delete the first character
		else
			newstr += ptr;
		newstr += '\n';
	}

	// insert function below will end up duplicating the bookmark
	// if the first line in selection  was bookmarked
	// remove the bookmark temporarely
	if (QTUTILS::startsWithBookmarker(newstr))
		newstr = newstr.mid(1);
	// delete the old selection and insert the new text
	insert(newstr);

	// re-select the text
	setSelection(para_from, index_from, para_to, index_to);
	update(); // without update setSelection will not work
}

void BriefEdit::shiftRightClicked(int id) {
	(void) id;
	int para_from;
	int index_from;
	int para_to;
	int index_to;
	getSelection(&para_from, &index_from, &para_to, &index_to);
	LOG("BriefEdit::shiftRightClicked:selection %d/%d, %d/%d\n", para_from, index_from, para_to, index_to);
	int ifrom = 0;
	QString txt = textLine(para_from);
	if (QTUTILS::startsWithBookmarker(txt))
		ifrom = 1;

	if (index_from != ifrom || index_to != 0) {
		editor_->msgScreen(full_line_err, 5000);
		return;
	}

	QString newstr = "";
	for (int i = para_from; i < para_to; i++) {
		QString line = textLine(i);
		bool bfound = QTUTILS::startsWithBookmarker(line);
		QChar c = ' ';
		if (bfound) {
			newstr += line.at(0);
			line = line.mid(1);
		}
		newstr += '\t';
		newstr += line;
		newstr += '\n';
	}

	// insert function below will end up duplicating the bookmark
	// if the first line in selection  was bookmarked
	// remove the bookmark temporarely
	if (QTUTILS::startsWithBookmarker(newstr))
		newstr = newstr.mid(1);
	// delete the old selection and insert the new text
	insert(newstr);

	// re-select the text
	setSelection(para_from, index_from, para_to, index_to);
	update(); // without update setSelection will not work
}

void BriefEdit::space2tabClicked(int id) {
	(void) id;
	int para_from;
	int index_from;
	int para_to;
	int index_to;
	getSelection(&para_from, &index_from, &para_to, &index_to);
//	printf("%d/%d : %d/%d\n", para_from, index_from, para_to, index_to);
	if (index_from != 0 || index_to != 0) {
		editor_->msgScreen(full_line_err, 5000);
		return;
	}

	QString newstr = "";
	for (int i = para_from; i < para_to; i++) {
		QString line = textLine(i);
		const char *ptr = (const char *) line;
		while(*ptr == ' ') {
			newstr += '\t';
			ptr++;
		}
		newstr += ptr;
		newstr += '\n';
	}

	// delete the old selection and insert the new text
	insert(newstr);

	// re-select the text
	setSelection(para_from, index_from, para_to, index_to);
	update(); // without update setSelection will not work
}

void BriefEdit::tab2spaceClicked(int id) {
	(void) id;
	int para_from;
	int index_from;
	int para_to;
	int index_to;
	getSelection(&para_from, &index_from, &para_to, &index_to);
//	printf("%d/%d : %d/%d\n", para_from, index_from, para_to, index_to);
	if (index_from != 0 || index_to != 0) {
		editor_->msgScreen(full_line_err, 5000);
		return;
	}

	QString newstr = "";
	for (int i = para_from; i < para_to; i++) {
		QString line = textLine(i);
		const char *ptr = (const char *) line;
		while(*ptr == '\t') {
			newstr += ' ';
			ptr++;
		}
		newstr += ptr;
		newstr += '\n';
	}

	// delete the old selection and insert the new text
	insert(newstr);

	// re-select the text
	setSelection(para_from, index_from, para_to, index_to);
	update(); // without update setSelection will not work
}

void BriefEdit::sortLinesClicked(int id) {
	(void) id;
	int para_from;
	int index_from;
	int para_to;
	int index_to;
	getSelection(&para_from, &index_from, &para_to, &index_to);
//	printf("%d/%d : %d/%d\n", para_from, index_from, para_to, index_to);
	if (index_from != 0 || index_to != 0) {
		editor_->msgScreen(full_line_err, 5000);
		return;
	}

	// build the list
	list<QString> lst;
	for (int i = para_from; i < para_to; i++) {
		lst.push_back(textLine(i));
	}
	// sort
	lst.sort();
	// put the list back
	QString newstr = "";
	list<QString>::iterator it;
	for (it = lst.begin(); it != lst.end(); ++it) {
		QString s = *it;
		newstr += s + "\n";
	}

	// delete the old selection and insert the new text
	insert(newstr);

	// re-select the text
	setSelection(para_from, index_from, para_to, index_to);
	update(); // without update setSelection will not work
}
