//*********************************************************************
// Copyright (C) 2005,  Lucian Constantin <struct_bylighting@yahoo.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//*********************************************************************

#include "dialogs.h"
#include "global.h"
#include <qmessagebox.h>

#ifdef USING_QT4
#include <Q3VBoxLayout>
#include <q3hbox.h>
#include <q3accel.h>
#include <QLabel>
#include <Q3ButtonGroup>
#include <Q3GroupBox>
#endif
#ifdef USING_QT3
#include <qlayout.h>
#include <qhbox.h>
#include <qaccel.h>
#include <qlabel.h>
#include <qbuttongroup.h>
#include <qgroupbox.h>
#endif

#include <qlineedit.h>
#include <qcheckbox.h>
#include <qpushbutton.h>
#include <qfiledialog.h>
#include <qcombobox.h>
#include <qradiobutton.h>

//*****************************************************************************/
// line dialog
//*****************************************************************************/
LineDialog::LineDialog(): QDialog(0, "Enter line number", true) {
	Q3VBoxLayout *vly = new Q3VBoxLayout(this, 10);
	vly->setAutoAdd(true);

	Q3HBox *hbox = new Q3HBox(this);
	hbox->setSpacing(10);
	label_ = new QLabel(tr("Line: "), hbox);
	text_ = new QLineEdit(hbox);

	// accept/reject
	new QWidget(this);
	Q3HBox *hbox2 = new Q3HBox(this);
	hbox2->setSpacing(10);
	(new QWidget(hbox2))->setMinimumWidth(80);
	(new QWidget(hbox2))->setMinimumWidth(80);
	button_ok_ = new QPushButton(tr("OK"), hbox2);
	button_cancel_ = new QPushButton(tr("Cancel"), hbox2);

	connect(button_ok_, SIGNAL(clicked()), SLOT(accept()));
	connect(button_cancel_, SIGNAL(clicked()), SLOT(reject()));
}

int LineDialog::getLine() {
	int retval;

	if (text_->text().isEmpty() == true)
		retval = -1;
	else {
		QString str = text_->text();
		bool ok;
		retval = str.toInt(&ok);
		if (!ok || retval <= 0) {
			retval = -1;
			QMessageBox::warning(0,tr("Error"), tr("Not a valid line number"));
		}
	}
	return retval;
}

//*****************************************************************************/
// bookmark dialog
//*****************************************************************************/
BookmarkDialog::BookmarkDialog(): QDialog(0, "Enter bookmark number", true) {
	Q3VBoxLayout *vly = new Q3VBoxLayout(this, 10);
	vly->setAutoAdd(true);

	Q3HBox *hbox = new Q3HBox(this);
	hbox->setSpacing(10);
	label_ = new QLabel(tr("Bookmark: "), hbox);
	text_ = new QLineEdit(hbox);

	// accept/reject
	new QWidget(this);
	Q3HBox *hbox2 = new Q3HBox(this);
	hbox2->setSpacing(10);
	(new QWidget(hbox2))->setMinimumWidth(80);
	(new QWidget(hbox2))->setMinimumWidth(80);
	button_ok_ = new QPushButton(tr("OK"), hbox2);
	button_cancel_ = new QPushButton(tr("Cancel"), hbox2);

	connect(button_ok_, SIGNAL(clicked()), SLOT(accept()));
	connect(button_cancel_, SIGNAL(clicked()), SLOT(reject()));
}

int BookmarkDialog::getBookmark() {
	int retval;

	if (text_->text().isEmpty() == true)
		retval = -1;
	else {
		QString str = text_->text();
		bool ok;
		retval = str.toInt(&ok);
		if (!ok || retval < 0 || retval >= MAXBOOKMARKS) {
			retval = -1;
			QMessageBox::warning(0,tr("Error"), tr("Not a valid bookmark number"));
		}
	}
	return retval;
}

//*****************************************************************************/
// bookmark dialog
//*****************************************************************************/
HexDialog::HexDialog(): QDialog(0, "Enter Unicode Char", true) {
	Q3VBoxLayout *vly = new Q3VBoxLayout(this, 10);
	vly->setAutoAdd(true);

	Q3HBox *hbox = new Q3HBox(this);
	hbox->setSpacing(10);
	label_ = new QLabel(tr("Unicode character code: "), hbox);
	text_ = new QLineEdit(hbox);

	new QLabel(tr("(i.e. U0218 or 218)"), this);

	// accept/reject
	new QWidget(this);
	Q3HBox *hbox2 = new Q3HBox(this);
	hbox2->setSpacing(10);
	(new QWidget(hbox2))->setMinimumWidth(80);
	(new QWidget(hbox2))->setMinimumWidth(80);
	button_ok_ = new QPushButton(tr("OK"), hbox2);
	button_cancel_ = new QPushButton(tr("Cancel"), hbox2);

	connect(button_ok_, SIGNAL(clicked()), SLOT(accept()));
	connect(button_cancel_, SIGNAL(clicked()), SLOT(reject()));
}

int HexDialog::getHex() {
	int retval;

	if (text_->text().isEmpty() == true)
		retval = -1;
	else {
		QString str = text_->text();
		if (str.startsWith("U") || str.startsWith("u"))
			str = str.mid(1);
		bool ok;
		retval = str.toInt(&ok, 16);
		if (!ok) {
			retval = -1;
			QMessageBox::warning(0,tr("Error"), tr("Not a valid Unicode code"));
		}
	}
	return retval;
}


//*****************************************************************************/
// www dialog
//*****************************************************************************/
WwwDialog::WwwDialog(): QDialog(0, "www location", true) {
	Q3VBoxLayout *vly = new Q3VBoxLayout(this, 10);
	vly->setAutoAdd(true);

	Q3HBox *hbox1 = new Q3HBox(this);
	hbox1->setSpacing(10);
	label_ = new QLabel(tr("Please enter the URL of a file you want to download\n(i.e. http://rospell.sourceforge.net). You might want to set\nyour proxy in Preferences/HTTP Proxy dialog."), hbox1);

	Q3HBox *hbox = new Q3HBox(this);
	hbox->setSpacing(10);
	text_ = new QLineEdit(hbox);

	Q3HBox *hbox2 = new Q3HBox(this);
	(new QWidget(hbox2))->setMinimumWidth(80);
	(new QWidget(hbox2))->setMinimumWidth(80);
	button_ = new QPushButton(tr("OK"), hbox2);

	connect(button_, SIGNAL(clicked()), SLOT(accept()));
}

QString WwwDialog::getWord() {
	QString retval;

	if (text_->text().isEmpty() == true)
		retval = "";
	else {
		retval = text_->text();
	}
	return retval;
}

//*****************************************************************************/
// status dialog
//*****************************************************************************/
StatusDialog::StatusDialog(QFont &fnt, char *msg): QDialog(0, "File status", true) {
	ASSERT(msg != 0);
	Q3VBoxLayout *vly = new Q3VBoxLayout(this, 10);
	vly->setAutoAdd(true);

	Q3HBox *hbox = new Q3HBox(this);

	QString qstr = QString::fromUtf8(msg);

	label_ = new QLabel(qstr, hbox);
	label_->setFont(fnt);

	new QWidget(this);
	Q3HBox *hbox1 = new Q3HBox(this);
	new QWidget(hbox1);
	new QWidget(hbox1);
	button_ = new QPushButton(tr("OK"), hbox1);
	new QWidget(hbox1);

	connect(button_, SIGNAL(clicked()), SLOT(accept()));
}



//*****************************************************************************/
// basic unicode dialog
//*****************************************************************************/
BasicUnicodeDialog::BasicUnicodeDialog(QString title): QDialog(0, title, true) {
	text_ = 0;

	Q3Accel *a = new Q3Accel(this);		  // create accels for myWindow
	a->connectItem(a->insertItem(Qt::ALT+Qt::Key_A), this, SLOT(unicode_a()));
	a->connectItem(a->insertItem(Qt::SHIFT+Qt::ALT+Qt::Key_A), this, SLOT(unicode_A()));
	a->connectItem(a->insertItem(Qt::ALT+Qt::Key_S), this, SLOT(unicode_s()));
	a->connectItem(a->insertItem(Qt::SHIFT+Qt::ALT+Qt::Key_S), this, SLOT(unicode_S()));
	a->connectItem(a->insertItem(Qt::ALT+Qt::Key_Q), this, SLOT(unicode_q()));
	a->connectItem(a->insertItem(Qt::SHIFT+Qt::ALT+Qt::Key_Q), this, SLOT(unicode_Q()));
	a->connectItem(a->insertItem(Qt::ALT+Qt::Key_T), this, SLOT(unicode_t()));
	a->connectItem(a->insertItem(Qt::SHIFT+Qt::ALT+Qt::Key_T), this, SLOT(unicode_T()));
	a->connectItem(a->insertItem(Qt::ALT+Qt::Key_I), this, SLOT(unicode_i()));
	a->connectItem(a->insertItem(Qt::SHIFT+Qt::ALT+Qt::Key_I), this, SLOT(unicode_I()));
}

QString BasicUnicodeDialog::getWord() {
	if (text_ == 0)
		return "";

	QString retval;

	if (text_->text().isEmpty() == true)
		retval = "";
	else {
		retval = text_->text();
	}
	return retval;
}


void BasicUnicodeDialog::unicode_a() {
	if (!hasFocus())
		return;
	if (text_ == 0)
		return;

	Global *glb = Global::instance();
	QString str;
	if (glb->lang == LANG_RO_COMMABELOW)
		str = QString::fromUtf8("ă");
	else if (glb->lang == LANG_RO_CEDILLA)
		str = QString::fromUtf8("ă");
	else
		return;
	text_->insert(str);
	text_->cursorForward(false);
}

void BasicUnicodeDialog::unicode_A() {
	if (!hasFocus())
		return;
	if (text_ == 0)
		return;
	Global *glb = Global::instance();
	QString str;
	if (glb->lang == LANG_RO_COMMABELOW)
		str = QString::fromUtf8("Ă");
	else if (glb->lang == LANG_RO_CEDILLA)
		str = QString::fromUtf8("Ă");
	else
		return;
	text_->insert(str);
	text_->cursorForward(false);
}

void BasicUnicodeDialog::unicode_s() {
	if (!hasFocus())
		return;
	if (text_ == 0)
		return;
	Global *glb = Global::instance();
	QString str;
	if (glb->lang == LANG_RO_COMMABELOW)
		str = QString::fromUtf8("ș");
	else if (glb->lang == LANG_RO_CEDILLA)
		str = QString::fromUtf8("ş");
	else
		return;
	text_->insert(str);
	text_->cursorForward(false);
}

void BasicUnicodeDialog::unicode_S() {
	if (!hasFocus())
		return;
	if (text_ == 0)
		return;
	Global *glb = Global::instance();
	QString str;
	if (glb->lang == LANG_RO_COMMABELOW)
		str = QString::fromUtf8("Ș");
	else if (glb->lang == LANG_RO_CEDILLA)
		str = QString::fromUtf8("Ş");
	else
		return;
	text_->insert(str);
	text_->cursorForward(false);
}

void BasicUnicodeDialog::unicode_q() {
	if (!hasFocus())
		return;
	if (text_ == 0)
		return;
	Global *glb = Global::instance();
	QString str;
	if (glb->lang == LANG_RO_COMMABELOW)
		str = QString::fromUtf8("â");
	else if (glb->lang == LANG_RO_CEDILLA)
		str = QString::fromUtf8("â");
	else
		return;
	text_->insert(str);
	text_->cursorForward(false);
}

void BasicUnicodeDialog::unicode_Q() {
	if (!hasFocus())
		return;
	if (text_ == 0)
		return;
	Global *glb = Global::instance();
	QString str;
	if (glb->lang == LANG_RO_COMMABELOW)
		str = QString::fromUtf8("Â");
	else if (glb->lang == LANG_RO_CEDILLA)
		str = QString::fromUtf8("Â");
	else
		return;
	text_->insert(str);
	text_->cursorForward(false);
}

void BasicUnicodeDialog::unicode_t() {
	if (!hasFocus())
		return;
	if (text_ == 0)
		return;
	Global *glb = Global::instance();
	QString str;
	if (glb->lang == LANG_RO_COMMABELOW)
		str = QString::fromUtf8("ț");
	else if (glb->lang == LANG_RO_CEDILLA)
		str = QString::fromUtf8("ţ");
	else
		return;
	text_->insert(str);
	text_->cursorForward(false);
}

void BasicUnicodeDialog::unicode_T() {
	if (!hasFocus())
		return;
	if (text_ == 0)
		return;
	Global *glb = Global::instance();
	QString str;
	if (glb->lang == LANG_RO_COMMABELOW)
		str = QString::fromUtf8("Ț");
	else if (glb->lang == LANG_RO_CEDILLA)
		str = QString::fromUtf8("Ţ");
	else
		return;
	text_->insert(str);
	text_->cursorForward(false);
}

void BasicUnicodeDialog::unicode_i() {
	if (!hasFocus())
		return;
	if (text_ == 0)
		return;
	Global *glb = Global::instance();
	QString str;
	if (glb->lang == LANG_RO_COMMABELOW)
		str = QString::fromUtf8("î");
	else if (glb->lang == LANG_RO_CEDILLA)
		str = QString::fromUtf8("î");
	else
		return;
	text_->insert(str);
	text_->cursorForward(false);
}

void BasicUnicodeDialog::unicode_I() {
	if (!hasFocus())
		return;
	if (text_ == 0)
		return;
	Global *glb = Global::instance();
	QString str;
	if (glb->lang == LANG_RO_COMMABELOW)
		str = QString::fromUtf8("Î");
	else if (glb->lang == LANG_RO_CEDILLA)
		str = QString::fromUtf8("Î");
	else
		return;
	text_->insert(str);
	text_->cursorForward(false);
}



//*****************************************************************************/
// basic unicode combo dialog
//*****************************************************************************/
BasicUnicodeComboDialog::BasicUnicodeComboDialog(QString title): BasicUnicodeDialog(title) {
	combotext_ = 0;

}

QString BasicUnicodeComboDialog::getWord() {
	if (combotext_ == 0)
		return "";

	QString retval;

	if (combotext_->currentText().isEmpty() == true)
		retval = "";
	else {
		retval = combotext_->currentText();
	}
	return retval;
}







//*****************************************************************************/
// search dialog
//*****************************************************************************/
SearchDialog::SearchDialog(): BasicUnicodeComboDialog("Enter search word") {
	Q3VBoxLayout *vly = new Q3VBoxLayout(this, 10);
	vly->setAutoAdd(true);

	// target word
	Q3HBox *hbox1 = new Q3HBox(this);
	hbox1->setSpacing(10);
	label_ = new QLabel(tr("Target: "), hbox1);
	text_ = new QLineEdit(hbox1);
	combotext_ = new QComboBox(TRUE, hbox1);
	combotext_->setLineEdit(text_);


#ifdef USING_QT3
            Q3ButtonGroup *bgrp2 = new Q3ButtonGroup( 1, Q3GroupBox::Horizontal, tr("Options"), this );
#else
            Q3ButtonGroup *bgrp2 = new Q3ButtonGroup( 1, Qt::Horizontal, "Options", this );
#endif
	ignore_case_ = new QCheckBox(tr("Ignore case"), bgrp2);
	whole_ = new QCheckBox(tr("Whole words"), bgrp2);
	backwards_ = new QCheckBox(tr("Search Backwards"), bgrp2);
	// accept/reject
	new QWidget(this);
	Q3HBox *hbox2 = new Q3HBox(this);
	hbox2->setSpacing(10);
	(new QWidget(hbox2))->setMinimumWidth(80);
	(new QWidget(hbox2))->setMinimumWidth(80);
//	new QWidget(hbox2);
	button_ok_ = new QPushButton(tr("OK"), hbox2);
	button_cancel_ = new QPushButton(tr("Cancel"), hbox2);


	connect(button_ok_, SIGNAL(clicked()), SLOT(clicked_ok()));
	connect(button_cancel_, SIGNAL(clicked()), SLOT(reject()));
}

bool SearchDialog::getIgnoreCase() {
	return ignore_case_->isChecked();
}

bool SearchDialog::getWhole() {
	return whole_->isChecked();
}

bool SearchDialog::getBackwards() {
	return backwards_->isChecked();
}


void SearchDialog::clicked_ok() {
	QString word = getWord();
	// check if the word is not already in the list
	bool found = false;
	for (int i = 0; i < combotext_->count(); i++) {
		if (combotext_->text(i) == word) {
			found = true;
			break;
		}
	}
	if (!found)
		combotext_->insertItem(word, 0);

	accept();
}

//*****************************************************************************/
// grep dialog
//*****************************************************************************/
GrepDialog::GrepDialog(): BasicUnicodeComboDialog("Enter search word") {
	Q3VBoxLayout *vly = new Q3VBoxLayout(this, 10);
	vly->setAutoAdd(true);

	// target word
	Q3HBox *hbox1 = new Q3HBox(this);
	hbox1->setSpacing(10);
	label_ = new QLabel(tr("Target: "), hbox1);
	text_ = new QLineEdit(hbox1);
	combotext_ = new QComboBox(TRUE, hbox1);
	combotext_->setLineEdit(text_);


	Q3HBox *hbox11 = new Q3HBox(this);
	hbox11->setSpacing(10);
	label2_ = new QLabel(tr("Starting directory: "), hbox11);
	sdirectory_ = new QLineEdit(hbox11);
	button_dir_ = new QPushButton("...", hbox11);
	sdirectory_->setText("./");

	Q3HBox *hbox12 = new Q3HBox(this);
	hbox12->setSpacing(10);
	label3_ = new QLabel(tr("File pattern: "), hbox12);
	fpattern_ = new QLineEdit(hbox12);
	fpattern_->setText("*.*");

	Q3HBox *hbox13 = new Q3HBox(this);
	hbox13->setSpacing(10);
	label4_ = new QLabel(tr("Exclude directories: "), hbox13);
	edirectory_ = new QLineEdit(hbox13);
	edirectory_->setText("");

#ifdef USING_QT3
	Q3ButtonGroup *bgrp2 = new Q3ButtonGroup( 1, Q3GroupBox::Horizontal, tr("Options"), this );
#else
	Q3ButtonGroup *bgrp2 = new Q3ButtonGroup( 1, Qt::Horizontal, "Options", this );
#endif
	ignore_case_ = new QCheckBox(tr("Ignore case"), bgrp2);
	whole_ = new QCheckBox(tr("Whole words"), bgrp2);
	recursive_ = new QCheckBox(tr("Recursive"), bgrp2);

	// accept/reject
	new QWidget(this);
	Q3HBox *hbox2 = new Q3HBox(this);
	hbox2->setSpacing(10);
	(new QWidget(hbox2))->setMinimumWidth(80);
	(new QWidget(hbox2))->setMinimumWidth(80);
	button_ok_ = new QPushButton(tr("OK"), hbox2);
	button_cancel_ = new QPushButton(tr("Cancel"), hbox2);


//	connect(button_ok_, SIGNAL(clicked()), SLOT(accept()));
	connect(button_ok_, SIGNAL(clicked()), SLOT(clicked_ok()));
	connect(button_cancel_, SIGNAL(clicked()), SLOT(reject()));
	connect(button_dir_, SIGNAL(clicked()), this, SLOT(clicked_dir()));
}

bool GrepDialog::getIgnoreCase() {
	return ignore_case_->isChecked();
}

bool GrepDialog::getWhole() {
	return whole_->isChecked();
}

bool GrepDialog::getRecursive() {
	return recursive_->isChecked();
}

QString GrepDialog::getSdirectory() {
	return sdirectory_->text();
}

QString GrepDialog::getFpattern() {
	return fpattern_->text();
}

QString GrepDialog::getEdirectory() {
	return edirectory_->text();
}

void GrepDialog::clicked_dir() {
	QString s = QFileDialog::getExistingDirectory("./", this, "get existing directory", tr("Choose a directory"), TRUE );
	if (s.isNull() || s.isEmpty()) {
		return;
	}
	sdirectory_->setText(s);
}

void GrepDialog::clicked_ok() {
	QString word = getWord();
	// check if the word is not already in the list
	bool found = false;
	for (int i = 0; i < combotext_->count(); i++) {
		if (combotext_->text(i) == word) {
			found = true;
			break;
		}
	}
	if (!found)
		combotext_->insertItem(word, 0);

	accept();
}

//*****************************************************************************/
// replace dialog
//*****************************************************************************/
ReplaceDialog::ReplaceDialog(QString tofind, QString replacement):
	BasicUnicodeComboDialog("Replace") {

	Q3VBoxLayout *vly = new Q3VBoxLayout(this, 10);
	vly->setAutoAdd(true);

	// target word
	Q3HBox *hbox1 = new Q3HBox(this);
	hbox1->setSpacing(10);
	label1_ = new QLabel(tr("Target: "), hbox1);
	text1_ = new QLineEdit(hbox1);
	text1_->setText(tofind);
	combotext1_ = new QComboBox(TRUE, hbox1);
	combotext1_->setLineEdit(text1_);

	Q3HBox *hbox2 = new Q3HBox(this);
	hbox2->setSpacing(10);
	label2_ = new QLabel(tr("Replacement: "), hbox2);
	text2_ = new QLineEdit(hbox2);
	text2_->setText(replacement);
	combotext2_ = new QComboBox(TRUE, hbox2);
	combotext2_->setLineEdit(text2_);

	// accept/reject
	new QWidget(this);
	Q3HBox *hbox3 = new Q3HBox(this);
	hbox3->setSpacing(10);
	button_findnext_ = new QPushButton(tr("Find Next"), hbox3);
	button_replace_ = new QPushButton(tr("Replace"), hbox3);
	button_replaceall_ = new QPushButton(tr("Replace All"), hbox3);
	new QWidget(hbox3);
	button_cancel_ = new QPushButton(tr("Cancel"), hbox3);


	connect(button_findnext_, SIGNAL(clicked()), this, SLOT(clicked_findnext()));
	connect(button_replace_, SIGNAL(clicked()), this, SLOT(clicked_replace()));
	connect(button_replaceall_, SIGNAL(clicked()), this, SLOT(clicked_replaceall()));
	connect(button_cancel_, SIGNAL(clicked()), SLOT(clicked_cancel()));
//	connect(button_cancel_, SIGNAL(clicked()), SLOT(reject()));

	text_ = 0;
	combotext_ = 0;
	action_ = ACTION_NONE;
}

bool ReplaceDialog::hasFocus() {
	if (text1_->hasFocus()) {
		text_ = text1_;
		combotext_ = combotext1_;
	}
	else if (text2_->hasFocus()) {
		text_ = text2_;
		combotext_ = combotext2_;
	}
	else {
		text_ = 0;
		combotext_ = 0;
	}

	return true;
}

void ReplaceDialog::printAction() {
	if (action_ == ACTION_FINDNEXT)
		LOG("ReplaceDialog::ACTION_FINDNEXT");
	else if (action_ == ACTION_REPLACE)
		LOG("ReplaceDialog::ACTION_REPLACE");
	else if (action_ == ACTION_REPLACEALL)
		LOG("ReplaceDialog::ACTION_REPLACEALL");
	else if (action_ == ACTION_CANCEL)
		LOG("ReplaceDialog::ACTION_CANCEL");
	else
		LOG("ReplaceDialog::ACTION_NONE");

}
void ReplaceDialog::clicked_findnext() {
	action_ = ACTION_FINDNEXT;
}

void ReplaceDialog::clicked_replace() {
	action_ = ACTION_REPLACE;
}

void ReplaceDialog::clicked_replaceall() {
	action_ = ACTION_REPLACEALL;
}

void ReplaceDialog::clicked_cancel() {
	action_ = ACTION_CANCEL;
}

void ReplaceDialog::getOut() {

	QString str = getWordToFind();
	// check if the word is not already in the list
	bool found = false;
	for (int i = 0; i < combotext1_->count(); i++) {
		if (combotext1_->text(i) == str) {
			found = true;
			break;
		}
	}
	if (!found)
		combotext1_->insertItem(str, 0);

	str = getReplacement();
	found = false;
	for (int i = 0; i < combotext2_->count(); i++) {
		if (combotext2_->text(i) == str) {
			found = true;
			break;
		}
	}
	if (!found)
		combotext2_->insertItem(str, 0);


	text_ = 0;
	combotext_ = 0;
	action_ = ACTION_NONE;
	accept();
}

void ReplaceDialog::closeEvent(QCloseEvent *evt) {
	(void) evt;
	action_ = ACTION_CANCEL;
}

QString ReplaceDialog::getWordToFind() {
	if (text1_ == 0)
		return "";

	QString retval;

	if (text1_->text().isEmpty() == true)
		retval = "";
	else {
		retval = text1_->text();
	}

	return retval;
}

QString ReplaceDialog::getReplacement() {
	if (text2_ == 0)
		return "";

	QString retval;

	if (text2_->text().isEmpty() == true)
		retval = "";
	else {
		retval = text2_->text();
	}

	return retval;
}


//*****************************************************************************/
// file opetions dialog
//*****************************************************************************/
FileOptionsDialog::FileOptionsDialog(): QDialog(0, "File options dialog", true) {
	Q3VBoxLayout *vly = new Q3VBoxLayout(this, 10);
	vly->setAutoAdd(true);

#ifdef USING_QT3
	Q3ButtonGroup *bgrp1 = new Q3ButtonGroup( 1, Q3GroupBox::Horizontal, tr("Backup: "), this );
#else
	Q3ButtonGroup *bgrp1 = new Q3ButtonGroup( 1, Qt::Horizontal, tr("Backup: "), this );
#endif
	backup_ = new QCheckBox(tr("backup"), bgrp1);
	autosave_ = new QCheckBox(tr("autosave"), bgrp1);
	backup_->setChecked(!Global::instance()->no_backup_);
	autosave_->setChecked(!Global::instance()->no_autosave_);

#ifdef USING_QT3
	Q3ButtonGroup *bgrp2 = new Q3ButtonGroup( 1, Q3GroupBox::Horizontal, tr("On save: "), this );
#else
	Q3ButtonGroup *bgrp2 = new Q3ButtonGroup( 1, Qt::Horizontal, tr("On save: "), this );
#endif
	strip_trailing_ = new QCheckBox(tr("strip EOL trailing spaces"), bgrp2);
	unix_eol_ = new QCheckBox(tr("use UNIX EOL"), bgrp2);
	tab2space_ = new QCheckBox(tr("TAB to SPACE"), bgrp2);
	strip_trailing_->setChecked(Global::instance()->strip_trailing_);
	unix_eol_->setChecked(Global::instance()->unix_eol_);
	tab2space_->setChecked(Global::instance()->tab2space_);



	// accept/reject
	new QWidget(this);
	Q3HBox *hbox2 = new Q3HBox(this);
	hbox2->setSpacing(10);
	(new QWidget(hbox2))->setMinimumWidth(80);
	(new QWidget(hbox2))->setMinimumWidth(80);
	button_ok_ = new QPushButton(tr("OK"), hbox2);
	button_cancel_ = new QPushButton(tr("Cancel"), hbox2);

	connect(button_ok_, SIGNAL(clicked()), SLOT(clicked_ok()));
	connect(button_cancel_, SIGNAL(clicked()), SLOT(reject()));
}

void FileOptionsDialog::clicked_ok() {
	if (backup_->isChecked())
		Global::instance()->no_backup_ = false;
	else
		Global::instance()->no_backup_ = true;

	if (autosave_->isChecked())
		Global::instance()->no_autosave_ = false;
	else
		Global::instance()->no_autosave_ = true;

	Global::instance()->strip_trailing_ = strip_trailing_->isChecked();
	Global::instance()->unix_eol_ = unix_eol_->isChecked();
	Global::instance()->tab2space_ = tab2space_->isChecked();

	accept();
}

//*****************************************************************************/
// file opetions dialog
//*****************************************************************************/
ChangeFormatDialog::ChangeFormatDialog(): QDialog(0, "Change format dialog", true) {
	Q3VBoxLayout *vly = new Q3VBoxLayout(this, 10);
	vly->setAutoAdd(true);

	// target word
	Q3HBox *hbox1 = new Q3HBox(this);
	hbox1->setSpacing(10);
	label_ = new QLabel(tr("Format: "), hbox1);
	lformat_ = new QLineEdit(hbox1);
//	lformat_->setText(t1);
	cformat_ = new QComboBox(TRUE, hbox1);
	cformat_->setLineEdit(lformat_);

	QStringList lst;
	ff_populateList(lst);
	cformat_->insertStringList(lst);

#ifdef USING_QT3
	Q3ButtonGroup *bgrp1 = new Q3ButtonGroup( 1, Q3GroupBox::Horizontal, tr("Apply to: "), this );
#else
	Q3ButtonGroup *bgrp1 = new Q3ButtonGroup( 1, Qt::Horizontal, tr("Apply to: "), this );
#endif
	global_format_ = new QRadioButton(tr("new files/buffers only"), bgrp1);
	current_format_ = new QRadioButton(tr("current file"), bgrp1);
	global_format_->setChecked(true);
	bgrp1->setRadioButtonExclusive(true);

	// accept/reject
	new QWidget(this);
	Q3HBox *hbox2 = new Q3HBox(this);
	hbox2->setSpacing(10);
	(new QWidget(hbox2))->setMinimumWidth(80);
	(new QWidget(hbox2))->setMinimumWidth(80);
	button_ok_ = new QPushButton(tr("OK"), hbox2);
	button_cancel_ = new QPushButton(tr("Cancel"), hbox2);
	connect(button_ok_, SIGNAL(clicked()), SLOT(accept()));
	connect(button_cancel_, SIGNAL(clicked()), SLOT(reject()));
}

bool ChangeFormatDialog::getChangeGlobal() {
	return global_format_->isChecked();
}

QString ChangeFormatDialog::getFormatString() {
	return lformat_->text();
}

CustomizeDialog::CustomizeDialog(): QDialog(0, "Customize", true) {
	Q3VBoxLayout *vly = new Q3VBoxLayout(this, 10, 10);
	vly->setAutoAdd(true);

	// generate local tags
#ifdef USING_QT3
	Q3ButtonGroup *bgrp1 = new Q3ButtonGroup( 1, Q3GroupBox::Horizontal, tr("View: "), this );
#else
	Q3ButtonGroup *bgrp1 = new Q3ButtonGroup( 1, Qt::Horizontal, tr("View: "), this );
#endif

	toolbar_ = new QCheckBox(tr("Toolbar"), bgrp1);
	if (Global::instance()->toolbar_) {
#ifdef USING_QT4
		toolbar_->setOn(true);
#else
		toolbar_->setChecked(true);
#endif
	}

	// accept/reject
	new QWidget(this);
	Q3HBox *hbox2 = new Q3HBox(this);
	hbox2->setSpacing(10);
	(new QWidget(hbox2))->setMinimumWidth(80);
	(new QWidget(hbox2))->setMinimumWidth(80);
	button_ok_ = new QPushButton(tr("OK"), hbox2);
	button_cancel_ = new QPushButton(tr("Cancel"), hbox2);

	connect(button_ok_, SIGNAL(clicked()), SLOT(accept()));
	connect(button_cancel_, SIGNAL(clicked()), SLOT(reject()));
}

bool CustomizeDialog::getToolbar() {
	return toolbar_->isChecked();
}
