//*********************************************************************
// Copyright (C) 2005,  Lucian Constantin <struct_bylighting@yahoo.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//*********************************************************************

#ifndef DICTIONARIES_H
#define DICTIONARIES_H

#include "hashdb.h"

class Editor;

typedef struct {
	const char *name; // language name
	const char *dict; // dictionary file name
} LangDef;
#define MAX_LANG_DEF 6

QString getLangName(int id);
QString getLangDict(int id);
int getLangId(QString name);

class Dictionaries: public QObject {
public:
	Dictionaries(Editor *e, bool readmaster = true, bool localread = true, bool localsave = true);
	~Dictionaries();

	// direct dictionary access
	DbDict *getMaster() {
		return hptr_;
	}
	DbUser *getIgnore() {
		return iptr_;
	}
	DbUser *getUser() {
		return uptr_;
	}

	// look for a word in all dictionaries
	bool findString(const char *active_word);

	//*******************************************************************
	// word suggestions
	//*******************************************************************
	// suggestion by flipping letters in the active word
	// Note: does not support flipping for utf8 chars yet	
	bool suggestFlip(const char *active_word, char *dest1, char *dest2, char *dest3);
	
	// suggestion by removing letters from the word
	bool suggestRemoveLetter(const char *active_word, char *dest1, char *dest2, char *dest3);
	
	// suggestion by inserting letters from the trylist
	bool suggestInsertTrylist(const char *active_word, char *dest1, char *dest2, char *dest3);

	// suggestion by replacing with letters from the trylist
	bool suggestReplaceTrylist(const char *active_word, char *dest1, char *dest2, char *dest3);

	// suggestion by replacing with letters from the trylist
	bool suggestAddEndTrylist(const char *active_word, char *dest1, char *dest2, char *dest3);

private:
	// read master dictionary
	void readMaster();
	// read user dictionary
	void readUser();
private:
	Editor *e_;
	DbDict *hptr_;	// master dictionary
	DbUser *iptr_;	// ignore dictionary
	DbUser *uptr_;	// user dictionary
	bool readmaster_; // read master when the dictionary is created
	bool localread_; 	  // read local dict when the dictionary is created
	bool localsave_;   // save local dict when the dictionary is deleted
	char *trylist_;
};

#endif
