//*********************************************************************
// Copyright (C) 2005,  Lucian Constantin <struct_bylighting@yahoo.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//*********************************************************************

#include "diffdialog.h"
#include "busycursor.h"
#include "progressbar.h"
#include <qmessagebox.h>
#include <qgroupbox.h>
#include <qsyntaxhighlighter.h>

#ifdef USING_QT3
#include <qfiledialog.h>
#endif
#ifdef USING_QT4
#include <q3filedialog.h>
#endif


#ifdef USING_QT3
#include <qlayout.h>
#include <qlabel.h>
#include <qhbox.h>
#include <qaccel.h>
#include <qmultilineedit.h>
#endif
#ifdef USING_QT4
#include <Q3VBoxLayout>
#include <QLabel>
#include <q3hbox.h>
#include <q3accel.h>
#include <q3multilineedit.h>
#endif

#include <qlineedit.h>
#include <qcheckbox.h>
#include <qpushbutton.h>
#include <qtextcodec.h>

extern QString utf82latex(QString txt);

//*****************************************************************************/
// dir diff dialog
//*****************************************************************************/
DirDiffDialog::DirDiffDialog(): QDialog(0, "dir", true) {
#ifndef ROSPELL_NO_DIFF

	Q3VBoxLayout *vly = new Q3VBoxLayout(this, 10);
	vly->setAutoAdd(true);


	// file1
	Q3HBox *hbox1 = new Q3HBox(this);
	hbox1->setSpacing(10);
	label1_ = new QLabel(tr("Dir 1: "), hbox1);
	file1_ = new QLineEdit(hbox1);
	button_browse1_ = new QPushButton(tr("browse"), hbox1);

	// file2
	Q3HBox *hbox2 = new Q3HBox(this);
	hbox2->setSpacing(10);
	label2_ = new QLabel(tr("Dir 2: "), hbox2);
	file2_ = new QLineEdit(hbox2);
	button_browse2_ = new QPushButton(tr("browse"), hbox2);

	Q3HBox *hbox21 = new Q3HBox(this);
	hbox21->setSpacing(10);
	new QLabel(tr("Use Dir 2 as the directory merge result"), hbox21);


	Q3HBox *hbox12 = new Q3HBox(this);
	hbox12->setSpacing(10);
	new QLabel(tr("File pattern: "), hbox12);
	fpattern_ = new QLineEdit(hbox12);
	fpattern_->setText("*");

	Q3HBox *hbox13 = new Q3HBox(this);
	hbox13->setSpacing(10);
	new QLabel(tr("Exclude directories: "), hbox13);
	edirectory_ = new QLineEdit(hbox13);
	edirectory_->setText("");



	// accept/reject
	new QWidget(this);
	Q3HBox *hbox3 = new Q3HBox(this);
	hbox3->setSpacing(10);
	// some space
	(new QWidget(hbox3))->setMinimumWidth(80);
	(new QWidget(hbox3))->setMinimumWidth(80);
	button_ok_ = new QPushButton(tr("OK"), hbox3);
	button_cancel_ = new QPushButton(tr("Cancel"), hbox3);

	connect(button_ok_, SIGNAL(clicked()), SLOT(accept()));
	connect(button_cancel_, SIGNAL(clicked()), SLOT(reject()));
	connect(button_browse1_, SIGNAL(clicked()), this, SLOT(clicked_browse1()));
	connect(button_browse2_, SIGNAL(clicked()), this, SLOT(clicked_browse2()));
#endif
}

QString DirDiffDialog::getFpattern() {
	return fpattern_->text();
}

QString DirDiffDialog::getEdirectory() {
	return edirectory_->text();
}
QString DirDiffDialog::getFile1() {
	if (file1_ == 0)
		return "";

	QString retval;

	if (file1_->text().isEmpty() == true)
		retval = "";
	else {
		retval = file1_->text();
	}
	return retval;
}

QString DirDiffDialog::getFile2() {
	if (file2_ == 0)
		return "";

	QString retval;

	if (file2_->text().isEmpty() == true)
		retval = "";
	else {
		retval = file2_->text();
	}
	return retval;
}

void DirDiffDialog::clicked_browse1() {
#ifndef ROSPELL_NO_DIFF
	QString file = Q3FileDialog::getExistingDirectory(Global::instance()->diff_dir1, this, QString::null, QString::null, true );
	if (!file.isNull()) {
		QFileInfo info(file);
		Global::instance()->diff_dir1 = info.absFilePath();
	}
	file1_->setText(file);
#endif
}

void DirDiffDialog::clicked_browse2() {
#ifndef ROSPELL_NO_DIFF
	QString file = Q3FileDialog::getExistingDirectory(Global::instance()->diff_dir2, this, QString::null, QString::null, true );
	if (!file.isNull()) {
		QFileInfo info(file);
		Global::instance()->diff_dir2 = info.absFilePath();
	}
	file2_->setText(file);
#endif
}


//*****************************************************************************/
// diff dialog
//*****************************************************************************/
DiffDialog::DiffDialog(): QDialog(0, "diff", true) {
#ifndef ROSPELL_NO_DIFF

	Q3VBoxLayout *vly = new Q3VBoxLayout(this, 10);
	vly->setAutoAdd(true);


	// file1
	Q3HBox *hbox1 = new Q3HBox(this);
	hbox1->setSpacing(10);
	label1_ = new QLabel(tr("File 1: "), hbox1);
	file1_ = new QLineEdit(hbox1);
	button_browse1_ = new QPushButton(tr("browse"), hbox1);

	// file2
	Q3HBox *hbox2 = new Q3HBox(this);
	hbox2->setSpacing(10);
	label2_ = new QLabel(tr("File 2: "), hbox2);
	file2_ = new QLineEdit(hbox2);
	button_browse2_ = new QPushButton(tr("browse"), hbox2);

	// accept/reject
	new QWidget(this);
	Q3HBox *hbox3 = new Q3HBox(this);
	hbox3->setSpacing(10);
	// some space
	(new QWidget(hbox3))->setMinimumWidth(80);
	(new QWidget(hbox3))->setMinimumWidth(80);
	button_ok_ = new QPushButton(tr("OK"), hbox3);
	button_cancel_ = new QPushButton(tr("Cancel"), hbox3);

	connect(button_ok_, SIGNAL(clicked()), SLOT(accept()));
	connect(button_cancel_, SIGNAL(clicked()), SLOT(reject()));
	connect(button_browse1_, SIGNAL(clicked()), this, SLOT(clicked_browse1()));
	connect(button_browse2_, SIGNAL(clicked()), this, SLOT(clicked_browse2()));
#endif
}

QString DiffDialog::getFile1() {
	if (file1_ == 0)
		return "";

	QString retval;

	if (file1_->text().isEmpty() == true)
		retval = "";
	else {
		retval = file1_->text();
	}
	return retval;
}

QString DiffDialog::getFile2() {
	if (file2_ == 0)
		return "";

	QString retval;

	if (file2_->text().isEmpty() == true)
		retval = "";
	else {
		retval = file2_->text();
	}
	return retval;
}

void DiffDialog::clicked_browse1() {
	QString file = Q3FileDialog::getOpenFileName(Global::instance()->diff_dir1, QString::null, this );
	if (!file.isNull()) {
		QFileInfo info(file);
		Global::instance()->diff_dir1 = info.absFilePath();
	}
	file1_->setText(file);
}

void DiffDialog::clicked_browse2() {
	QString file = Q3FileDialog::getOpenFileName(Global::instance()->diff_dir2, QString::null, this );
	if (!file.isNull()) {
		QFileInfo info(file);
		Global::instance()->diff_dir2 = info.absFilePath();
	}
	file2_->setText(file);
}

static inline bool isRegularLine(QCharRef ch) {
	if (ch != QChar(0x01) && ch != QChar(0x02) && ch != QChar(0x03))
		return true;
	return false;
}

//*****************************************************************************/
// diff works dialog
//*****************************************************************************/
DiffWorksDialog::~DiffWorksDialog() {
#ifndef ROSPELL_NO_DIFF
	delete syntax_;
#endif
}
DiffWorksDialog::DiffWorksDialog(QFont &fnt, QString file1, QString file2): QDialog(0, "diffworks", true) {
#ifndef ROSPELL_NO_DIFF
LOG("file1 %s, file2 %s", (const char *) file1, (const char *) file2);
	Global *g = Global::instance();
	Q3VBoxLayout *vly = new Q3VBoxLayout(this, 10);
	vly->setAutoAdd(true);

	// file1
	Q3HBox *hbox1 = new Q3HBox(this);
	hbox1->setSpacing(10);
	f1_ = new QLabel("1: " + file1, hbox1);

	// file2
	Q3HBox *hbox2 = new Q3HBox(this);
	hbox2->setSpacing(10);
	f2_ = new QLabel("2: " + file2, hbox2);

	// text box
	Q3HBox *hbox3 = new Q3HBox(this);
	hbox3->setSpacing(10);
	text_ = new Q3MultiLineEdit(hbox3);
	syntax_ = getSyntax(".diff", text_);
	// config text
	text_->setFont(fnt);
	text_->setWordWrap(Q3TextEdit::NoWrap);
	text_->setReadOnly(TRUE);
	// colors
	text_->setPaletteBackgroundColor(g->c_background);
	text_->setPaletteForegroundColor(g->c_foreground);
	QPalette qp = text_->palette();
	qp.setColor(QPalette::Active, QColorGroup::Highlight, g->c_highlight);
	qp.setColor(QPalette::Inactive, QColorGroup::Highlight, g->c_highlight);
	text_->setPalette(qp);
	// do diff
	BCursor busy;
	QString result = doDiff((const char *) file1, (const char *) file2);
	text_->setText(QString::fromUtf8(result));
	resize(Global::instance()->diff_x, Global::instance()->diff_y);

	// accept/reject
	Q3HBox *hbox4 = new Q3HBox(this);
	hbox4->setSpacing(10);
	if (result == "Files do not differ!") {
		// only ok button is shown
		new QWidget(hbox4);
		new QWidget(hbox4);
		new QWidget(hbox4);
		new QWidget(hbox4);
		button_ok_ = new QPushButton(tr("OK"), hbox4);
		connect(button_ok_, SIGNAL(clicked()), SLOT(accept()));
		return;
	}
	// else show next/previous/ok
	button_next_ = new QPushButton(tr("Next"), hbox4);
	button_previous_ = new QPushButton(tr("Previous"), hbox4);
	// some space
	(new QWidget(hbox4))->setMinimumWidth(80);
	(new QWidget(hbox4))->setMinimumWidth(80);
	button_ok_ = new QPushButton("OK", hbox4);
	connect(button_ok_, SIGNAL(clicked()), SLOT(accept()));
	connect(button_next_, SIGNAL(clicked()), this, SLOT(clicked_next()));
	connect(button_previous_, SIGNAL(clicked()), this, SLOT(clicked_previous()));

	// set initial selection
	int line;
	int col;
	text_->getCursorPosition(&line, &col);
	QString str = text_->textLine(line);
	if (isRegularLine(str.at(0)))
		clicked_next();
	else {
		int start = line;
		int end = start + 1;
		while (!text_->textLine(end).isNull() &&
		       !text_->textLine(end).isEmpty() &&
		       !isRegularLine(text_->textLine(end).at(0)))
			end++;

		text_->setSelection(start, 0, end, 0);
	}
#endif
}

void DiffWorksDialog::clicked_next() {
#ifndef ROSPELL_NO_DIFF
	int line;
	int col;
	BCursor busy;
	text_->getCursorPosition(&line, &col);
	QString str = "";
	bool found = false;
	if (line != 0)
		line++;
	while (1) {
		text_->setCursorPosition(line, 0, false);
		if (text_->atEnd())
			return;
		str = text_->textLine(line);
		if (!isRegularLine(str.at(0))) {
			found = true;
			break;
		}
		line++;
	}

	int start = line;
	int end = line;
	while (found) {
		line++;
		end = line;
		text_->setCursorPosition(line, 0, false);
		if (text_->atEnd())
			break;
		str = text_->textLine(line);
		if (str.isNull() || str.isEmpty())
			break;
		if (isRegularLine(str.at(0)))
			break;
	}

	text_->setSelection(start, 0, end, 0);
	update();
#endif
}

void DiffWorksDialog::clicked_previous() {
#ifndef ROSPELL_NO_DIFF
	int line;
	int col;
	BCursor busy;
	text_->getCursorPosition(&line, &col);
	if (!text_->atEnd())
		line--;
	QString str = text_->textLine(line);
	while (line >= 0 && !isRegularLine(str.at(0))) {
		line--;
		str = text_->textLine(line);
	}

	bool found = false;
	while (line >= 0) {
		line--;
		text_->setCursorPosition(line, 0, false);
		str = text_->textLine(line);
		if (!isRegularLine(str.at(0))) {
			found = true;
			break;
		}
	}

	int start = line;
	int end = line;
	while (found && line >= 0) {
		line--;
		end = line;
		text_->setCursorPosition(line, 0, false);
		if (text_->atEnd())
			return;
		str = text_->textLine(line);
		if (str.isNull() || str.isEmpty())
			break;
		if (isRegularLine(str.at(0)))
			break;
	}
	text_->setSelection(end + 1, 0, start + 1, 0);
	update();
#endif
}

void DiffWorksDialog::resizeEvent( QResizeEvent * )
{
	LOG("DiffWorksDialog::resizeEvent sizes %d %d", width(), height());
//	if (e_ && m_ )
//		e_->setGeometry( 0, m_->height(), width(), height() - m_->height());
	Global::instance()->diff_x = width();
	Global::instance()->diff_y = height();
}


//*****************************************************************************/
// diff works dialog
//*****************************************************************************/
DirMergeWorksDialog::~DirMergeWorksDialog() {
#ifndef ROSPELL_NO_DIFF
	delete syntax_;
#endif
}
DirMergeWorksDialog::DirMergeWorksDialog(QFont &fnt, QString file1, QString file2,
					QString list1, QString list2, ProgressBar *progress): QDialog(0, "dir merge works", true) {
#ifndef ROSPELL_NO_DIFF

LOG("file1 %s, file2 %s", (const char *) file1, (const char *) file2);
	dir1_ = file1;
	dir2_ = file2;
	Global *g = Global::instance();
	Q3VBoxLayout *vly = new Q3VBoxLayout(this, 10);
	vly->setAutoAdd(true);

	// file1
	Q3HBox *hbox1 = new Q3HBox(this);
	hbox1->setSpacing(10);
	f1_ = new QLabel(tr("1: only in ") + file1, hbox1);

	// file2
	Q3HBox *hbox2 = new Q3HBox(this);
	hbox2->setSpacing(10);
	f2_ = new QLabel(tr("2: only in ") + file2, hbox2);

	Q3HBox *hbox21 = new Q3HBox(this);
	hbox21->setSpacing(10);
	new QLabel(tr("3: to be merged"), hbox21);
	Q3HBox *hbox22 = new Q3HBox(this);
	hbox22->setSpacing(10);
	new QLabel(tr("The directory merge result is stored in 2: ") + file2, hbox22);

	// text box
	Q3HBox *hbox3 = new Q3HBox(this);
	hbox3->setSpacing(10);
	text_ = new Q3MultiLineEdit(hbox3);
	syntax_ = getSyntax(".diff", text_);
	// config text
	text_->setFont(fnt);
	text_->setWordWrap(Q3TextEdit::NoWrap);
	text_->setReadOnly(TRUE);
	// colors
	text_->setPaletteBackgroundColor(g->c_background);
	text_->setPaletteForegroundColor(g->c_foreground);
	QPalette qp = text_->palette();
	qp.setColor(QPalette::Active, QColorGroup::Highlight, g->c_highlight);
	qp.setColor(QPalette::Inactive, QColorGroup::Highlight, g->c_highlight);
	text_->setPalette(qp);


	// save first the files in backup directory
	BCursor busy;
	char *backup_fname1 = Global::getBackupFname("xyzfgh1");
	UTILS::write_file(backup_fname1, list1);
	char *backup_fname2 = Global::getBackupFname("xyzfgh2");
	UTILS::write_file(backup_fname2, list2);
LOG("backup file1 %s, file2 %s", backup_fname1, backup_fname2);
//lucian: error recovery in case the files couldn't be saved under backup directory

	// do the initial diff; this will highlight the files missing from one directory to another
	QString result = doDiff(backup_fname1, backup_fname2);
	delete[] backup_fname1;
	delete[] backup_fname2;

	if (result == "Files do not differ!")
		result = list1;

	// find modified files
	{
		QStringList lst(QStringList::split( "\n", result));
		result = "";
		QStringList::Iterator it = lst.begin();
		int steps = lst.size();
		int step = 0;
		int i = 0;
		progress->start(steps, true);
		for (; it != lst.end(); ++it) {
			if (i >= 20) {
				step += 20;
				progress->adjust(step);
				qApp->processEvents();
				i = 0;
			}
			i++;

			QString file = *it;
			if (!isRegularLine(file.at(0))) {
				result += file + "\n\n";
			}
			else {
				// compare the two files
				QString f1 = file1 + file;
				QString f2 = file2 + file;
				QFile qf1(f1);
				QFile qf2(f2);
				if (qf1.size() != qf2.size()) {
					result += QChar(0x03);
					result += "3: " + file + "\n\n";
				}
				else {
					int equal = UTILS::file_compare((const char *) f1, (const char *) f2);
					if (equal != 0) {
						result += QChar(0x03);
						result += "3: " + file + "\n\n";
					}
				}
			}
		}
	}

	// set the text area
	text_->setText(QString::fromUtf8(result));
	resize(Global::instance()->diff_x, Global::instance()->diff_y);

	// accept/reject
	Q3HBox *hbox4 = new Q3HBox(this);
	hbox4->setSpacing(10);
	if (result.isEmpty()) {
		text_->setText(QString(tr("no diferences!")));
		// only ok button is shown
		new QWidget(hbox4);
		new QWidget(hbox4);
		new QWidget(hbox4);
		new QWidget(hbox4);
		button_ok_ = new QPushButton(tr("OK"), hbox4);
		connect(button_ok_, SIGNAL(clicked()), SLOT(accept()));
		return;
	}
	// else show next/previous/ok
	button_next_ = new QPushButton(tr("Next"), hbox4);
	button_previous_ = new QPushButton(tr("Previous"), hbox4);
	// some space
	new QWidget(hbox4);
	button_one_ = new QPushButton(tr("Copy"), hbox4);
	button_two_ = new QPushButton(tr("Skip"), hbox4);
	button_merge_ = new QPushButton(tr("Merge"), hbox4);
	// some space
	new QWidget(hbox4);
	button_ok_ = new QPushButton(tr("OK"), hbox4);
	connect(button_ok_, SIGNAL(clicked()), SLOT(accept()));
	connect(button_next_, SIGNAL(clicked()), this, SLOT(clicked_next()));
	connect(button_previous_, SIGNAL(clicked()), this, SLOT(clicked_previous()));
	connect(button_merge_, SIGNAL(clicked()), this, SLOT(clicked_merge()));
	connect(button_one_, SIGNAL(clicked()), SLOT(clicked_one()));
	connect(button_two_, SIGNAL(clicked()), SLOT(clicked_two()));

	// set initial selection
	int line;
	int col;
	text_->getCursorPosition(&line, &col);
	QString str = text_->textLine(line);
	if (isRegularLine(str.at(0)))
		clicked_next();
	else {
		int start = line;
		int end = start + 1;
		while (!text_->textLine(end).isNull() &&
		       !text_->textLine(end).isEmpty() &&
		       !isRegularLine(text_->textLine(end).at(0)))
			end++;

		text_->setSelection(start, 0, end, 0);
	}
#endif
}

void DirMergeWorksDialog::clicked_one() {
#ifndef ROSPELL_NO_DIFF
	if (!text_->hasSelectedText())
		return;

	switch (QMessageBox::warning( this, tr("Warning"),
		tr("This will copy the file from dir 1 into dir 2\n"),
		tr("&Yes"),
		tr("&Cancel"),
		0, 1)) {
		case 0: // yes
			break;
		default: // cancel
			return;
	}
	// copy the file here
	int line;
	int col;
	BCursor busy;
	text_->getCursorPosition(&line, &col);
	QString str = text_->textLine(line - 1);
	if (str.isNull() || str.isEmpty())
		return;
	QChar ch = str.at(0);
	if (ch == QChar(0x01) || ch == QChar(0x03)) {
		QString file = str.mid(4);
		QString f1 = dir1_ + file;
		QString f2 = dir2_ + file;
//printf("file1 %s, file2 %s\n", (const char *) f1, (const char *) f2);
		char *txt = UTILS::read_file_malloc((const char *) f1);
		if (txt != 0) {
			bool fine = UTILS::write_file((const char *) f2, txt);
			if (!fine) {
				QFileInfo qf(f2);
				QString msg = qf.dirPath(true) + tr("\n\ndirectory will be created. Continue?");
				switch (QMessageBox::warning( this, tr("Warning"),
					msg,
					tr("&Yes"),
					tr("&Cancel"),
					0, 1)) {
					case 0: { // yes
						printf("directory %s not found\n", (const char *) qf.dirPath(true));
						QString prg = "mkdir -p " + qf.dirPath(true); // -p is supported on Linux and freeBSD
						printf("%s\n", (const char *) prg);
						char *rval = UTILS::run_prg_malloc((const char *) prg);
						if (rval != 0)
							delete[] rval;
						fine = UTILS::write_file((const char *) f2, txt);
						if (fine == false)
							QMessageBox::warning( this, tr("Warning"), tr("Directory or file creation failed!"));
					} break;
					default: // cancel
						break;
				}
			}
		}
		delete[] txt;
	}
	clicked_two();
#endif
}
void DirMergeWorksDialog::clicked_two() {
#ifndef ROSPELL_NO_DIFF
	// the file in dir2 is left as is
	int para_from;
	int index_from;
	int para_to;
	int index_to;
	BCursor busy;

	text_->getSelection(&para_from, &index_from, &para_to, &index_to);
	if (para_from == -1 || index_from == -1 || para_to == -1 || index_to == -1) {
		return;
	}
	text_->del();
	clicked_next();
#endif
}
void DirMergeWorksDialog::clicked_merge() {
#ifndef ROSPELL_NO_DIFF
	if (!text_->hasSelectedText())
		return;


	// copy the file here
	int line;
	int col;
	text_->getCursorPosition(&line, &col);
	QString str = text_->textLine(line - 1);
	if (str.isNull() || str.isEmpty())
		return;
	QChar ch = str.at(0);
	if (ch == QChar(0x03)) {
		QString file = str.mid(4);
		QString f1 = dir1_ + file;
		QString f2 = dir2_ + file;
//printf("file1 %s, file2 %s\n", (const char *) f1, (const char *) f2);
		QFont fnt = text_->font();
		MergeWorksDialog *wd = new MergeWorksDialog(fnt, f1, f2);
		wd->setCaption(tr("merge"));
		wd->exec();
		delete wd;
	}
	clicked_two();
#endif
}

void DirMergeWorksDialog::clicked_next() {
#ifndef ROSPELL_NO_DIFF
	int line;
	int col;
	BCursor busy;
	text_->getCursorPosition(&line, &col);
	QString str = "";
	bool found = false;
	if (line != 0)
		line++;
	while (1) {
		text_->setCursorPosition(line, 0, false);
		if (text_->atEnd())
			return;
		str = text_->textLine(line);
		if (!isRegularLine(str.at(0))) {
			found = true;
			break;
		}
		line++;
	}

	int start = line;
	int end = line;
	while (found) {
		line++;
		end = line;
		text_->setCursorPosition(line, 0, false);
		if (text_->atEnd())
			break;
		str = text_->textLine(line);
		if (str.isNull() || str.isEmpty())
			break;
		if (isRegularLine(str.at(0)))
			break;
	}

	text_->setSelection(start, 0, end, 0);
	update();
#endif
}

void DirMergeWorksDialog::clicked_previous() {
#ifndef ROSPELL_NO_DIFF
	int line;
	int col;
	BCursor busy;
	text_->getCursorPosition(&line, &col);
	if (!text_->atEnd())
		line--;
	QString str = text_->textLine(line);
	while (line >= 0 && !isRegularLine(str.at(0))) {
		line--;
		str = text_->textLine(line);
	}

	bool found = false;
	while (line >= 0) {
		line--;
		text_->setCursorPosition(line, 0, false);
		str = text_->textLine(line);
		if (!isRegularLine(str.at(0))) {
			found = true;
			break;
		}
	}

	int start = line;
	int end = line;
	while (found && line >= 0) {
		line--;
		end = line;
		text_->setCursorPosition(line, 0, false);
		if (text_->atEnd())
			return;
		str = text_->textLine(line);
		if (str.isNull() || str.isEmpty())
			break;
		if (isRegularLine(str.at(0)))
			break;
	}
	text_->setSelection(end + 1, 0, start + 1, 0);
	update();
#endif
}

void DirMergeWorksDialog::resizeEvent( QResizeEvent * )
{
#ifndef ROSPELL_NO_DIFF
	LOG("DiffWorksDialog::resizeEvent sizes %d %d", width(), height());
//	if (e_ && m_ )
//		e_->setGeometry( 0, m_->height(), width(), height() - m_->height());
	Global::instance()->diff_x = width();
	Global::instance()->diff_y = height();
#endif
}


//*****************************************************************************/
// merge works dialog
//*****************************************************************************/
MergeWorksDialog::~MergeWorksDialog() {
#ifndef ROSPELL_NO_DIFF
	delete syntax_;
#endif
}
MergeWorksDialog::MergeWorksDialog(QFont &fnt, QString file1, QString file2): QDialog(0, "diffworks", true) {
#ifndef ROSPELL_NO_DIFF
LOG("file1 %s, file2 %s", (const char *) file1, (const char *) file2);
	saved_ = false;
	Global *g = Global::instance();
	Q3VBoxLayout *vly = new Q3VBoxLayout(this, 10);
	vly->setAutoAdd(true);

	// file1
	Q3HBox *hbox1 = new Q3HBox(this);
	hbox1->setSpacing(10);
	f1_ = new QLabel("1: " + file1, hbox1);

	// file2
	Q3HBox *hbox2 = new Q3HBox(this);
	hbox2->setSpacing(10);
	f2_ = new QLabel("2: " + file2, hbox2);

	// text box
	Q3HBox *hbox3 = new Q3HBox(this);
	hbox3->setSpacing(10);
	text_ = new Q3MultiLineEdit(hbox3);
	syntax_ = getSyntax(".diff", text_);
	// configure text
	text_->setFont(fnt);
	text_->setWordWrap(Q3TextEdit::NoWrap);
	// colors
	text_->setPaletteBackgroundColor(g->c_background);
	text_->setPaletteForegroundColor(g->c_foreground);
	QPalette qp = text_->palette();
	qp.setColor(QPalette::Active, QColorGroup::Highlight, g->c_highlight);
	qp.setColor(QPalette::Inactive, QColorGroup::Highlight, g->c_highlight);
	text_->setPalette(qp);
	connect(text_, SIGNAL(textChanged()), this, SLOT(textChanged()));
	// diff
	BCursor busy;
	QString result = doDiff((const char *) file1, (const char *) file2);
	text_->setText(QString::fromUtf8(result));
	resize(Global::instance()->diff_x, Global::instance()->diff_y);



	// accept/reject
	Q3HBox *hbox4 = new Q3HBox(this);
	hbox4->setSpacing(10);
	if (result == "Files do not differ!") {
		new QWidget(hbox4);
		new QWidget(hbox4);
		new QWidget(hbox4);
		new QWidget(hbox4);
		new QWidget(hbox4);
		new QWidget(hbox4);
		button_cancel_ = new QPushButton(tr("Cancel"), hbox4);
		connect(button_cancel_, SIGNAL(clicked()), SLOT(reject()));
		return;
	}
	// else show all buttons
	button_next_ = new QPushButton(tr("Next"), hbox4);
	button_previous_ = new QPushButton(tr("Previous"), hbox4);
	// some space
	new QWidget(hbox4);
	button_one_ = new QPushButton("1", hbox4);
	button_two_ = new QPushButton("2", hbox4);
	button_save_ = new QPushButton(tr("Save"), hbox4);
	// some space
	new QWidget(hbox4);
	button_ok_ = new QPushButton(tr("OK"), hbox4);
	button_cancel_ = new QPushButton(tr("Cancel"), hbox4);

	connect(button_ok_, SIGNAL(clicked()), SLOT(clicked_ok())); //accept()
	connect(button_cancel_, SIGNAL(clicked()), SLOT(reject()));
	connect(button_next_, SIGNAL(clicked()), this, SLOT(clicked_next()));
	connect(button_previous_, SIGNAL(clicked()), this, SLOT(clicked_previous()));
	connect(button_save_, SIGNAL(clicked()), this, SLOT(clicked_save()));
	connect(button_one_, SIGNAL(clicked()), SLOT(clicked_one()));
	connect(button_two_, SIGNAL(clicked()), SLOT(clicked_two()));


	// set initial selection
	int line;
	int col;
	text_->getCursorPosition(&line, &col);
	QString str = text_->textLine(line);
	if (isRegularLine(str.at(0)))
		clicked_next();
	else {
		int start = line;
		int end = start + 1;
		while (!text_->textLine(end).isNull() &&
		       !text_->textLine(end).isEmpty() &&
		       !isRegularLine(text_->textLine(end).at(0)))
			end++;

		text_->setSelection(start, 0, end, 0);
	}
#endif
}

void MergeWorksDialog::textChanged() {
	saved_ = false;
}

bool MergeWorksDialog::isMoreToMerge() {
#ifndef ROSPELL_NO_DIFF
	QString str1 = QChar(0x01);
	QString str2 = QChar(0x02);
	int index = 0;
	int para = 0;
	bool found = text_->find(str1, true, false, true, &index, &para);
	if (found) {
		QMessageBox::warning(this,tr("Merge Info"), QString(tr("There is more to merge!\n")));
		text_->setCursorPosition(index, para, false);
		text_->setFocus();
		return true;
	}
	found = text_->find(str2, true, false, true, &index, &para);
	if (found) {
		QMessageBox::warning(this,tr("Merge Info"), QString(tr("There is more to merge!\n")));
		text_->setCursorPosition(index, para, false);
		text_->setFocus();
		return true;
	}
#endif
	return false;

}
void MergeWorksDialog::clicked_one() {
#ifndef ROSPELL_NO_DIFF
	int para_from;
	int index_from;
	int para_to;
	int index_to;
	BCursor busy;

	text_->getSelection(&para_from, &index_from, &para_to, &index_to);
	if (para_from == -1 || index_from == -1 || para_to == -1 || index_to == -1) {
		return;
	}

	for (int i = para_from; i < para_to; i++) {
		QString line = text_->textLine(i);
		if (line.startsWith(QChar(0x01))) {
			line = line.mid(4);
			text_->setSelection(i, 0, i + 1, 0);
			text_->del();
			text_->insert(line + QString("\n"));
		}
		else if (line.startsWith(QChar(0x02))) {
			text_->setSelection(i, 0, i + 1, 0);
			text_->del();
			i--;
			para_to--;
		}
	}

	clicked_next();
#endif
}

void MergeWorksDialog::clicked_two() {
#ifndef ROSPELL_NO_DIFF
	int para_from;
	int index_from;
	int para_to;
	int index_to;
	BCursor busy;

	text_->getSelection(&para_from, &index_from, &para_to, &index_to);
	if (para_from == -1 || index_from == -1 || para_to == -1 || index_to == -1) {
		return;
	}

	for (int i = para_from; i < para_to; i++) {
		QString line = text_->textLine(i);
		if (line.startsWith(QChar(0x02))) {
			line = line.mid(4);
			text_->setSelection(i, 0, i + 1, 0);
			text_->del();
			text_->insert(line + QString("\n"));
		}
		else if (line.startsWith(QChar(0x01))) {
			text_->setSelection(i, 0, i + 1, 0);
			text_->del();
			i--;
			para_to--;
		}
	}

	clicked_next();
#endif
}

void MergeWorksDialog::clicked_save() {
#ifndef ROSPELL_NO_DIFF
	if (isMoreToMerge())
		return;

//	BCursor busy;
	saved_ = true;
	QString fileName = Q3FileDialog::getSaveFileName( QString::null, QString::null, this );
	BCursor busy;
	if (fileName.isEmpty()) {
		QMessageBox::warning(this,tr("I/O Error"), QString(tr("The file could not be opened.\n\n")));
		return;
	}

	FILE_FORMAT format = FILE_UTF8;
	QFile f( fileName );
#ifdef USING_QT3
	if (!f.open(IO_WriteOnly )) {
#endif
#ifdef USING_QT4
	if (!f.open( QIODevice::WriteOnly )) {
#endif

		QMessageBox::warning(this,tr("I/O Error"), QString(tr("The file could not be opened.\n\n")) + fileName);
		return;
	}
	QTextStream t(&f);
	LOG("save common format %d", format);
//lucian: ???????????????????
	switch (format) {
		case FILE_UTF8:
			t.setEncoding( QTextStream::UnicodeUTF8 );
			LOG("Editor::saveCommon utf8");
			break;
		case FILE_CP1250: {
				// lucian: qt has some problems with this conversion!!!
				QTextCodec *codec = QTextCodec::codecForName("CP1250");
				t.setCodec(codec);
				LOG("Editor::saveCommon cp1250");
			}
			break;
		case FILE_ISO8859_2: {
				QTextCodec *codec = QTextCodec::codecForName("ISO8859-2");
				t.setCodec(codec);
				LOG("Editor::saveCommon iso8859-2");
			}
			break;
		case FILE_ISO8859_16: {
				QTextCodec *codec = QTextCodec::codecForName("ISO8859-16");
				t.setCodec(codec);
				LOG("Editor::saveCommon iso8859-16");
			}
			break;
		default:
			ASSERT(0);
			return;
	}

	QString tmp;

	// if this is a latex file save it in latex/ascii format
	if (fileName.endsWith(".tex"))
		tmp = utf82latex(text_->text());
	else
		tmp = text_->text();
	t << tmp;
	f.close();
#endif
}

void MergeWorksDialog::clicked_ok() {
	if (isMoreToMerge())
		return;
	if (!saved_) {
		QMessageBox::warning(this,tr("Merge Info"), QString(tr("Merge result was not saved!\n")));
		return;
	}

	accept();
}

void MergeWorksDialog::clicked_next() {
#ifndef ROSPELL_NO_DIFF
	int line;
	int col;
	BCursor busy;
	text_->getCursorPosition(&line, &col);
	QString str = "";
	bool found = false;
	if (!line != 0)
		line++;
	while (1) {
		text_->setCursorPosition(line, 0, false);
		if (text_->atEnd())
			return;
		str = text_->textLine(line);
		if (!isRegularLine(str.at(0))) {
			found = true;
			break;
		}
		line++;
	}

	int start = line;
	int end = line;
	while (found) {
		line++;
		end = line;
		text_->setCursorPosition(line, 0, false);
		if (text_->atEnd())
			break;
		str = text_->textLine(line);
		if (str.isNull() || str.isEmpty())
			break;
		if (isRegularLine(str.at(0)))
			break;
	}

	text_->setSelection(start, 0, end, 0);
	update();
#endif
}

void MergeWorksDialog::clicked_previous() {
#ifndef ROSPELL_NO_DIFF
	int line;
	int col;
	BCursor busy;
	text_->getCursorPosition(&line, &col);
	if (!text_->atEnd())
		line--;
	QString str = text_->textLine(line);
	while (line >= 0 && !isRegularLine(str.at(0))) {
		line--;
		str = text_->textLine(line);
	}

	bool found = false;
	while (line >= 0) {
		line--;
		text_->setCursorPosition(line, 0, false);
		str = text_->textLine(line);
		if (!isRegularLine(str.at(0))) {
			found = true;
			break;
		}
	}

	int start = line;
	int end = line;
	while (found && line >= 0) {
		line--;
		end = line;
		text_->setCursorPosition(line, 0, false);
		if (text_->atEnd())
			return;
		str = text_->textLine(line);
		if (str.isNull() || str.isEmpty())
			break;
		if (isRegularLine(str.at(0)))
			break;
	}
	text_->setSelection(end + 1, 0, start + 1, 0);
	update();
#endif
}

void MergeWorksDialog::resizeEvent( QResizeEvent * )
{
	LOG("MergeWorksDialog::resizeEvent sizes %d %d", width(), height());
//	if (e_ && m_ )
//		e_->setGeometry( 0, m_->height(), width(), height() - m_->height());
	Global::instance()->diff_x = width();
	Global::instance()->diff_y = height();
}

