//*********************************************************************
// Copyright (C) 2005,  Lucian Constantin <struct_bylighting@yahoo.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//*********************************************************************
// Code based on utelnetd package. Original copyright message as follows
/* utelnetd.c
 *
 * Simple telnet server
 *
 * Artur Bajor, Centrum Informatyki ROW
 * <abajor@cirow.pl>
 *
 * Joerg Schmitz-Linneweber, Aston GmbH
 * <schmitz-linneweber@aston-technologie.de>
 *
 * Vladimir Oleynik
 * <dzo@simtreas.ru>
 *
 * Robert Schwebel, Pengutronix
 * <r.schwebel@pengutronix.de>
 * 
 * Bjorn Wesen, Axis Communications AB 
 * <bjornw@axis.com>
 * 
 * Sepherosa Ziehau
 * <sepherosa@myrealbox.com>
 * 
 * This file is distributed under the GNU General Public License (GPL),
 * please see the file LICENSE for further information.
 * 
 * ---------------------------------------------------------------------------
 * (C) 2000, 2001, 2002, 2003 by the authors mentioned above
 * ---------------------------------------------------------------------------
*/

#include "gdb.h"
#include "gdbedit.h"
#include "qwerty.h"
#include "utils.h"
#include "programdialog.h"
#include "briefedit.h"
#include "dictionaries.h"
#include <qapplication.h>
#include <time.h>
#include <sys/types.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <signal.h>

#include <qaction.h>
#include <qtoolbar.h>
#include "xpm/gdb_next.xpm"
#include "xpm/gdb_run.xpm"
#include "xpm/gdb_sout.xpm"
#include "xpm/gdb_step.xpm"
#include "xpm/gdb_stop.xpm"


// lucian: clean run_proc up
static int run_proc = 0;

#include <sys/time.h>
#include <sys/socket.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <string.h>
#include <unistd.h>

/* we need getpty() */
#define __USE_GNU 1
#define __USE_XOPEN 1
#include <stdlib.h>
#undef __USE_GNU
#undef __USE_XOPEN

#include <errno.h>
#include <netinet/in.h>
#include <fcntl.h>
#include <stdio.h>
#include <signal.h>


#include <termios.h>

#ifdef ROSPELL_FREEBSD
#define XTABS OXTABS
#define getpt() posix_openpt(O_RDWR|O_NOCTTY)
#endif


static QString loginpath = "";
static QString arg1 = "-f";
/* shell name and arguments */

//#define DEBUG_OUT(...) fprintf(stderr, __VA_ARGS__)
#define DEBUG_OUT(...)
static struct tsession *sessions;

struct tsession {
	struct tsession *next;
	int ptyfd;
	int shell_pid;
};

void perror_msg_and_die(const char *text)
{
	fprintf(stderr,text);
	exit(1);
}

void error_msg_and_die(const char *text)
{
	perror_msg_and_die(text);
}

static int getpty(char *line)
{
#ifndef ROSPELL_CYGWIN
        int p;

        p = getpt();
        if (p < 0) {
                DEBUG_OUT("getpty(): couldn't get pty\n");
                close(p);
                return -1;
        }
        if (grantpt(p)<0 || unlockpt(p)<0) {
                DEBUG_OUT("getpty(): couldn't grant and unlock pty\n");
                close(p);
                return -1;
        }
        DEBUG_OUT("getpty(): got pty %s\n",ptsname(p));
        strcpy(line, (const char*)ptsname(p));

        return(p);
#else
	return 0;
#endif
}

static struct tsession *
make_new_session()
{
#ifndef ROSPELL_CYGWIN
	struct termios termbuf;
	int pty, pid;
	static char tty_name[32];
	struct tsession *ts = (struct tsession *)malloc(sizeof(struct tsession));
	int t1, t2;
	/* Got a new connection, set up a tty and spawn a shell.  */

	pty = getpty(tty_name);

	if (pty < 0) {
		fprintf(stderr, "All network ports in use!\n");
		return 0;
	}


	ts->ptyfd = pty;

	if ((pid = fork()) < 0) {
		perror("fork");
	}
	if (pid == 0) {
		/* In child, open the child's side of the tty.  */
		int i, t;

		for(i = 0; i <= pty; i++)
			close(i);

		/* make new process group */
		if (setsid() < 0)
			perror_msg_and_die("setsid");
		t = open(tty_name, O_RDWR | O_NOCTTY);
		if (t < 0)
			perror_msg_and_die("Could not open tty");

		t1 = dup(0);
		t2 = dup(1);

		tcsetpgrp(0, getpid());
		if (ioctl(t, TIOCSCTTY, NULL)) {
			perror_msg_and_die("could not set controlling tty");
		} 
   
		/* The pseudo-terminal allocated to the client is configured to operate in
		 * cooked mode, and with XTABS CRMOD enabled (see tty(4)).
		 */

		tcgetattr(t, &termbuf);
// lucian: modified for noecho		
//		termbuf.c_lflag |= ECHO; /* if we use readline we dont want this */
		termbuf.c_lflag &= ~ECHO; /* if we use readline we dont want this */
		termbuf.c_oflag |= ONLCR|XTABS;
		termbuf.c_iflag |= ICRNL;
		termbuf.c_iflag &= ~IXOFF;
		/* termbuf.c_lflag &= ~ICANON; */
		tcsetattr(t, TCSANOW, &termbuf);

		DEBUG_OUT("stdin, stdout, stderr: %d %d %d\n", t, t1, t2);

		/* exec shell, with correct argv and env */
		execl((const char *) loginpath, (const char *) loginpath, (const char *) arg1, NULL);
//		execv(loginpath, argv_init);

	        /* NOT REACHED */
		perror_msg_and_die("execv");
	}

	ts->shell_pid = pid;
run_proc = pid;
	return ts;
#else
	return 0;
#endif	
}

static void
free_session(struct tsession *ts)
{
	struct tsession *t = sessions;

	/* Unlink this telnet session from the session list.  */
	if(t == ts)
		sessions = ts->next;
	else {
		while(t->next != ts)
			t = t->next;
		t->next = ts->next;
	}

//	free(ts->buf1);
//	free(ts->buf2);

	kill(ts->shell_pid, SIGKILL);

	wait4(ts->shell_pid, NULL, 0, NULL);

	close(ts->ptyfd);

	free(ts);
}

GdbThread::GdbThread(Editor *parent):
#ifdef USING_QT4
	QThread(parent),
#endif
#ifdef USING_QT3
	QThread(),
#endif
	parent_(parent), active_(false), w_(0) {


	LOG("gdb thread created");
}

GdbThread::~GdbThread() {
}

void GdbThread::injectCommand(QString cmd) {
	cmd += '\n';
	write(sessions->ptyfd, (const char *) cmd, cmd.length());
	fflush(0);
}
#define MAX_BUFF 10000



void GdbThread::run() {
	LOG("dbg thread run");

	run_proc = 0;
	sessions = make_new_session();
	// add gdb user work tag
	w_ = parent_->getGdbTab();
	ASSERT(w_ != 0);
	active_ = true;
	// connect the file descriptor to the widget
	w_->setFd(sessions->ptyfd);
	
	while (1) {
		char string[MAX_BUFF + 1];
		bool found_display_line = false;
		
		int len = read(sessions->ptyfd, string, MAX_BUFF);
		if (len > 0) // valid data
			string[len] = '\0';
		else if (len == 0) // end of file
			break;
		else if (len == -1) // error
			break;
			
		char dline[4];
		dline[0] = 0x1a;
		dline[1] = 0x1a;
		dline[2] = '/';
		dline[3] = '\0';
		char *ptr;
		if ((ptr = strstr(string, dline)) != 0) {
			// extract the display line
			char *start = ptr;
			while (*ptr != '\n' && *ptr != '\0')
				ptr++;
			if (*ptr != '\0') {
				ptrdiff_t len = ptr - start;
				char *d = new char[len + 1 - 2]; // the first two chars are not copied
				memcpy(d, start + 2, len - 2);
				d[len - 2] = '\0';
				LOG("dline #%s#\n", d);
				display_line_ = d;
				delete [] d;
				for (int i = 0; i < len; i++)
					start[i] = ' ';
				// send message to parent
				sendDisplayLine();
				found_display_line = true;
			}
		}
		// this is only necessary before the program was started under gdb
		if (QString(string).startsWith("(gdb) "))
			sendDisplayText("\n");
//			w_->insert("\n");

		sendDisplayText(QString::fromUtf8(string));

//		w_->insert(QString::fromUtf8(string));
//		w_->update();
		fflush(0);


	}
	// disconnect the file descriptor
	w_->setFd(0);
	
	run_proc = 0;
	w_->setText("gdb inactive");
	active_ = false;
	free_session(sessions);
}

void GdbThread::sendDisplayLine() {
#ifdef USING_QT4
	emit gdbDisplayLine();
#else
	QApplication::postEvent(parent_, new QCustomEvent(65431));
#endif
}

void GdbThread::sendDisplayText(QString txt) {
	char *str = new char [strlen((const char *) txt) + 1];
	strcpy(str, (const char *) txt);
#ifdef USING_QT4
	emit gdbDisplayText();
#else
	QCustomEvent *evt = new QCustomEvent(65430);
	evt->setData(str);
	QApplication::postEvent(parent_, evt);
#endif
}


void Editor::gdbEnable() {

	// remove dictionary from memory if prezent
	if (pdict_ != 0) {
		delete pdict_;
		pdict_ = 0;
		LOG("Editor::gdb - dictionaries deleted");
	}

	if (!gthread_->isActive()) {
		if (tw_gdb_ == 0) {
			QString cmd;
			if (Global::instance()->gdb_cmd_.isEmpty()) {
				ProgramDialog *d = new ProgramDialog("/usr/bin/gdb");
				d->setCaption(tr("gdb"));
				if (!d->exec()) {
					delete d;
					return;
				}
				cmd = d->getCmd();
				if (cmd.isEmpty()) {
					msgScreen(tr("Gdb command not found!"));
					return;
				}

				if (d->getDontShow()) {
					Global::instance()->gdb_cmd_ = cmd;
				}
				delete d;
			}
			else {
				cmd = Global::instance()->gdb_cmd_;
			}
			// set gdb program path
			loginpath = cmd;

			tw_gdb_ = new GdbEdit(this);
//			tw_user_->setReadOnly(TRUE);
			tw_gdb_->setPaletteBackgroundColor(Global::instance()->c_background);
			tw_gdb_->setPaletteForegroundColor(Global::instance()->c_foreground);
//			tw_gdb_->setPalette(qp);
			tw_->addTab(tw_gdb_, "gdb");
			tw_->setCurrentPage(ID_GDB);

#ifdef USING_QT4
			connect((const QObject *) gthread_, SIGNAL(gdbDisplayLine()), this, SLOT(gdbDisplayLine()));
#endif
			tool4_->show();
		}
		mwords_->hide();
		show_speller_ = false;
//		mgdb_->show();
		gthread_->start(QThread::LowPriority);
		// make the menu visible
		m_->setItemVisible(mgdbid_, true);
		tw_->setCurrentPage(ID_GDB);
		tw_gdb_->setText("");
		tw_gdb_->setFocus();
	}
}


void Editor::gdbStepOver() {
	if (gthread_ == 0 || gthread_->isActive() == false) {
		msgScreen("Gdb not enabled!");
		return;
	}
	// unselect all in e_
	// lucian: do a setselection on the current line
//	clrSelection();
	gthread_->injectCommand("next");
}

void Editor::gdbStepInto() {
	if (gthread_ == 0  || gthread_->isActive() == false) {
		msgScreen("Gdb not enabled!");
		return;
	}
//	clrSelection();
	gthread_->injectCommand("step");
}

void Editor::gdbRunRestart() {
	if (gthread_ == 0 || gthread_->isActive() == false) {
		msgScreen(tr("Gdb not enabled!"));
		return;
	}
//	clrSelection();
	gthread_->injectCommand("run");
}

void Editor::gdbStepOut() {
	if (gthread_ == 0 || gthread_->isActive() == false) {
		msgScreen(tr("Gdb not enabled!"));
		return;
	}
//	clrSelection();
	gthread_->injectCommand("finish");
}

void Editor::gdbRunCursor() {
	if (gthread_ == 0 || gthread_->isActive() == false) {
		msgScreen(tr("Gdb not enabled!"));
		return;
	}
}

void Editor::gdbContinue() {
	if (gthread_ == 0 || gthread_->isActive() == false) {
		msgScreen(tr("Gdb not enabled!"));
		return;
	}
//	clrSelection();
	gthread_->injectCommand("cont");
}

void Editor::gdbStop() {
	if (gthread_ == 0 || gthread_->isActive() == false) {
		msgScreen(tr("Gdb not enabled!"));
		return;
	}
//	clrSelection();
	if (run_proc == 0) {
		msgScreen(tr("Running porcess not known!"), 2000);
		return;
	}
//	kill(run_proc, SIGINT); - not working
	char str = 3;
	write(sessions->ptyfd, &str, 1);
}

void Editor::gdbExit() {
	if (gthread_ == 0 || gthread_->isActive() == false) {
		msgScreen(tr("Gdb not enabled!"));
		return;
	}
//	clrSelection();
	if (run_proc == 0) {
		msgScreen(tr("Running porcess not known!"), 2000);
		return;
	}
	kill(run_proc, SIGKILL);
}

void Editor::addGdbMenu() {
	tool4_ = new QToolBar(this, "toolbar");
 	tool4_->setLabel(tr("gdb"));
 	tool4_->hide();

	mgdb_ = new Q3PopupMenu(m_);
	mgdbid_ = m_->insertItem( "Gdb", mgdb_);
	m_->setItemVisible(mgdbid_, false);
	mgdb_->insertItem( tr("Run/Restart"),   this, SLOT(gdbRunRestart()));


	QAction *a = new QAction(tr("continue"), QPixmap(xpm_gdb_run), "", 0, this);
	connect( a, SIGNAL( activated() ), this, SLOT(gdbContinue()));
 	a->addTo(tool4_);
	mgdb_->insertItem( tr("Continue"),   this, SLOT(gdbContinue()), Qt::Key_F9);

	a = new QAction(tr("stop"), QPixmap(xpm_gdb_stop), "", 0, this);
	connect( a, SIGNAL( activated() ), this, SLOT(gdbStop()));
 	a->addTo(tool4_);
	mgdb_->insertItem( tr("Stop"),   this, SLOT(gdbStop()));
	mgdb_->insertSeparator();
	
	a = new QAction(tr("step over"), QPixmap(xpm_gdb_next), "", 0, this);
	connect( a, SIGNAL( activated() ), this, SLOT(gdbStepOver()));
 	a->addTo(tool4_);
	mgdb_->insertItem( tr("Step Over"),   this, SLOT(gdbStepOver()), Qt::Key_F10);
 	
	a = new QAction(tr("step into"), QPixmap(xpm_gdb_step), "", 0, this);
	connect( a, SIGNAL( activated() ), this, SLOT(gdbStepInto()));
 	a->addTo(tool4_);
	mgdb_->insertItem( tr("Step Into"),   this, SLOT(gdbStepInto()), Qt::Key_F11);
 	
	a = new QAction(tr("step out"), QPixmap(xpm_gdb_sout), "", 0, this);
	connect( a, SIGNAL( activated() ), this, SLOT(gdbStepOut()));
 	a->addTo(tool4_);
	mgdb_->insertItem( tr("Step Out"),   this, SLOT(gdbStepOut()));
	mgdb_->insertSeparator();

	mgdb_->insertItem( tr("Exit gdb"),   this, SLOT(gdbExit()));
}


