//*********************************************************************
// Copyright (C) 2005,  Lucian Constantin <struct_bylighting@yahoo.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//*********************************************************************

#ifndef GDBEDIT_H_
#define GDBEDIT_H_
#include "editor.h"
#include "debug.h"

#ifdef USING_QT3
#include <qmultilineedit.h>
#endif
#ifdef USING_QT4
#include <q3multilineedit.h>
#endif



class GdbEdit: public Q3MultiLineEdit {
	Q_OBJECT
public:
	GdbEdit(QWidget * parent = 0, const char * name = 0): Q3MultiLineEdit(parent, name), fd_(0) {
		buffer_ = "";
	}
	virtual ~GdbEdit() {}

	void setFd(int fd) {
		fd_ = fd;
	}

protected:
	void keyPressEvent(QKeyEvent *e) {
		if (fd_ == 0)
			return;
			
		int key = e->key();
		// ctrl-c pressed
		if (e->state() & Qt::ControlButton && key == Qt::Key_C) {
			// clean the buffer
			char str = 3;
			write(fd_, &str, 1);
			buffer_ = "";
			return;
		}

		// 1. add key to the local command buffer
		// 2. return key does not go in the command buffer
		// 3. when return key is pressed, sombody listning on returnPressed() signal
		//      will call getCommand() function above and the buffer_ will
		//      be cleaned
		QString s = "";

		QString txt = e->text();
		if (key == Qt::Key_Return); // return doesn't go in the buffer
		else if (key == Qt::Key_Backspace) {
			int len = buffer_.length();
			if (len > 0) {
				len--;
				buffer_.truncate(len);
			}
			else
				return;
		}
		else
			s = e->text();

		buffer_ += s;

		// if return pressed, send the buffer
		if (key == Qt::Key_Return) {
			buffer_ += '\n';
			write(fd_, (const char *) buffer_, buffer_.length());
			buffer_ = "";
		}


			
		LOG("GdbEdit::keyPressEvent #%s#, %x", (const char *) buffer_, (unsigned) *(const char *) buffer_);

		//add the text on the last line
		int lines = numLines();
		QString line = textLine(lines - 1);
		int len = line.length();
		setCursorPosition(lines - 1, len);

		// continue processing the key
		return Q3MultiLineEdit::keyPressEvent(e);
	}


private:
	QString getCommand() {
		QString retval = buffer_;
		buffer_ = "";
		return retval;
	}

	QString buffer_;
	volatile int fd_;
};

#endif
