//****************************************************************************
// Copyright (C) 2005,  Lucian Constantin <struct_bylighting@yahoo.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//****************************************************************************

#include <qapplication.h>
#include <qdesktopwidget.h>
#include <qtranslator.h>
#include <qtextcodec.h>
#include "spell.h"
#include "qwerty.h"
#include "dialogs.h"
#include "diffdialog.h"
#include "tstring.h"

void clearGlobal() {
	if (Global::instance()->sd != 0)
		delete Global::instance()->sd;
	if (Global::instance()->gd != 0)
		delete Global::instance()->gd;
#ifndef ROSPELL_NO_DIFF
	if (Global::instance()->ddd != 0)
		delete Global::instance()->ddd;
#endif
}

static void usage(TString &ts) {
	printf("%s\n", (const char *) ts.help);
}


static Editor *start(const char *fname, FILE_FORMAT format) {
	LOG("start");
	Editor *e = new Editor();
	e->setDefaultFormat(format);
	if (fname == 0) {
		LOG("start with no file");
		e->setFname(0, format);
	} else {
		e->load(fname);
	}

	bool isSmall =  qApp->desktop()->size().width() < 450
				 || qApp->desktop()->size().height() < 450;

	if (isSmall) {
		e->showMaximized();
	} else {
		LOG("main: set editor sizes %d %d", Global::instance()->xsize, Global::instance()->ysize);
//		e->resize(Global::instance()->xsize, Global::instance()->ysize);
		e->show();
	}

	return e;
}

int main( int argc, char **argv )
{
	Action *aptr = 0;
	int start_arg = 1;
	LOG("argc %d", argc);
	LOG("argv %s %s", argv[0], argv[1]);
	FILE_FORMAT format = FILE_UTF8;

	QApplication a( argc, argv);
	QTranslator translator1(0);
	QTranslator translator2(0);
	QString locale_id = QTextCodec::locale();
	locale_id.truncate(2);
	// translations are searched starting with the last file added
	QString translation_fname1 = QString(INSTALL_PLACE) + QString("qt_") + locale_id;
	QString translation_fname2 = QString(INSTALL_PLACE) + QString("rospell_") + locale_id;

	bool loaded1 = translator1.load(translation_fname1, ".");
	bool loaded2 = translator2.load(translation_fname2, ".");
	a.installTranslator(&translator1);
	a.installTranslator(&translator2);
	LOG("loaded %s translation %s", (loaded1)? "yes": "no", (const char *) translation_fname1);
	LOG("loaded %s translation %s", (loaded2)? "yes": "no", (const char *) translation_fname2);
	TString ts;

	if (argc > 1) {
		FILE_FORMAT f;
		if ((f = ff_parse_arg(argv[1])) != FILE_FORMAT_MAX) {
			LOG("main: option %s", argv[1]);
			format = f;
			start_arg = 2;
		} else if (strcmp(argv[1], "-u") == 0) {
			aptr = new ActionAddAllUser();
			Global::instance()->addalluser = true;
			start_arg = 2;
			if (argc <= 2) {
				usage(ts);
				return 1;
			}
		} else if (strcmp(argv[1], "-p") == 0) {
			aptr = new ActionDumpAll();
			Global::instance()->dumpall = true;
			start_arg = 2;
			if (argc <= 2) {
				usage(ts);
				return 1;
			}
		} else if (strcmp(argv[1], "-h") == 0 ||
				 strcmp(argv[1], "-?") == 0 ||
				 strcmp(argv[1], "--help") == 0) {
			usage(ts);
			return 0;
		} else if (strcmp(argv[1], "-v") == 0) {
			printf("%s %s\n", (const char *) ts.version, VERSION);
			return 0;
		}
	}
	if (aptr != 0) {
		// dumpall (-p) will load only the master dictionary, no local dictionary save
		// addalluser (-u) will load the master and the user dictionary, local dictionary save
		Dictionaries *pdict;
		if (Global::instance()->dumpall) {
			fprintf(stderr, ts.loading1);
			pdict = new Dictionaries(0, true, false, false); // no local read and save
		}
		else if (Global::instance()->addalluser) {
			fprintf(stderr, ts.loading2);
			pdict = new Dictionaries(0);
		}
		else {
			ASSERT(0);
			return 1;
		}

		for (int i = start_arg; i < argc; i++) {
			char *fstr = UTILS::read_file_malloc(argv[i]);
			if (fstr == 0)
				continue;
			fprintf(stderr, ts.processing, argv[i]);
			LOG("Process file %s", argv[i]);
			char *ptmp = spell(pdict, aptr, fstr);
			delete[] ptmp;
		}


		delete aptr;
		delete pdict;

		Stats::instance()->print();
		Stats::instance(true);
		if (Global::instance()->addalluser || Global::instance()->dumpall) {
			return 0;
		}
	}


	int i;
	Editor *e = 0;
	if (argc == start_arg) {
		e = start(0, format);
		e->setCaption(/*ts.empty_fname*/"empty buffer");
	}
	else {
		e = start(argv[start_arg], format);
		start_arg++;
		for (i= start_arg; i<argc; i++) {
			e->load(argv[i]);
		}
	}
	if (e == 0) // just in case!
		return 1;
	a.connect( &a, SIGNAL(lastWindowClosed()), &a, SLOT(quit()) );

	int retval = a.exec();
	delete e;
	Global::instance(true);
	Stats::instance(true);
	return retval;
//    return a.exec();
}

