//****************************************************************************
// Copyright (C) 2005,  Lucian Constantin <struct_bylighting@yahoo.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//*****************************************************************************/
// rospell user interface
//*****************************************************************************/

#include "qwerty.h"
#include "briefedit.h"
#include "tstring.h"
#include <qtextedit.h>
#include <qregexp.h>
#include <qtextcodec.h>

static QString HSTART="<html><head></head><body>";
static QString HEND="</body></html>";

static QString get_handbook() {
	static QString handbook = "";

	if (handbook.isEmpty()) {
		// get handbook fname based on locale
		QString locale_id = QTextCodec::locale();
		locale_id.truncate(2);
		locale_id = locale_id.upper();
		QString handbook_fname = QString(INSTALL_PLACE) + QString("HANDBOOK-") + locale_id;
		if (locale_id == "C")
			handbook_fname = QString(INSTALL_PLACE) + QString("HANDBOOK");
		// read the handbook
		LOG("try to open %s", (const char *) handbook_fname);
		char *h = UTILS::read_file_malloc((const char *) handbook_fname);
		if (h == 0) {
			handbook_fname = QString(INSTALL_PLACE) + QString("HANDBOOK");
			LOG("try to open %s", (const char *) handbook_fname);
			h = UTILS::read_file_malloc((const char *) handbook_fname);
			if (h == 0) {
				LOG("Couldn't open HANDBOOK\n");
				TString ts;
				QString errstr = QString(ts.err1) + QString(ts.err2);
				return HSTART + errstr + HEND;
			}
		}

		// convert the handbook in html
		handbook = QString::fromUtf8(h);
		delete [] h;
		handbook = HSTART + handbook + HEND;
		handbook = handbook.replace(QRegExp("\n\n"), "<br><br>\n");
		// chapter titles
		handbook = handbook.replace(QRegExp("==1. "), "<h2><a name=\"1\"></a>1. ");
		handbook = handbook.replace(QRegExp("==2. "), "<h2><a name=\"2\"></a>2. ");
		handbook = handbook.replace(QRegExp("==3. "), "<h2><a name=\"3\"></a>3. ");
		handbook = handbook.replace(QRegExp("==4. "), "<h2><a name=\"4\"></a>4. ");
		handbook = handbook.replace(QRegExp("==5. "), "<h2><a name=\"5\"></a>5. ");
		handbook = handbook.replace(QRegExp("==6. "), "<h2><a name=\"6\"></a>6. ");
		handbook = handbook.replace(QRegExp("==7. "), "<h2><a name=\"7\"></a>7. ");
		handbook = handbook.replace(QRegExp("==8. "), "<h2><a name=\"8\"></a>8. ");
		handbook = handbook.replace(QRegExp("\n=="), "<h2>");
		handbook = handbook.replace(QRegExp("=="), "</h2>");
		handbook = handbook.replace(QRegExp("\n "), "\n<br><br>&nbsp;&nbsp;&nbsp;&nbsp;");
		handbook = handbook.replace(QRegExp("<br />"), "<br>");
		handbook = handbook.replace(QRegExp("<br/>"), "<br>");

	}
	return handbook;
}

static void show_help(QFont f, QString anchor) {
	QTextEdit *qb = new QTextEdit(0, "help browser");
	qb->setText(get_handbook());
	qb->setReadOnly(true);
	qb->scrollToAnchor(anchor);
	qb->setCaption("Handbook"); //lucian: tr doesn't work
	qb->setPaletteBackgroundColor(Global::instance()->c_background);
	qb->setPaletteForegroundColor(Global::instance()->c_foreground);
	QPalette qp = qb->palette();
	qp.setColor(QPalette::Active, QColorGroup::Highlight, Global::instance()->c_highlight);
	qp.setColor(QPalette::Inactive, QColorGroup::Highlight, Global::instance()->c_highlight);
	qb->setPalette(qp);
	qb->setFont(f);
	qb->resize(400, 300);
	qb->show();
}

void Editor::addHelpMenu() {
	ASSERT(m_ != 0);

	Q3PopupMenu * help_ = new Q3PopupMenu(m_);
	Q_CHECK_PTR( help_ );
	m_->insertItem( tr("&Help"), help_);

	help_->insertItem( tr("&Getting Started"), this, SLOT(helpGettingStarted()));
	help_->insertItem( tr("&Basic Operation"), this, SLOT(helpBasicOperation()));
	help_->insertItem( tr("&Programming with rospell"), this, SLOT(helpProgramming()));
	help_->insertItem( tr("&Keyboard Shortcuts"), this, SLOT(helpKeyboard()));
	help_->insertSeparator();
	help_->insertItem( tr("&About"), this, SLOT(helpAbout()));
	help_->insertItem( tr("About &Qt"), this, SLOT(helpAboutQt()));
}

void Editor::helpAbout() {
	show_help(e_->font(), "1");
}

void Editor::helpAboutQt() {
	QMessageBox::aboutQt( this, "rospell/Qt" );
}

void Editor::helpGettingStarted() {
	show_help(e_->font(), "2");
}


void Editor::helpBasicOperation() {
	show_help(e_->font(), "3");
}

void Editor::helpKeyboard() {
	show_help(e_->font(), "4");
}

void Editor::helpProgramming() {
	show_help(e_->font(), "5");
}


