//****************************************************************************
// Copyright (C) 2005,  Lucian Constantin <struct_bylighting@yahoo.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//****************************************************************************
//** $Id: qt/qwerty.cpp   3.3.4   edited May 27 2003 $
//** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
//**
//** This file is part of an example program for Qt.  This example
//** program may be used, distributed and modified without limitation.
//**
//*****************************************************************************/
// rospell user interface
//*****************************************************************************/

// for development/debug purposes only
#define WORK_DEBUG

#include "qwerty.h"
#include <qapplication.h>
#include <qdesktopwidget.h>
#include <qdatetime.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qpainter.h>
#include <qmessagebox.h>
#include <qfontdialog.h>
#include <qtextcodec.h>
#include "busycursor.h"
#include <qdir.h>

#ifdef USING_QT4
#include <q3filedialog.h>
#include <q3popupmenu.h>
#include <q3paintdevicemetrics.h>
#include <q3ptrlist.h>
//Added by qt3to4:
#include <Q3CString>
#include <QCloseEvent>
#include <QResizeEvent>
#include <Q3VBoxLayout>
//Added by qt3to4:
#include <QLabel>
#endif
#ifdef USING_QT3
#include <qfiledialog.h>
#include <qpaintdevicemetrics.h>
#endif
#include <qfileinfo.h>
#include <qinputdialog.h>
#include <qsplitter.h>
#include <qcombobox.h>
#include <qregexp.h>

#include "qtutils.h"
#include "spell.h"
#include "stats.h"
#include "dialogs.h"
#include "briefedit.h"
#include "gdbedit.h"
#include "statusfield.h"

#include "latex.h"
#include "bufmgr.h"
#include "ctags.h"
#include "gdb.h"
#include "filevisitor.h"

#ifdef ROSPELL_CYGWIN
extern char *strcasestr (const char *phaystack, const char *pneedle);
#endif


void Editor::setDirfiles(QString dfiles) {
	if (dfiles.isNull() || dfiles.isEmpty()) {
printf("*****empty dirfiles*****\n");
printf("dirfiles_path_: #%s#\n", (const char *) dirfiles_path_);
}
	else
		dirfiles_->setText(dfiles);
}

void Editor::updateDirpath(QString filename) {

	if (filename.startsWith("*")) // this is the case when after Alt-n the new file was modified
		filename = filename.mid(1);

	// update dirfiles_path_
	QFileInfo qf1(filename);
	QString fn = qf1.fileName();
	QString qf = qf1.filePath();
	if (qf.isNull() || qf.isEmpty())
printf("***********Editor::updateDirpath******************\nempty path #%s#\n", (const char *)dirfiles_path_);
	else {
		dirfiles_path_ = qf1.filePath();
		dirfiles_path_ = dirfiles_path_.left(dirfiles_path_.length() - fn.length());
		QString dfiles = getDirfiles();
		setDirfiles(dfiles);
	}

}


void Editor::load() {
	QString fn = Q3FileDialog::getOpenFileName( QString::null, QString::null, this );
	if (!fn.isEmpty()) {
		msgScreen(tr("loading file, please wait..."));
		load(fn);
	}
}


void Editor::load(const QString& fileName, bool absolute) {
	(void) absolute;
	// save curent buffer first
	storeCurentBuffer();
	// move to an absolute file name
	QFileInfo qf(fileName);
	// check file name; if it is a name of a directory put a warning on the screen and return
	QString chk_dir = qf.fileName();
	if (qf.isDir() || chk_dir.isNull() || chk_dir == "." || chk_dir == "..") {
		QString msg = QString(tr("\nFile %1 seems to be\na directory and will not be loaded!\n")).arg(fileName);
		QMessageBox::warning(this,tr("I/O Info"), msg);
		return;
	}

	BCursor busy;
	msgScreen(tr("loading file, please wait..."));
	QString newfileName = qf.absFilePath();

	// update dirfiles_path_
	updateDirpath(newfileName);

	// check if the file is already loaded
	int index = 0;
	bool foundit = b_->hasFile(newfileName, &index);
	if (foundit) {
		pickBuffer(index + 1);
		msgScreen(tr("done"), 2000);
		return;
	}

	// check if the file is not loaded in the inactive buffer manager
	BufMgr *b1 = (using_aux_)? b_main_: b_aux_;
	BriefEdit *e1 = (using_aux_)? emain_: eaux_;
	foundit = b1->hasFile(newfileName, &index);
	if (foundit) {
		pickBuffer(b1, e1, index + 1);
		return;
	}

	if (using_aux_)
		changed_aux_ = true;
	else
		changed_main_ = true;
	Q3CString qfname = newfileName.utf8();
	const char *fname = qfname;
	LOG("Editor::load %s %d", fname, (int) default_format_);
	e_->setAutoUpdate( FALSE );
	// lucian: is the format correct?
	setFname(newfileName, default_format_, false);

	QFile f( fileName );
	if (QTUTILS::openFileReadOnly(f)) {
		QTextStream t(&f);
		if (default_format_ == FILE_FORMAT_MAX) {
			ASSERT(0);
			QMessageBox::warning(this,tr("I/O Info"),
				QString(tr("encoding not found; file not loaded")));
			return;
		}
		else if (default_format_ == FILE_UTF8) {
			t.setEncoding( QTextStream::UnicodeUTF8 );
			LOG("Editor::load utf8");
		}
		else {
			QTextCodec *codec = QTextCodec::codecForName(ff_str(default_format_));
			t.setCodec(codec);
			LOG("Editor::load %s", ff_str(default_format_));
		}

		e_->setText( t.read() );
		f.close();
	}					  // if file open
	else {
		e_->setText("");
	}

	// backup
	QString fname_only = qf.fileName();
	LOG("about to open file %s", (const char *) fname_only);
	if (Global::instance()->no_backup_ == false) {
		char *backup_fname = Global::getBackupFname((const char *) qf.fileName());
		bool noerr = saveCommonText(QString(backup_fname), default_format_, e_->text());
		if (noerr == FALSE) {
			// error recovery: create backup directory
			Global::buildDirs();
			QMessageBox::warning(this,tr("I/O Info"),
				QString(tr("Backup directory created\n")));
		}
		delete[] backup_fname;
	}
	// if this is a latex file, convert it to utf8 for editing purposes
	if (fname_only.endsWith(".tex")) {
		QString tmp = latex2utf8(e_->text());
		e_->setText(tmp);
	}

	e_->setAutoUpdate( TRUE );
	e_->repaint();
	if (using_aux_)
		changed_aux_ = false;
	else
		changed_main_ = false;
	cleanMWords();

	// set buffer
	b_->add(newfileName, default_format_);
	b_->setActiveText(e_->text());
	e_->setCursorPosition(0, 0, false);
	redoBufMgrList();
	msgScreen(tr("file loaded"), 5000);
	dirboxChanged(1);
	// this is load, adding history file at line 0
	Global::instance()->addHistoryFile(newfileName, -1);
	updateRecent();
}


bool Editor::save() {
	bool retval = false;
	QString fname = getFname();
	if (fname.isNull() || fname == "empty buffer")
		retval = saveAs();
	else {
		BCursor busy;
		Q3CString utext = fname.utf8();
		const char *cstr = utext;
		LOG("Editor::save file %s", cstr);
		msgScreen(tr("saving file, please wait..."));
		if (!fname.isEmpty())
			retval = saveCommon(fname, b_->getActiveFormat());
	}
	msgScreen(tr("file saved"), 5000);
	e_->update();
	return retval;
}


bool Editor::saveAs() {
	msgScreen(tr("saving file, please wait..."));
	// name change in bufmgr is done in saveComon

	//storing filename (proper save) is left as an exercise...
	QString fn = Q3FileDialog::getSaveFileName( QString::null, QString::null, this );
	BCursor busy;
	if (!fn.isEmpty()) {
		bool retval = saveCommon(fn, b_->getActiveFormat());
		// update dirfiles_path_
		updateDirpath(fn);
		return retval;
	}

	msgScreen("");
	return FALSE;
}


bool Editor::saveCommonText(const QString& fileName, FILE_FORMAT format, QString text) {
	QFile f( fileName );
	if (!QTUTILS::openFileWriteOnly(f)) {
		QMessageBox::warning(this,tr("I/O Error"),
			QString(tr("Cannot open file for writing:\n\n"))
			+fileName);
		return FALSE;
	}
	QTextStream t(&f);
	LOG("save common format %d", format);

	if (format == FILE_FORMAT_MAX) {
		ASSERT(0);
		QMessageBox::warning(this,tr("I/O Error"),
			QString(tr("File not saved\n\n"))
			+fileName);
		return FALSE;
	}
	else if (format == FILE_UTF8) {
		t.setEncoding( QTextStream::UnicodeUTF8 );
		LOG("Editor::saveCommonBuffer utf8");
	}
	else {
		QTextCodec *codec = QTextCodec::codecForName(ff_str(format));
		t.setCodec(codec);
		LOG("Editor::saveCommonBuffer %s", ff_str(format));
	}

	QString tmp;
	// if this is a latex file save it in latex/ascii format
	if (fileName.endsWith(".tex"))
		tmp = utf82latex(text);
	else
		tmp = text;
	tmp = QTUTILS::cleanBookmarkers(tmp);
	if (Global::instance()->strip_trailing_)
		tmp = QTUTILS::stripTrailingSpaces(tmp);
	if (Global::instance()->unix_eol_)
		tmp = QTUTILS::unixEol(tmp);
	if (Global::instance()->tab2space_)
		tmp = QTUTILS::tab2space(tmp);
	t << tmp;
	f.close();
	dirboxChanged(1);
	return TRUE;
}


bool Editor::saveCommon(const QString& fileName, FILE_FORMAT format) {
	if (FALSE == saveCommonText(fileName, format, e_->text()))
		return FALSE;

	setFname(fileName, format);
	if (using_aux_)
		changed_aux_ = false;
	else
		changed_main_ = false;
	cleanMWords();
	bool isnew = b_->setActiveFName(fileName);
	if (isnew)
		b_->add(fileName, format);
	b_->setActiveText(e_->text());
	int line;
	int col;
	e_->getCursorPosition(&line, &col);
	b_->setActiveLine(line);
	b_->setActiveCol(col);
	redoBufMgrList();
	msgScreen(tr("file saved"), 5000);
	e_->syntaxHighlighter()->rehighlight();
	e_->setFocus();
	dirboxChanged(1);

	// save puts the file at the top of the history list, with the line number
	Global::instance()->addHistoryFile(fileName, line);
	return TRUE;
}

// lucian: more work on format detection here
FILE_FORMAT Editor::detectWwwFormat(const char *str) {
	ASSERT(str != 0);
	FILE_FORMAT retval = FILE_UTF8;

#ifndef ROSPELL_NO_WGET
	char *ptr = (char *) strcasestr(str, "<head>");
	if (ptr == 0)
		return retval;

	char *ptr1 = ptr;
	if ((ptr = strcasestr(ptr, "charset=iso-8859-1")) != 0 && (ptr = strcasestr(ptr, "</head>")) != 0)
		return FILE_ISO8859_1;

	ptr = ptr1;
	if ((ptr = strcasestr(ptr, "charset=iso-8859-2")) != 0 && (ptr = strcasestr(ptr, "</head>")) != 0)
		return FILE_ISO8859_2;

	ptr = ptr1;
	if ((ptr = strcasestr(ptr, "charset=iso-8859-3")) != 0 && (ptr = strcasestr(ptr, "</head>")) != 0)
		return FILE_ISO8859_3;

	ptr = ptr1;
	if ((ptr = strcasestr(ptr, "charset=iso-8859-4")) != 0 && (ptr = strcasestr(ptr, "</head>")) != 0)
		return FILE_ISO8859_4;

	ptr = ptr1;
	if ((ptr = strcasestr(ptr, "charset=iso-8859-5")) != 0 && (ptr = strcasestr(ptr, "</head>")) != 0)
		return FILE_ISO8859_5;

	ptr = ptr1;
	if ((ptr = strcasestr(ptr, "charset=iso-8859-6")) != 0 && (ptr = strcasestr(ptr, "</head>")) != 0)
		return FILE_ISO8859_6;

	ptr = ptr1;
	if ((ptr = strcasestr(ptr, "charset=iso-8859-7")) != 0 && (ptr = strcasestr(ptr, "</head>")) != 0)
		return FILE_ISO8859_7;

	ptr = ptr1;
	if ((ptr = strcasestr(ptr, "charset=iso-8859-8")) != 0 && (ptr = strcasestr(ptr, "</head>")) != 0)
		return FILE_ISO8859_8;

	ptr = ptr1;
	if ((ptr = strcasestr(ptr, "charset=iso-8859-9")) != 0 && (ptr = strcasestr(ptr, "</head>")) != 0)
		return FILE_ISO8859_9;

	ptr = ptr1;
	if ((ptr = strcasestr(ptr, "charset=iso-8859-10")) != 0 && (ptr = strcasestr(ptr, "</head>")) != 0)
		return FILE_ISO8859_10;

	ptr = ptr1;
	if ((ptr = strcasestr(ptr, "charset=iso-8859-11")) != 0 && (ptr = strcasestr(ptr, "</head>")) != 0)
		return FILE_ISO8859_11;

	ptr = ptr1;
	if ((ptr = strcasestr(ptr, "charset=iso-8859-12")) != 0 && (ptr = strcasestr(ptr, "</head>")) != 0)
		return FILE_ISO8859_12;

	ptr = ptr1;
	if ((ptr = strcasestr(ptr, "charset=iso-8859-13")) != 0 && (ptr = strcasestr(ptr, "</head>")) != 0)
		return FILE_ISO8859_13;

	ptr = ptr1;
	if ((ptr = strcasestr(ptr, "charset=iso-8859-14")) != 0 && (ptr = strcasestr(ptr, "</head>")) != 0)
		return FILE_ISO8859_14;

	ptr = ptr1;
	if ((ptr = strcasestr(ptr, "charset=iso-8859-15")) != 0 && (ptr = strcasestr(ptr, "</head>")) != 0)
		return FILE_ISO8859_15;

	ptr = ptr1;
	if ((ptr = strcasestr(ptr, "charset=iso-8859-16")) != 0 && (ptr = strcasestr(ptr, "</head>")) != 0)
		return FILE_ISO8859_16;

	ptr = ptr1;
	if ((ptr = strcasestr(ptr, "charset=windows-1250")) != 0 && (ptr = strcasestr(ptr, "</head>")) != 0)
		return FILE_CP1250;
#endif
	return retval;
}


void Editor::openWwwLocation() {
#ifndef ROSPELL_NO_WGET
						  // the text has been changed
	if (using_aux_ && changed_aux_ || (!using_aux_) && changed_main_) {
		switch (QMessageBox::warning( this, tr("Question"),
			tr("Save changes to Document?"),
			tr("&Yes"),
			tr("&No"),
			tr("Cancel"),
		0, 2)) {
			case 0:			  // yes
				save();
				break;
			case 1:			  // no
				break;
			default:		  // cancel
				return;
				break;
		}
	}

	// check for wget presence
	if (!Global::checkWhich("wget"))
		return;

	WwwDialog *wd = new WwwDialog();
	wd->setCaption(tr("URL"));
	wd->exec();
	QString sword = wd->getWord();

	if (sword == "")
		return;
	Q3CString utext = sword.utf8();
	const char *cstr = utext;
	LOG("Editor::openWwwLocation  %s", cstr);
	delete wd;
	msgScreen(tr("Loading..."));
	QApplication::setOverrideCursor( QCursor(Qt::WaitCursor) );

	char cmd[1000];
	snprintf(cmd, 1000, "wget --timeout=60 --tries=1 -q --output-document=- %s", cstr);
	char *www_content = UTILS::run_prg_malloc(cmd);
	QApplication::restoreOverrideCursor();
	if (www_content != 0) {
		FILE_FORMAT format = detectWwwFormat(www_content);
		QString txt = ff_toUtf8(www_content, format);
		e_->setText(txt);
		sword.replace(QChar('/'), "_");
		setFname(sword, format);
		msgScreen(ff_str(format), 5000);
		delete[] www_content;
		if (using_aux_)
			changed_aux_ = false;
		else
			changed_main_ = false;
		b_->add(sword, format);
		b_->setActiveText(txt);
		int line;
		int col;
		e_->getCursorPosition(&line, &col);
		b_->setActiveLine(line);
		b_->setActiveCol(col);
		e_->setCursorPosition(1, 1, false);

		redoBufMgrList();
		msgScreen(tr("file loaded"), 5000);
	}
	else {
		QMessageBox::warning(this,tr("I/O Info"),
			QString(tr("Failed to load the file")));
	}
#endif
}

void Editor::openRecent0() {
	Global::HistEntry *hentry = Global::instance()->getHistoryFile(0);
	QString fname = hentry->name;
	if (fname.isEmpty()) {
		msgScreen(tr("Cannot open file"), 5000);
		return;
	}

	load(fname);
	e_->setCursorPosition(hentry->line, 0, false);
}

void Editor::openRecent1() {
	Global::HistEntry *hentry = Global::instance()->getHistoryFile(1);
	QString fname = hentry->name;
	if (fname.isEmpty()) {
		msgScreen(tr("Cannot open file"), 5000);
		return;
	}

	load(fname);
	e_->setCursorPosition(hentry->line, 0, false);
}
void Editor::openRecent2() {
	Global::HistEntry *hentry = Global::instance()->getHistoryFile(2);
	QString fname = hentry->name;
	if (fname.isEmpty()) {
		msgScreen(tr("Cannot open file"), 5000);
		return;
	}

	load(fname);
	e_->setCursorPosition(hentry->line, 0, false);
}
void Editor::openRecent3() {
	Global::HistEntry *hentry = Global::instance()->getHistoryFile(3);
	QString fname = hentry->name;
	if (fname.isEmpty()) {
		msgScreen(tr("Cannot open file"), 5000);
		return;
	}

	load(fname);
	e_->setCursorPosition(hentry->line, 0, false);
}
void Editor::openRecent4() {
	Global::HistEntry *hentry = Global::instance()->getHistoryFile(4);
	QString fname = hentry->name;
	if (fname.isEmpty()) {
		msgScreen(tr("Cannot open file"), 5000);
		return;
	}

	load(fname);
	e_->setCursorPosition(hentry->line, 0, false);
}
void Editor::openRecent5() {
	Global::HistEntry *hentry = Global::instance()->getHistoryFile(5);
	QString fname = hentry->name;
	if (fname.isEmpty()) {
		msgScreen(tr("Cannot open file"), 5000);
		return;
	}

	load(fname);
	e_->setCursorPosition(hentry->line, 0, false);
}

void Editor::updateRecent() {
	// only max_hlist are saved
	list<Global::HistEntry*>::iterator it;
	QString str = "";
	int i = 0;
	for (it = Global::instance()->hlist.begin(); it != Global::instance()->hlist.end(); ++it) {
		if (i >= Global::instance()->max_hist_)
			break;
		Global::HistEntry he = *(*it);
		QString h = he.name;
		if (h.length() > 25) {
			h = "..." + h.right(25);
		}
		if (i == 0) {
			file_->changeItem(rc0_, h);
			file_->setItemVisible(rc0_, true);
		}
		else if (i == 1) {
			file_->changeItem(rc1_, h);
			file_->setItemVisible(rc1_, true);
		}
		else if (i == 2) {
			file_->changeItem(rc2_, h);
			file_->setItemVisible(rc2_, true);
		}
		else if (i == 3) {
			file_->changeItem(rc3_, h);
			file_->setItemVisible(rc3_, true);
		}
		else if (i == 4) {
			file_->changeItem(rc4_, h);
			file_->setItemVisible(rc4_, true);
		}
		else if (i == 5) {
			file_->changeItem(rc5_, h);
			file_->setItemVisible(rc5_, true);
		}
		i++;
	}

	for (; i < Global::instance()->max_hist_; i++) {
		if (i == 0) {
			file_->setItemVisible(rc0_, false);
		}
		else if (i == 1) {
			file_->setItemVisible(rc1_, false);
		}
		else if (i == 2) {
			file_->setItemVisible(rc2_, false);
		}
		else if (i == 3) {
			file_->setItemVisible(rc3_, false);
		}
		else if (i == 4) {
			file_->setItemVisible(rc4_, false);
		}
		else if (i == 5) {
			file_->setItemVisible(rc5_, false);
		}
	}
}
