//****************************************************************************
// Copyright (C) 2005,  Lucian Constantin <struct_bylighting@yahoo.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//****************************************************************************
// Spelling menu implementation
//****************************************************************************

#include "qwerty.h"
#include "spell.h"
#include "briefedit.h"
#include "busycursor.h"
#include "qtutils.h"
#include "progressbar.h"
#include <qregexp.h>
#ifdef USING_QT4
#include <Q3CString>
#endif

// extract word from position inside line
static QString extract_word(QString str) {
	QChar qc = str[0];
	if (!qc.isLetterOrNumber())
		return 0;
	int i = 0;
	while (qc.isLetterOrNumber() || qc == '-' || qc == '\'') {
		i++;
		qc = str[i];
	}

	str.truncate(i);
	return str;
}


void Editor::advanceStartWord() {
//	LOG("enter advanceStartWord function with index %d", x_);
	int lines = e_->numLines();
//	LOG("lines %d", lines);

	while (1) {
		QChar ch = line_.at(x_);
		for (int i = 0; !ch.isNull(); i++, x_++) {
			ch = line_.at(x_);
			if (ch.isLetterOrNumber() || ch == '<' || ch == '/' || ch == '&') { // check for utf8 also
//				LOG("return advanceStartWord function with index %d", x_);
				return;
			}
		}

		if ((y_ + 1) >= lines) {
//			LOG("advanceStartWord advance end of file detected");
			e_->setCursorPosition(y_, x_, false);
			break;
		}

		y_++;
		x_ = 0;
		line_ = e_->textLine(y_);
		e_->setCursorPosition(y_, x_, false);
//		LOG("advanceStartWord advance para %d, index %d", y_, x_);
	}
}

QString Editor::advanceEndWord() {
	if (e_->atEnd())
		return QString("");

	xword_ = x_;
//	LOG("enter advanceEndWord with index %d", x_);

	QChar ch = line_.at(x_);
	int i;
	for (i = 0; !ch.isNull() && (ch.isLetterOrNumber() || ch =='-' || ch == '\''); i++) {
		x_++;
		ch = line_.at(x_);
	}
	if (i == 0) {
//		LOG("return from 0");
		return QString("");
	}


//	LOG("advanceWord selection %d/%d %d/%d", y_, xword_, y_, x_);
	//lucian: it might return all string!!!
	QString retval = line_.mid(xword_, x_ - xword_);
	if (retval.isNull()) {
//		LOG("return isNull");
		return QString("");
	}

//	LOG("advanceEndWord %s", (const char *)retval.utf8());
//	LOG("exit advanceEndWord with index %d", x_);
	return retval;
}


bool Editor::skeepXML() {
	// save the status in case we run into a non-terminated xml tag
	int tmp_x = x_;
	int tmp_y = y_;

//	LOG("enter skeepXML with index %d", x_);
	int lines = e_->numLines();
	LOG("lines %d", lines);
	QChar ch = line_.at(x_);

	if (ch == '<') {
//		LOG("Editor::skeepXML - xml detected");
		while (1) {
			x_++;
			ch = line_.at(x_);
			if (ch == '>')
				break;
			if (ch.isNull()) {
				// advance here to a new line
				if ((y_ + 1) > lines) {
					x_ = tmp_x + 1;
					y_ = tmp_y;
					line_ = e_->textLine(y_);
//					LOG("advanceXML end of file detected");
					e_->setCursorPosition(y_, x_, false);
					return true;
				}
				y_++;
				x_ = 0;
				line_ = e_->textLine(y_);
//				LOG("skeepXML sml advance para %d, index %d", y_, x_);
			}
		}

//		LOG("exit skeepXML with index %d", x_);
		return true;
	} else if (ch == '/' || ch == '&') {
//		LOG("Editor::skeepXML - path detected");
		x_++;
		QChar ch = line_.at(x_);
		while (ch.isLetterOrNumber()) {
			x_++;
			ch = line_.at(x_);
		}
//		LOG("exit skeepXML path with index %d", x_);
		return true;
	}


//	LOG("exit skeepXML with index %d", x_);
	return false;

}

void Editor::spellAdvance()
{
	if (e_->atEnd()) {
		e_->selectAll(true);
		msgScreen(tr("end of file"));
		return;
	}

//	e_->moveCursor(QTextEdit::MoveForward, true);
//	e_->moveCursor(QTextEdit::MoveBackward, true);

	e_->getCursorPosition(&y_, &x_);
	line_ = e_->textLine(y_);

	// just in case the cursor is in the middle of a word
	advanceEndWord();

	if (pdict_ == 0)
		initDictionary();

	while (1) {
		e_->selectAll(false);
		// advance to a new word
		advanceStartWord();
		if (e_->atEnd()) {
			cleanMWords();
			msgScreen(tr("end of file"));
			return;
		}

		// skeep XML markup
		if (skeepXML())
			continue;

		//e->cursorWordForward(true);
		QString text = advanceEndWord();
// hasnumeric added here
		if (text.isEmpty() || text.isNull() || hasnumeric(text))
			continue;

		QString extracted = extract_word(text);

		Q3CString utext = extracted.utf8();
		const char *ctext = utext;
		char *active_word = (char *) ctext;

		if (active_word == 0) {
			LOG("couldn't extract word");
			// lucian to fix1: we might get here because the word started with an
			// utf8 char that is not a valid word char
			// just jump over this char for now
			e_->moveCursor(Q3TextEdit::MoveForward, false);
			continue;
		}

		LOG("text active:%s", active_word);
		if (!pdict_->findString(active_word)) {
			// not found
			Stats::instance()->notFound();
			cleanMWords();
			// skeep lookalike for words consisting in a single letter
			if (strlen(active_word) == 1) {
				break;
			}
			doLookalike(active_word);
			break;
		} else
			Stats::instance()->found();
	}

	e_->setCursorPosition(y_, x_, false);
	e_->setSelection(y_, xword_, y_, x_);
}


void Editor::cleanMWords() {
	mwords_->changeItem(mw1_, QString(" "));
	mwords_->changeItem(mw2_, QString(" "));
	mwords_->changeItem(mw3_, QString(" "));
	mwords_->changeItem(mw4_, QString(" "));
	mwords_->changeItem(mw5_, QString(" "));
	*word1_ = '\0';
	*word2_ = '\0';
	*word3_ = '\0';
	*word4_ = '\0';
	*word5_ = '\0';
}

void Editor::replaceWord1() {
	if (*word1_ == ' ' || *word1_ == '\0')
		return;
	LOG("replaceWord1 %s", word1_);
	char word[100];
	strcpy(word, word1_);
	e_->cut();
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = QString::fromUtf8(word);
	e_->insertAt(str, para, index, false);
	spellAdvance();
}

void Editor::replaceWord2() {
	if (*word2_ == ' ' || *word2_ == '\0')
		return;
	LOG("replaceWord2 %s", word2_);
	char word[100];
	strcpy(word, word2_);
	e_->cut();
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = QString::fromUtf8(word);
	e_->insertAt(str, para, index, false);
	spellAdvance();
}

void Editor::replaceWord3() {
	if (*word3_ == ' ' || *word3_ == '\0')
		return;
	LOG("replaceWord3 %s", word3_);
	char word[100];
	strcpy(word, word3_);
	e_->cut();
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = QString::fromUtf8(word);
	e_->insertAt(str, para, index, false);
	spellAdvance();
}

void Editor::replaceWord4() {
	if (*word4_ == ' ' || *word4_ == '\0')
		return;
	LOG("replaceWord4 %s", word4_);
	char word[100];
	strcpy(word, word4_);
	e_->cut();
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = QString::fromUtf8(word);
	e_->insertAt(str, para, index, false);
	spellAdvance();
}

void Editor::replaceWord5() {
	if (*word5_ == ' ' || *word5_ == '\0')
		return;
	LOG("replaceWord5 %s", word5_);
	char word[100];
	strcpy(word, word5_);
	e_->cut();
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = QString::fromUtf8(word);
	e_->insertAt(str, para, index, false);
	spellAdvance();
}

void Editor::spellIgnore()
{
	QString text = e_->markedText();
	QString extracted = extract_word(text);
	if (extracted.isEmpty()) {
		text = text.mid(1);
		extracted = extract_word(text);
	}
	if (extracted.isEmpty()) {
		spellAdvance();
		return;
	}
	Q3CString utext = extracted.utf8();
	const char *ctext = utext;
	char *active_word = (char *) ctext;
	pdict_->getIgnore()->setString(active_word, true);
	spellAdvance();
}

void Editor::spellAdd()
{
	QString text = e_->markedText();
	QString extracted = extract_word(text);
	if (extracted.isEmpty()) {
		text = text.mid(1);
		extracted = extract_word(text);
	}
	if (extracted.isEmpty()) {
		spellAdvance();
		return;
	}
	Q3CString utext = extracted.utf8();
	const char *ctext = utext;
	char *active_word = (char *) ctext;
	pdict_->getUser()->setString(active_word, true);
	spellAdvance();
}


void Editor::insertLookalike(const char *str) {
	ASSERT(str != 0);
	if (*word1_ == '\0') {
		strcpy(word1_, str);
		mwords_->changeItem(mw1_, QString::fromUtf8(word1_));
		return;
	}
	if (*word2_ == '\0' && strcmp(str, word1_) != 0) {
		strcpy(word2_, str);
		mwords_->changeItem(mw2_, QString::fromUtf8(word2_));
		return;
	}
	if (*word3_ == '\0' && strcmp(str, word1_) != 0 && strcmp(str, word2_) != 0) {
		strcpy(word3_, str);
		mwords_->changeItem(mw3_, QString::fromUtf8(word3_));
		return;
	}
	if (*word4_ == '\0' && strcmp(str, word1_) != 0 && strcmp(str, word2_) != 0 && strcmp(str, word3_) != 0) {
		strcpy(word4_, str);
		mwords_->changeItem(mw4_, QString::fromUtf8(word4_));
		return;
	}
	if (*word5_ == '\0' && strcmp(str, word1_) != 0 && strcmp(str, word2_) != 0 && strcmp(str, word3_) != 0 && strcmp(str, word4_) != 0) {
		strcpy(word5_, str);
		mwords_->changeItem(mw5_, QString::fromUtf8(word5_));
		return;
	}
}


void Editor::doLookalike(char *active_word) {
	if (pdict_ == 0)
		initDictionary();

	DbDict *hptr = pdict_->getMaster();
	char dest1[100];
	char dest2[100];
	char dest3[100];
	char dest4[100];
	char dest5[100];

	// Romanian specific suggestions
	Global *glb = Global::instance();
	if (glb->lang == LANG_RO_CEDILLA || glb->lang == LANG_RO_COMMABELOW) {
		if (strcmp((const char *) active_word, "Romania") == 0)
			insertLookalike("România");
		else if (strcmp((const char *) active_word, "Romaniei") == 0)
			insertLookalike("României");
	}

	// grab any valid words form dict based on active_words
	hptr->findLookalike(active_word, dest1, dest2, dest3, dest4, dest5);
	if (*dest1 != 0)
		insertLookalike(dest1);
	if (*dest2 != 0)
		insertLookalike(dest2);
	if (*dest3 != 0)
		insertLookalike(dest3);
	if (*dest4 != 0)
		insertLookalike(dest4);
	if (*dest5 != 0)
		insertLookalike(dest5);

	
	// add one more letter at the end of the word using the trylist
	if (pdict_->suggestAddEndTrylist(active_word, dest1, dest2, dest3)) {
		LOG("found suggestInsertTrylist");
		if (*dest1 != 0)
			insertLookalike(dest1);
		if (*dest2 != 0)
			insertLookalike(dest2);
		if (*dest3 != 0)
			insertLookalike(dest3);
	}
	// flip two chars
	if (pdict_->suggestFlip(active_word, dest1, dest2, dest3)) {
		LOG("found suggestFlip");
		if (*dest1 != 0)
			insertLookalike(dest1);
		if (*dest2 != 0)
			insertLookalike(dest2);
		if (*dest3 != 0)
			insertLookalike(dest3);
	}
	// remove extra letters in the word
	if (pdict_->suggestRemoveLetter(active_word, dest1, dest2, dest3)) {
		LOG("found suggestRemoveLetter");
		if (*dest1 != 0)
			insertLookalike(dest1);
		if (*dest2 != 0)
			insertLookalike(dest2);
		if (*dest3 != 0)
			insertLookalike(dest3);
	}
	// insert from trylist
	if (pdict_->suggestInsertTrylist(active_word, dest1, dest2, dest3)) {
		LOG("found suggestInsertTrylist");
		if (*dest1 != 0)
			insertLookalike(dest1);
		if (*dest2 != 0)
			insertLookalike(dest2);
		if (*dest3 != 0)
			insertLookalike(dest3);
	}
	// replace from trylist
	if (pdict_->suggestReplaceTrylist(active_word, dest1, dest2, dest3)) {
		LOG("found suggestInsertTrylist");
		if (*dest1 != 0)
			insertLookalike(dest1);
		if (*dest2 != 0)
			insertLookalike(dest2);
		if (*dest3 != 0)
			insertLookalike(dest3);
	}

	// if necessary change from cedilla in commabelow
	if (Global::instance()->lang == LANG_RO_COMMABELOW) {
		char *w = UTF8::strcedilla2commabelow(word1_);
		strcpy(word1_, w);
		mwords_->changeItem(mw1_, QString::fromUtf8(word1_));
		delete [] w;

		w = UTF8::strcedilla2commabelow(word2_);
		strcpy(word2_, w);
		mwords_->changeItem(mw2_, QString::fromUtf8(word2_));
		delete [] w;

		w = UTF8::strcedilla2commabelow(word3_);
		strcpy(word3_, w);
		mwords_->changeItem(mw3_, QString::fromUtf8(word3_));
		delete [] w;

		w = UTF8::strcedilla2commabelow(word4_);
		strcpy(word4_, w);
		mwords_->changeItem(mw4_, QString::fromUtf8(word4_));
		delete [] w;

		w = UTF8::strcedilla2commabelow(word5_);
		strcpy(word5_, w);
		mwords_->changeItem(mw5_, QString::fromUtf8(word5_));
		delete [] w;
	} 

	LOG("word1:%s", word1_);
	LOG("word2:%s", word2_);
	LOG("word3:%s", word3_);
	LOG("word4:%s", word4_);
	LOG("word5:%s", word5_);
}

// strings to be replaced by autospelling; using cedilla fromat
const char *autolist[] = {
	" Romania", " România",
	" sint ", " sunt ", // also deal with "academia romana" 1993 new style
	" si ", " şi ",
	"Si ", "Şi ",
	" in ", " în ",
	"In ", "În ",
	" ti ", " ţi ",
	" sa ", " să ",
	"Sa ", "Să ",
	" ma ", " mă ",
	"Ma ", "Mă ",
	" daca ", " dacă ",
	" cand ", " când ",
	" cind ", " când ",
	" cate ", " câte ",
	" dupa ", " după ",
	" parca ", " parcă ",
	" decat ", " decât ",
	" pana ", " până ",
	" asa ", " aşa ",
	" fara ", " fără ",
	" caci ", " căci ",
	" imi ", " îmi ",
	" ii ", " îi ",
	" decat ", " decât ",
	" doua ", " două ",
	" intr-o ", " într-o ",
	" toti ", " toţi ",
	" stiu ", " ştiu ",
	" isi ", " îşi ",
	" il ", " îl ",
	" inca ", " încă ",
	" intre ", " între ",
	" insa ", " însâ ",
	" toata ", " toată ",
	" facut ", " făcut ",
	" langa ", " lângă ",
	" vad ", " văd ",
	" cateva ", " câteva ",
	" vazut ", " văzut ",
	" inainte ", " înainte ",
	" noastra ", " noastră ",
	" ala ", " ăla "
	" as ", " aş "

	// "ca" list
	" ca sunt ", " că sunt ",
	" ca exista ", " că există ",
	" ca se ", " că se ",
	" ca a ", " că a ",
	" ca va ", " că va ",
	" ca ar ", " că ar ",
	" ca s-a ", " că s-a ",
	" ca îşi ", " că îşi ",
	" ca i-a ", " că i-a ",
	" ca i-au ", " că i-au ",
	" ca l-a ", " că l-a ",
	" ca l-au ", " că l-au ",
	" ca l-am ", " că l-am ",
	" ca m-a ", " că m-a ",
	" ca te-a ", " că te-a ",
	" ca s-au ", " că s-au ",
	" ca am ", " că am ",
	" ca au ", " că au ",
	" ca are ", " că are ",
	" ca ai ", " că ai ",
	" ca vom ", " că vom ",
	" ca le ", " că le ",
	" ca te ", " că te ",
	" ca ne ", " că ne ",
	" ca mă ", " că mă ",
	" ca aş ", " că aş ",
	" ca nu-mi ", " că nu-mi ",
	" ca nu-ti ", " că nu-ţi ",
	" ca nu m-a ", " că nu m-a ",
	" ca nu te-a ", " că nu te-a ",
	" ca e ", " că e ",
	" ca este ", " că este ",
	0
};

/*****************************
statistica de aparitie de cuvinte pe un text de aprox 150.000 cuvinte
101, câteva << included
103, m-am
103, faţa
103, putea
104, văzut << included
104, mi-a
104, ceva
105, vin
106, fac
106, departe
107, prea
107, unul
107, foarte
107, ale
107, ori
108, spune
109, zi
110, doi
111, înainte << included
112, noastră << included
113, lumea
114, aş
114, s-au
115, totul
115, timp
117, oameni
119, eram
119, spus
122, mari
122, este
122, văd << included
122, oamenii
124, despre
124, n-am
124, puţin
124, într-un
126, da
127, unui
128, apoi
132, această
133, aici
135, cât
136, fel
137, avea
138, cele
141, aproape
141, nişte
142, face
143, îl
145, atunci
145, pot
147, chiar
147, erau
151, atât
152, între
153, încă
153, dintre
153, lângă >> included
154, făcut >> included
154, ochii
157, va
158, mi
163, îşi >> included
166, toată >> included
167, fie
170, te
170, sau
171, trebuie
172, ştiu >> included
175, acolo
176, toţi
177, nimic
179, domnule
183, parcă >> included
192, lor
194, peste
195, unde
195, cei
199, meu
199, mare
201, are
204, mea
205, într-o >> included
206, două >> included
207, bine
211, ar
215, cel
216, mult
217, toate
220, însă << included
226, urmă
231, le
234, îi << included
241, ea
243, ai
255, până << included
256, decât << included
256, spre
267, poate
277, prin
279, numai
285, căci << included
292, s-a
305, îmi
312, asta
324, tot
329, al
339, fără << included
344, aşa << included
354, mine
356, noi
357, iar
374, nici
386, fost
387, ne
389, acum
390, lui
406, dacă >> included
415, el
429, au
433, după >> included
440, ei
469, cum
508, sunt
509, era
511, fi
532, eu
569, pentru
657, mă << included
717, dar
748, când << included
763, am
858, ce
1095, e
1228, din
1320, ca << ???
1332, un
1373, mai
1416, care
1573, se
1680, a
1855, nu
1929, că << ???
1947, o
2048, pe
2074, -
2317, la
2356, să << included
2636, cu
3413, în << included
5515, şi << included
6330, de
**************************************/

void Editor::autospell() {
	Global *glb = Global::instance();
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString intext = e_->text();

	// command available only for Romanian language
	if (glb->lang == LANG_RO_CEDILLA || glb->lang == LANG_RO_COMMABELOW) {
		// do the processing in cedilla
		intext = QTUTILS::commabelow2cedilla(intext);
	}
	else {
		QMessageBox::warning(this,"Info", QString(tr("This command only works for Romanian language settings\n")));
		return;
	}

	BCursor busy;

	// replacement in autolist
	QString tmp = intext;
	int i = 0;
	while (autolist[i] != 0) {
		tmp = tmp.replace(QRegExp(QString::fromUtf8(autolist[i])), QString::fromUtf8(autolist[i + 1]));
		QString qs1 = QString::fromUtf8(autolist[i]);
		QString qs2 = QString::fromUtf8(autolist[i + 1]);
		if (qs1.startsWith(" ") && qs2.startsWith(" ")) {
			qs1 = "\n" + qs1.mid(1);
			qs2 = "\n" + qs2.mid(1);
			tmp = tmp.replace(QRegExp(qs1), qs2);
		}
		i += 2;
	}

	{

		if (pdict_ == 0)
			initDictionary();

		ProgressBar *progress = 0;
		if (tmp.length() < 100000)
			progress = 0;
		else if (qvb_ != 0) {
			progress = new ProgressBar(qvb_, "add diacritics");
			progress->show();
			progress->start(550);
		}
		int cnt = 0;

		QString qs = tmp;
		QChar qc = qs[0];
		int i1 = 0;
		int i2 = 0;
		QString lastword1 = "";
		QString lastword2 = "";
		while (!qs.isEmpty()) {
			// advance to next word
			while (!qc.isNull() && !qc.isLetterOrNumber()) {
				i1++;
				qc = qs[i1];
			}
			if (qc.isNull())
				break;
			i2 = i1;

			// extract the word
			while (qc.isLetterOrNumber() || qc == '-') {
				i2++;
				qc = qs[i2];
			}
			QString extracted = qs.mid(i1, i2 - i1);
			int wstart = i1;
			int wend = i2;
			i1 = i2;

			Q3CString utext = extracted.utf8();
			const char *a = utext;
			char *active_word = (char *) a;

			// words not in the dictionary are replaced in case
			// there is only one sugestion
			if (!pdict_->findString(active_word) &&	// if not in dictionary
			    !hasnumeric(extracted) &&		// no numeric values
			    strlen(active_word) != 1) {		// not a single letter
			    	// word found
				// grab any valid words form dict based on active_words
				*word1_ = '\0';
				*word2_ = '\0';
				*word3_ = '\0';
				*word4_ = '\0';
				*word5_ = '\0';
				pdict_->getMaster()->findLookalike(active_word, word1_, word2_, word3_, word4_, word5_);

				// only one sugestion
				if (*word1_ != '\0' && *word2_ == '\0') {
					QString t = qs.left(wstart) + QString::fromUtf8(word1_) + qs.mid(wend);
					qs = t;
					active_word = word1_; // the replacement becomes the active word
				}
			}

			// solve cases such as: "o imprimanta" becomes "o imprimantă"
			QString word = QString::fromUtf8(active_word);
			if ((lastword1 == "o" || lastword1 == "O" || lastword1 == "nicio") && word.endsWith("a")) {
				// generate the word ending in "ă" and check in in the dictionary
				QString newword = word;
				int len = newword.length();
				newword = newword.left(len - 1) + QString::fromUtf8("ă");
				QString t = qs.left(wstart) + newword + qs.mid(wend);
				qs = t;
				word = newword; // the replacement becomes the active word
			}

			// solve cases such as: "să fie citita" becomes "să fie citită"
			if ((lastword2 == QString::fromUtf8("să") || lastword2 == QString::fromUtf8("Să")) &&
			     lastword1 == "fie" && word.endsWith("a")) {
				// generate the word ending in "ă" and check in in the dictionary
				QString newword = word;
				int len = newword.length();
				newword = newword.left(len - 1) + QString::fromUtf8("ă");
				QString t = qs.left(wstart) + newword + qs.mid(wend);
				qs = t;
				word = newword; // the replacement becomes the active word
			}

			// update last word(s)
			lastword2 = lastword1;
			lastword1 = word;
			if (++cnt % 100 == 0 && progress != 0)
				qApp->processEvents();

		}

		tmp = qs;
		if (progress != 0)
			delete progress;
	}

	if (glb->lang == LANG_RO_CEDILLA) {
		tmp = QTUTILS::commabelow2cedilla(tmp);
	}
	else if (glb->lang == LANG_RO_COMMABELOW) {
		tmp = QTUTILS::cedilla2commabelow(tmp);
	}
	e_->setText(tmp);
	e_->setCursorPosition(para, index, false);
}

void Editor::wordCount() {
	QString qs = e_->text();
	QChar qc = qs[0];
	int i1 = 0;
	int i2 = 0;
	int count = 0;

	while (!qs.isEmpty()) {
		// advance to next word
		while (!qc.isNull() && !qc.isLetterOrNumber()) {
			i1++;
			qc = qs[i1];
		}
		if (qc.isNull())
			break;
		i2 = i1;

		// extract the word
		while (qc.isLetterOrNumber() || qc == '-') {
			i2++;
			qc = qs[i2];
		}
		QString extracted = qs.mid(i1, i2 - i1);
		i1 = i2;
		count++;
	}

	QMessageBox::warning(this,"Info", QString("The current file has %1 words.").arg(count));
}

const char *dialist[] = {
	"ă", "a",
	"Ă", "A",
	"â", "a",
	"Â", "A",
	"ş", "s",
	"Ş", "S",
	"ţ", "t",
	"Ţ", "T",
	"î", "i",
	"Î", "I",
	0
};




void Editor::removeDiacritics() {
	Global *glb = Global::instance();
	BCursor busy;
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString intext = e_->text();


	// do the processing in cedilla
	if (glb->lang == LANG_RO_COMMABELOW || glb->lang == LANG_RO_CEDILLA) {
		intext = QTUTILS::commabelow2cedilla(intext);
	}
	else {
		QMessageBox::warning(this,"Info", QString("This command only works for Romanian language settings\n"));
		return;
	}


	// replacement in dialist
	QString tmp = intext;
	int i = 0;
	while (dialist[i] != 0) {
		tmp = tmp.replace(QRegExp(QString::fromUtf8(dialist[i])), QString::fromUtf8(dialist[i + 1]));
		i += 2;
	}

	if (glb->lang == LANG_RO_CEDILLA) {
		tmp = QTUTILS::commabelow2cedilla(tmp);
	}
	else if (glb->lang == LANG_RO_COMMABELOW) {
		tmp = QTUTILS::cedilla2commabelow(tmp);
	}
	e_->setText(tmp);
	e_->setCursorPosition(para, index, false);
}
