//****************************************************************************
// Copyright (C) 2005,  Lucian Constantin <struct_bylighting@yahoo.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//****************************************************************************

#ifndef QTUTILS_H
#define QTUTILS_H

#include "editor.h"
#include <qapplication.h>
#ifdef USING_QT3
#include <qmultilineedit.h>
#endif
#ifdef USING_QT4
#include <q3multilineedit.h>
#endif

#include <qfile.h>

#include "debug.h"

class QTUTILS {
	static const int PROG_BUF = 10000;
public:
	static void run_program(QString program, Q3MultiLineEdit *wdg, char *termination_string = 0);

	// suplimenting QString::find with whole word flag
	static int find(QString &haystack, QString &needle, int index, bool cs, bool whole);

	// suplimenting QString::findRev with whole word flag
	static int findRev(QString &haystack, QString &needle, int index, bool cs, bool whole);

	// strip trailing spaces from the end of each line in the string
	static QString stripTrailingSpaces(QString &input);

	// UNIX end of line translation
	static QString unixEol(QString &input);

	// bookmark helpers
	static QString cleanBookmarkers(QString &input);
	static QString cleanBookmarker(QString &input, int bookmark);
	static unsigned short getBookmarker(int bookmark);
	static bool hasBookmarkers(QString &input);
	static bool hasBadBookmarkers(QString &input);
	static bool startsWithBookmarker(QString &input);

	// Romanian language translation from cedilla chars to commabelow chars
	static QString cedilla2commabelow(QString &input);

	// Romanian language translation from commabelow chars to cedilla chars
	static QString commabelow2cedilla(QString &input);

	// convert tabs into space - hardcoded for one tab = 8 spaces
	static QString tab2space(QString &input);

	// opening files is different in QT3 and QT4
	static bool openFileReadOnly(QFile &f) {
#ifdef USING_QT3
		return f.open(IO_ReadOnly);
#else // USING_QT4
		return f.open(QIODevice::ReadOnly);
#endif
	}

	static bool openFileWriteOnly(QFile &f) {
#ifdef USING_QT3
		return f.open(IO_WriteOnly);
#else // USING_QT4
		return f.open(QIODevice::WriteOnly);
#endif
	}

};
#endif

