//****************************************************************************
// Copyright (C) 2005,  Lucian Constantin <struct_bylighting@yahoo.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//****************************************************************************
//** $Id: qt/qwerty.cpp   3.3.4   edited May 27 2003 $
//**
//** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
//**
//** This file is part of an example program for Qt.  This example
//** program may be used, distributed and modified without limitation.
//**
//*****************************************************************************/
// rospell user interface
//*****************************************************************************/

// for development/debug purposes only
#define WORK_DEBUG

#include "qwerty.h"
#include <qapplication.h>
#include <qdesktopwidget.h>
#include <qdatetime.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qpainter.h>
#include <qmessagebox.h>
#include <qfontdialog.h>
#include <qtextcodec.h>
#include "busycursor.h"
#include <qdir.h>

#ifdef USING_QT4
#include <q3filedialog.h>
#include <q3popupmenu.h>
#include <q3paintdevicemetrics.h>
#include <q3ptrlist.h>
//Added by qt3to4:
#include <Q3CString>
#include <QCloseEvent>
#include <QResizeEvent>
#include <Q3VBoxLayout>
//Added by qt3to4:
#include <QLabel>
#endif
#ifdef USING_QT3
#include <qfiledialog.h>
#include <qpaintdevicemetrics.h>
#endif
#include <qfileinfo.h>
#include <qinputdialog.h>
#include <qsplitter.h>
#include <qcombobox.h>
#include <qregexp.h>
#include <qtooltip.h>

#include "qtutils.h"
#include "spell.h"
#include "stats.h"
#include "dialogs.h"
#include "briefedit.h"
#include "gdbedit.h"
#include "statusfield.h"

#include "latex.h"
#include "bufmgr.h"
#include "ctags.h"
#include "gdb.h"
#include "filevisitor.h"


#include <qaction.h>
#include <qtoolbar.h>
#include "xpm/filenew.xpm"
#include "xpm/fileopen.xpm"
#include "xpm/fileprint.xpm"
#include "xpm/filesave.xpm"
#include "xpm/filesaveall.xpm"
#include "xpm/editcopy.xpm"
#include "xpm/editpaste.xpm"
#include "xpm/editundo.xpm"
#include "xpm/editcut.xpm"
#include "xpm/editredo.xpm"
#include "xpm/editfind.xpm"
#include "xpm/grep.xpm"
#include "xpm/make.xpm"
#include "xpm/gdb.xpm"
#include "xpm/spellcheck.xpm"
#include "xpm/brackets.xpm"
//const bool no_writing = FALSE;

// set the size of the last window in a vertical split to the value specified in the argument
// the requested size for the last window has to be less than the size of the split divided by the number of windows
static void balance_split(QSplitter *split, int last_size) {
	Q3ValueList<int> list = split->sizes();
	Q3ValueList<int>::Iterator it = list.begin();
	int split_size = 0; // total split size
	int cnt = 0;	// number of windows
	while( it != list.end() ) {
		cnt++;
		split_size += *it;
		++it;
	}

	if ((last_size * cnt) >= split_size)
		return; // the caller is asking for too much!!!
	if (cnt == 1) {
		it = list.begin();
		*it = split_size - last_size;
	}
	else {
		int first_size = (split_size - last_size) / (cnt - 1);
		it = list.begin();
		int i = 0;
		while( it != list.end() ) {
			*it = first_size;
			if (i == (cnt - 1))
				*it = last_size;
			++it;
			i++;
		}
	}
	split->setSizes(list);
}




Editor::Editor( QWidget * parent , const char * name ): QMainWindow(parent, name) {
//: Q3VBox(parent, name) {
	//: QWidget( parent, name, Qt::WDestructiveClose )
	LOG("before resize in constructor");
	resize(Global::instance()->xsize, Global::instance()->ysize);
	LOG("after resize in constructor");

	Q3VBox *qvb = new Q3VBox(this);
	setCentralWidget(qvb);
	qvb_ = qvb;
//printf("this %p\n", qvb);
//qApp->setMainWidget(qvb);

	pdict_ = 0;
	search_ = "";
	default_format_ = FILE_UTF8;
	*word1_ = '\0';
	*word2_ = '\0';
	*word3_ = '\0';
	*word4_ = '\0';
	*word5_ = '\0';
	resize(Global::instance()->xsize, Global::instance()->ysize);

	//*****************************************************
	// accelerators
	//*****************************************************
	Q3Accel *a_ = new Q3Accel(this);
	// set bookmarks
	a_->connectItem(a_->insertItem(Qt::ALT+Qt::Key_1), this, SLOT(setBookmark1()));
	a_->connectItem(a_->insertItem(Qt::ALT+Qt::Key_2), this, SLOT(setBookmark2()));
	a_->connectItem(a_->insertItem(Qt::ALT+Qt::Key_3), this, SLOT(setBookmark3()));
	a_->connectItem(a_->insertItem(Qt::ALT+Qt::Key_4), this, SLOT(setBookmark4()));
	a_->connectItem(a_->insertItem(Qt::ALT+Qt::Key_5), this, SLOT(setBookmark5()));
	a_->connectItem(a_->insertItem(Qt::ALT+Qt::Key_6), this, SLOT(setBookmark6()));
	a_->connectItem(a_->insertItem(Qt::ALT+Qt::Key_7), this, SLOT(setBookmark7()));
	a_->connectItem(a_->insertItem(Qt::ALT+Qt::Key_8), this, SLOT(setBookmark8()));
	a_->connectItem(a_->insertItem(Qt::ALT+Qt::Key_9), this, SLOT(setBookmark9()));
	a_->connectItem(a_->insertItem(Qt::ALT+Qt::Key_0), this, SLOT(setBookmark0()));
	// user programable keys
	a_->connectItem(a_->insertItem(Qt::Key_F1), this, SLOT(userF1()));
	a_->connectItem(a_->insertItem(Qt::Key_F2), this, SLOT(userF2()));
	a_->connectItem(a_->insertItem(Qt::Key_F3), this, SLOT(userF3()));
	a_->connectItem(a_->insertItem(Qt::Key_F4), this, SLOT(userF4()));

	// debug
	a_->connectItem(a_->insertItem(Qt::Key_F12), this, SLOT(roDebug()));
#if 0 // F11 currently used by the debugger
	a_->connectItem(a_->insertItem(Qt::Key_F11), this, SLOT(roDevel()));
#endif

	//*****************************************************
	// menubar
	//*****************************************************
	m_ = new QMenuBar( this, "menu" );

	// file
	tool1_ = new QToolBar(this, "toolbar");
 	tool1_->setLabel(tr("File Operations"));
 	if (!Global::instance()->toolbar_)
 		tool1_->hide();
	file_ = new Q3PopupMenu(m_);
	Q_CHECK_PTR( file_ );
	m_->insertItem( tr("&File"), file_ );
	file_->insertItem( tr("&New Window"),   this, SLOT(newDoc()));

 	QAction *a = new QAction(tr("new buffer"), QPixmap(xpm_file_new), "", 0, this);
	connect( a, SIGNAL( activated() ), this, SLOT(newDocBuffer()));
	a->addTo(tool1_);
	file_->insertItem(tr("New &Buffer"),   this, SLOT(newDocBuffer()));

	a = new QAction(tr("open"), QPixmap(xpm_file_open), "", 0, this);
	connect( a, SIGNAL( activated() ), this, SLOT(load()));
	a->addTo(tool1_);
	file_->insertItem(tr("&Open..."),  this, SLOT(load()), Qt::ALT+Qt::Key_O);

#ifndef ROSPELL_NO_WGET
	file_->insertItem( tr("Open &Location"),  this, SLOT(openWwwLocation()));
#endif

	file_->insertSeparator();
	file_->insertItem( tr("&Change format"), this, SLOT(changeFormat()));
#ifndef ROSPELL_NO_PRINTER
	a = new QAction(tr("print"), QPixmap(xpm_file_print), "", 0, this);
	connect( a, SIGNAL( activated() ), this, SLOT(print()));
	a->addTo(tool1_);
	file_->insertItem(tr("&Print..."), this, SLOT(print()));
#endif
	file_->insertSeparator();

	a = new QAction(tr("save"), QPixmap(xpm_file_save), "", 0, this);
	connect( a, SIGNAL( activated() ), this, SLOT(save()));
	a->addTo(tool1_);
	file_->insertItem(tr("&Save..."),  this, SLOT(save()), Qt::ALT+Qt::Key_W);

	file_->insertItem( tr("Sa&ve As"),  this, SLOT(saveAs()));

	a = new QAction(tr("save all"), QPixmap(xpm_file_saveall), "", 0, this);
	connect( a, SIGNAL( activated() ), this, SLOT(saveAll()));
	a->addTo(tool1_);
	file_->insertItem( tr("Save &All"),  this, SLOT(saveAll()));

	// recent files
	file_->insertSeparator();
	rc0_ = file_->insertItem( "",   this, SLOT(openRecent0()));
	rc1_ = file_->insertItem( "",   this, SLOT(openRecent1()));
	rc2_ = file_->insertItem( "",   this, SLOT(openRecent2()));
	rc3_ = file_->insertItem( "",   this, SLOT(openRecent3()));
	rc4_ = file_->insertItem( "",   this, SLOT(openRecent4()));
	rc5_ = file_->insertItem( "",   this, SLOT(openRecent5()));
	updateRecent();

	file_->insertSeparator();
	file_->insertItem( tr("&Close Buffer"),  this, SLOT(closeBuffer()), Qt::ALT+Qt::Key_D );
	file_->insertItem( tr("E&xit"), this, SLOT(close()), Qt::ALT+Qt::Key_X);

	// edit
	tool2_ = new QToolBar(this, "toolbar");
 	tool2_->setLabel(tr("Edit Operations"));
 	if (!Global::instance()->toolbar_)
 		tool2_->hide();
	Q3PopupMenu *edit_ = new Q3PopupMenu(m_);
	Q_CHECK_PTR(edit_);
	m_->insertItem(tr("&Edit"), edit_);

	a = new QAction(tr("undo"), QPixmap(xpm_edit_undo), "", 0, this);
	connect( a, SIGNAL( activated() ), this, SLOT(undo()));
	a->addTo(tool2_);
	edit_->insertItem( tr("&Undo"), this, SLOT(undo()), Qt::ALT+Qt::Key_U);

	a = new QAction(tr("redo"), QPixmap(xpm_edit_redo), "", 0, this);
	connect( a, SIGNAL( activated() ), this, SLOT(redo()));
	a->addTo(tool2_);
	edit_->insertItem( tr("Redo"), this, SLOT(redo()));

	edit_->insertItem( tr("Toggle &Insert/Overwrite"), this, SLOT(toggleOverwrite()));
	edit_->insertSeparator();

	a = new QAction(tr("cut"), QPixmap(xpm_edit_cut), "", 0, this);
	connect( a, SIGNAL( activated() ), this, SLOT(cut()));
 	a->addTo(tool2_);
	edit_->insertItem( tr("Cut"), this, SLOT(cut()), Qt::CTRL+Qt::Key_X);

	a = new QAction(tr("copy"), QPixmap(xpm_edit_copy), "", 0, this);
	connect( a, SIGNAL( activated() ), this, SLOT(copy()));
	a->addTo(tool2_);
	edit_->insertItem( tr("Copy"), this, SLOT(copy()), Qt::CTRL+Qt::Key_C);

	a = new QAction(tr("paste"), QPixmap(xpm_edit_paste), "", 0, this);
	connect( a, SIGNAL( activated() ), this, SLOT(paste()));
	a->addTo(tool2_);
	edit_->insertItem( tr("Paste"), this, SLOT(paste()), Qt::CTRL+Qt::Key_V);

	edit_->insertItem( tr("Delete"), this, SLOT(e_delete()));
	edit_->insertItem( tr("Select All"), this, SLOT(selectAll()));

	a = new QAction(tr("select block"), QPixmap(xpm_brackets), "", 0, this);
	connect( a, SIGNAL( activated() ), this, SLOT(selectBlock()));
	a->addTo(tool2_);
	edit_->insertItem( tr("Select Block"), this, SLOT(selectBlock()), Qt::CTRL+Qt::Key_K);
	edit_->insertSeparator();


	tool3_ = new QToolBar(this, "toolbar");
 	tool3_->setLabel(tr("Tools"));
 	if (!Global::instance()->toolbar_)
 		tool3_->hide();
	a = new QAction(tr("find"), QPixmap(xpm_edit_find), "", 0, this);
	connect( a, SIGNAL( activated() ), this, SLOT(search()));
 	a->addTo(tool3_);
	edit_->insertItem( tr("&Find"), this, SLOT(search()), Qt::Key_F5);

	edit_->insertItem( tr("Find Ne&xt"), this, SLOT(searchAgain()), Qt::SHIFT+Qt::Key_F5);
	edit_->insertItem( tr("&Replace"), this, SLOT(replace()), Qt::Key_F6);

	a = new QAction(tr("grep"), QPixmap(xpm_grep), "", 0, this);
	connect( a, SIGNAL( activated() ), this, SLOT(grep()));
 	a->addTo(tool3_);
	edit_->insertItem( tr("&Grep"), this, SLOT(grep()), Qt::Key_F7);
	edit_->insertItem( tr("Function Lis&t"), this, SLOT(toc()), Qt::Key_F8);
	edit_->insertSeparator();
	edit_->insertItem( tr("Goto &Line"), this, SLOT(gotoLine()), Qt::ALT+Qt::Key_G);
	edit_->insertItem( tr("&Start of File"), this, SLOT(startOfBuffer()), Qt::CTRL+Qt::Key_PageUp);
	edit_->insertItem( tr("&End of File"), this, SLOT(endOfBuffer()), Qt::CTRL+Qt::Key_PageDown);
	edit_->insertItem( tr("Set &Bookmark"), this, SLOT(setBookmark()));
	edit_->insertItem( tr("Goto Boo&kmark"), this, SLOT(gotoBookmark()), Qt::ALT+Qt::Key_J);
	edit_->insertSeparator();
	edit_->insertItem( tr("Insert Unicode Char"), this, SLOT(insertHex()));

	// options/preferences
	Q3PopupMenu * config_ = new Q3PopupMenu(m_);
	Q_CHECK_PTR( config_ );
	m_->insertItem( tr("Preferences"), config_ );
	config_->insertItem(tr("Select &Font"), this, SLOT(font()));
	config_->insertItem(tr("&Colors"), this, SLOT(colors()));
	config_->insertItem(tr("&Alias"), this, SLOT(editAlias()));
	config_->insertItem(tr("&Language"), this, SLOT(configLanguage()));
#ifndef ROSPELL_NO_WGET
	config_->insertItem(tr("&HTTP Proxy"), this, SLOT(configWWW()));
#endif
	Q3PopupMenu *user = new Q3PopupMenu(config_);
	user->insertItem( tr("Edit F&1 Script") ,   this, SLOT(editF1()));
	user->insertItem( tr("Edit F&2 Script") ,   this, SLOT(editF2()));
	user->insertItem( tr("Edit F&3 Script") ,   this, SLOT(editF3()));
	user->insertItem( tr("Edit F&4 Script") ,   this, SLOT(editF4()));
	config_->insertItem( tr("User &Scripts") ,  user);
	config_->insertSeparator();
	config_->insertItem(tr("Cta&gs"), this, SLOT(configCtags()));
	config_->insertItem(tr("L&ock Ctags"), this, SLOT(lockCtags()));
	config_->insertItem(tr("&Unlock Ctags"), this, SLOT(unlockCtags()));
	config_->insertSeparator();
	config_->insertItem(tr("File &Options"), this, SLOT(fileOptions()));
	config_->insertItem(tr("Customize"), this, SLOT(customize()));
	config_->insertItem(tr("S&tatus"), this, SLOT(status()));

	// buffers
	buffers_ = new Q3PopupMenu(m_);
	Q_CHECK_PTR(buffers_);
	m_->insertItem(tr("&Buffers"), buffers_);
	connect(buffers_, SIGNAL(activated(int)), this, SLOT(pickBuffer(int)));
	buffers_->insertItem(tr("&Next"), this, SLOT(nextBuffer()), Qt::ALT+Qt::Key_N);
	buffers_->insertSeparator();

	// tools
	tools_ = new Q3PopupMenu(m_);
	Q_CHECK_PTR( tools_ );
	m_->insertItem( tr("Too&ls"), tools_ );
	tools_->insertItem( tr("&Record") , this, SLOT(recordAction()), Qt::ALT+Qt::Key_R);
	tools_->insertItem( tr("&Playback") , this, SLOT(playbackAction()), Qt::ALT+Qt::Key_P);
	tools_->insertSeparator();
	tools_->insertItem( tr("&Uppercase") ,   this, SLOT(uppercase()));
	tools_->insertItem( tr("&Lowercase") ,   this, SLOT(lowercase()));
	cleaners_ = new Q3PopupMenu(tools_);
	cleaners_->insertItem( tr("UNI&X CR/LF cleanup") ,   this, SLOT(unixCleanup()));
	cleaners_->insertItem( tr("&Strip trailing spaces") ,   this, SLOT(stripTrailingSpaces()));
	cleaners_->insertItem( tr("&Tab to space") ,   this, SLOT(tabToSpace()));
	ced2comaid_ = cleaners_->insertItem( tr("&Cedilla to commabelow") ,   this, SLOT(cedilla2commabelow()));
	coma2cedid_ = cleaners_->insertItem( tr("Comma&below to cedilla") ,   this, SLOT(commabelow2cedilla()));
	addroid_ = cleaners_->insertItem( tr("&Add Romanian Diacritics") ,   this, SLOT(autospell()));
	remroid_ = cleaners_->insertItem( tr("&Remove Romanian Diacritics") ,   this, SLOT(removeDiacritics()));
	tools_->insertItem( tr("&Cleaners"),  cleaners_);
	tools_->insertSeparator();
	code_ = new Q3PopupMenu(tools_);
	code_->insertItem( tr("&Beautify") ,   this, SLOT(beautify()));
	code_->insertItem( tr("&Syntax check") ,   this, SLOT(htmltidy()));
	code_->insertItem( tr("&KLOC") ,   this, SLOT(kloc()));
	tools_->insertItem( tr("C&ode"),  code_);

	a = new QAction(tr("make"), QPixmap(xpm_make), "", 0, this);
	connect( a, SIGNAL( activated() ), this, SLOT(make()));
 	a->addTo(tool3_);
	tools_->insertItem( tr("&make") ,   this, SLOT(make()));

#ifndef ROSPELL_CYGWIN
//#ifndef ROSPELL_FREEBSD
	a = new QAction(tr("gdb"), QPixmap(xpm_gdb), "", 0, this);
	connect( a, SIGNAL( activated() ), this, SLOT(gdbEnable()));
 	a->addTo(tool3_);
	tools_->insertItem( tr("&gdb"),   this, SLOT(gdbEnable()));
//#endif
#endif
#ifndef ROSPELL_NO_DIFF
	tools_->insertSeparator();
	tools_->insertItem( tr("&Diff") ,   this, SLOT(diff()));
	tools_->insertItem( tr("&Session Diff") ,   this, SLOT(sessionDiff()));
	tools_->insertItem( tr("M&erge") ,   this, SLOT(merge()));
	tools_->insertItem( tr("D&irMerge") ,   this, SLOT(dirMerge()));
#endif
	tools_->insertSeparator();
#ifndef ROSPELL_NO_WGET
	dexid_ = tools_->insertItem( tr("&Dictionary") ,   this, SLOT(dexonline()));
#endif
	a = new QAction(tr("spelling"), QPixmap(xpm_spell_check), "", 0, this);
	connect( a, SIGNAL( activated() ), this, SLOT(toggleSpeller()));
 	a->addTo(tool3_);
	tools_->insertItem( tr("Spelli&ng") ,   this, SLOT(toggleSpeller()));
	tools_->insertItem( tr("&Word Count") ,   this, SLOT(wordCount()));

	// latex
	addLatexMenu();


	// gdb
#ifndef ROSPELL_CYGWIN
//#ifndef ROSPELL_FREEBSD
	addGdbMenu();
//#endif
#endif
	// help
	m_->insertSeparator();
	addHelpMenu();


	//*****************************************************
	// spelling menubar
	//*****************************************************
	Q3VBox *vbox = new Q3VBox(qvb);
	mwords_ = new QMenuBar(vbox, "words menu" );
	setSpellerBar();
	show_speller_ = false;
	mwords_->hide();




	// starting splitters!
	QSplitter *split1 = new QSplitter(vbox);
#ifdef USING_QT3
	split1->setOrientation(QSplitter::Vertical);
#else
	split1->setOrientation(Qt::Vertical);
#endif
	QSplitter *split2 = new QSplitter(split1);

	//*****************************************************
	// directory window
	//*****************************************************
	Q3VBox *vdirbox = new Q3VBox(split2);
	vdirbox->setSpacing(1);
	dirfiles_size_ = 150;
	vdirbox->setMaximumWidth(dirfiles_size_);
	dircombo_ = new QComboBox(TRUE, vdirbox);
	dircombo_->insertItem("*");
	dircombo_->insertItem("*.asm,*.S");
	dircombo_->insertItem("*.c,*.cpp,*.h");
	dircombo_->insertItem("*.html");
	dircombo_->insertItem("*.java");
	dircombo_->insertItem("*.php");
	dircombo_->insertItem("*.pl");
	dircombo_->insertItem("*.py");
	dircombo_->insertItem("*.sh");
	dircombo_->insertItem("*.tex");

	dirfiles_ = new Q3MultiLineEdit(vdirbox);
	dirfiles_->setReadOnly(TRUE);
	dirfiles_path_ = "./";
	QString files = getDirfiles();
	setDirfiles(files);
	connect(dirfiles_, SIGNAL(clicked(int, int)), this, SLOT(clickedDirfiles(int, int)));
	connect(dircombo_, SIGNAL(activated(int)), this, SLOT(dirboxChanged(int)));

	// colors
	dircombo_->setPaletteBackgroundColor(Global::instance()->c_background);
	dircombo_->setPaletteForegroundColor(Global::instance()->c_foreground);
	QPalette qp = dircombo_->palette();
	qp.setColor(QPalette::Active, QColorGroup::Highlight, Global::instance()->c_highlight);
	qp.setColor(QPalette::Inactive, QColorGroup::Highlight, Global::instance()->c_highlight);
	dircombo_->setPalette(qp);
	dirfiles_->setPaletteBackgroundColor(Global::instance()->c_background);
	dirfiles_->setPaletteForegroundColor(Global::instance()->c_foreground);
	qp = dirfiles_->palette();
	qp.setColor(QPalette::Active, QColorGroup::Highlight, Global::instance()->c_highlight);
	qp.setColor(QPalette::Inactive, QColorGroup::Highlight, Global::instance()->c_highlight);
	dirfiles_->setPalette(qp);

	//*****************************************************
	// first editor
	//*****************************************************
	QSplitter *split3 = new QSplitter(split2);
#ifdef USING_QT3
	split3->setOrientation(QSplitter::Vertical);
#else
	split3->setOrientation(Qt::Vertical);
#endif
	emain_ = new BriefEdit(this, split3, "editor" );
	// get default XML syntax
	getSyntax("", emain_);
	connect( emain_, SIGNAL( textChanged() ), this, SLOT( textChanged() ) );
	connect( emain_, SIGNAL( clicked(int, int) ), this, SLOT( clicked(int, int) ) );
	emain_->setWordWrap(Q3TextEdit::NoWrap);
	b_main_ = new BufMgr();
	b_main_->dbgname_ = tr("main window");
	changed_main_ = false;
	// active window setting
	using_aux_ = false;
	changed_main_ = false;
	e_ = emain_;
	b_ = b_main_;
	book_ = book_main_;

	// colors
	emain_->setPaletteBackgroundColor(Global::instance()->c_background);
	emain_->setPaletteForegroundColor(Global::instance()->c_foreground);
	qp = emain_->palette();
	qp.setColor(QPalette::Active, QColorGroup::Highlight, Global::instance()->c_highlight);
	qp.setColor(QPalette::Inactive, QColorGroup::Highlight, Global::instance()->c_highlight);
	emain_->setPalette(qp);

	//*****************************************************
	// aux editor
	//*****************************************************
	eaux_ = new BriefEdit(this, split3, "editor_aux" );
	// get default XML syntax
	getSyntax("", eaux_);
	connect( eaux_, SIGNAL( textChanged() ), this, SLOT( textChanged() ) );
	connect( eaux_, SIGNAL( clicked(int, int) ), this, SLOT( clickedAux(int, int) ) );
	eaux_->setWordWrap(Q3TextEdit::NoWrap);

	// colors
	eaux_->setPaletteBackgroundColor(Global::instance()->c_background);
	eaux_->setPaletteForegroundColor(Global::instance()->c_foreground);
	qp = eaux_->palette();
	qp.setColor(QPalette::Active, QColorGroup::Highlight, Global::instance()->c_highlight);
	qp.setColor(QPalette::Inactive, QColorGroup::Highlight, Global::instance()->c_highlight);
	eaux_->setPalette(qp);
	changed_aux_ = false;
	b_aux_ = new BufMgr();
	b_aux_->dbgname_ = tr("aux window");

	// set size of aux window to 0
#ifdef USING_QT3
	balance_split(split3, 0);
//lucian??? bad size calculated
#endif

	//*****************************************************
	// user work tabs area
	//*****************************************************
	tw_ = new QTabWidget(split1);
	tw_size_ = 500;
	tw_->setMaximumHeight(tw_size_);
	// user
	tw_user_ = new Q3MultiLineEdit(split1);
	tw_user_->setReadOnly(TRUE);
	tw_user_->setPaletteBackgroundColor(Global::instance()->c_background);
	tw_user_->setPaletteForegroundColor(Global::instance()->c_foreground);
	tw_user_->setPalette(qp);
	// search
	tw_search_ = new Q3MultiLineEdit(split1);
	tw_search_->setReadOnly(TRUE);
	tw_search_->setPaletteBackgroundColor(Global::instance()->c_background);
	tw_search_->setPaletteForegroundColor(Global::instance()->c_foreground);
	tw_search_->setPalette(qp);
	// tags
	tw_tags_ = new Q3MultiLineEdit(split1);
	tw_tags_->setReadOnly(TRUE);
	tw_tags_->setPaletteBackgroundColor(Global::instance()->c_background);
	tw_tags_->setPaletteForegroundColor(Global::instance()->c_foreground);
	tw_tags_->setPalette(qp);
	getSyntax("file.cpp", tw_tags_);
	ctags_locked_ = false;
	// bookmarks
	tw_bookmarks_ = new Q3MultiLineEdit(split1);
	tw_bookmarks_->setReadOnly(TRUE);
	tw_bookmarks_->setPaletteBackgroundColor(Global::instance()->c_background);
	tw_bookmarks_->setPaletteForegroundColor(Global::instance()->c_foreground);
	tw_bookmarks_->setPalette(qp);
	// hex
	tw_hex_ = new Q3MultiLineEdit(split1);
	tw_hex_->setReadOnly(TRUE);
	tw_hex_->setPaletteBackgroundColor(Global::instance()->c_background);
	tw_hex_->setPaletteForegroundColor(Global::instance()->c_foreground);
	tw_hex_->setPalette(qp);
	QFont courier("Courier",12);
	tw_hex_->setFont(courier);
	// gdb
	tw_gdb_ = 0;

	connect(tw_tags_, SIGNAL(clicked(int, int)), this, SLOT(clickedTags(int, int)));
	connect(tw_search_, SIGNAL(clicked(int, int)), this, SLOT(clickedSearch(int, int)));
	connect(tw_user_, SIGNAL(clicked(int, int)), this, SLOT(clickedUser(int, int)));
	connect(tw_bookmarks_, SIGNAL(clicked(int, int)), this, SLOT(clickedBookmarks(int, int)));
	connect(tw_hex_, SIGNAL(clicked(int, int)), this, SLOT(clickedHex(int, int)));
	// tw_gdb_ is not added to the tag list

	tw_->addTab(tw_tags_, tr("tags"));
	tw_->addTab(tw_search_, tr("find"));
	tw_->addTab(tw_user_, tr("user"));
	tw_->addTab(tw_bookmarks_, tr("bookmarks"));
	tw_->addTab(tw_hex_, tr("hex"));


	tw_current_ = tw_tags_;
	connect(tw_, SIGNAL(currentChanged(QWidget*)), this, SLOT(twChanged(QWidget*)));

	// set size of tw_ to a maximum of 200
#ifdef USING_QT3
	balance_split(split1, 200);
#endif



	//*****************************************************
	// status bar
	//*****************************************************
	status_ = new QStatusBar(vbox, "status bar");

	connect( emain_, SIGNAL(cursorPositionChanged(int, int)), this, SLOT(cursorPosChanged(int, int)));
	connect( eaux_, SIGNAL(cursorPositionChanged(int, int)), this, SLOT(cursorPosChanged(int, int)));
	lang_ = new StatusField(this, status_, "language", "language");
	status_->addWidget(lang_, 0, TRUE);
	lang_->setText(getLangName(Global::instance()->lang));
	ff_ = new StatusField(this, status_, "format", ff_str(default_format_));
	status_->addWidget(ff_, 0, TRUE);
	iow_ = new StatusField(this, status_, "ins_ovr", "Insert");
	status_->addWidget(iow_, 0, TRUE);
	attention_ = new StatusField(this, status_, "red");
	attention_->setShown(false);
	attention_->setPaletteBackgroundColor(QColor(255, 0, 0));
	status_->addWidget(attention_, 0, TRUE);
	lac_ = new StatusField(this, status_, "line and column widget", "1:1");
	status_->addWidget(lac_, 0, TRUE);
	// set keyboard focus in the editor window
	e_->setFocus();


	// connect ctags thread
	thread_ = new CtagsThread(this);
	thread_->start(QThread::LowPriority);
#ifdef USING_QT4
	connect((const QObject *) thread_, SIGNAL(alldone()), this, SLOT(ctagsDone()));
#endif

	// gdb thread
	gthread_ = new GdbThread(this);

	// connect and initialise timer
	connect( &timer_, SIGNAL( timeout() ), this, SLOT( slotTimeout() ) );
	timer_.start(5000);			  // 5 second timer
	timer_cnt_ = 0;

	// enable or disable dynamic menus
	enableDisableMenus();

}


Editor::~Editor() {
	LOG("Editor::~Editor");
}

void Editor::releaseAll() {
	//	if (tw_tags_syntax_ == 0) // in order not to release twice
	//		return;
	//printf("releaseAll\n");
	LOG("releaseAll");

	// release syntax
	delete tw_tags_->syntaxHighlighter();
	delete emain_->syntaxHighlighter();
	delete eaux_->syntaxHighlighter();

	delete b_main_;
	delete b_aux_;


	if (pdict_ != 0)
		delete pdict_;
//	delete m_;
	delete mwords_;
	//	delete e_main_;
	// terminate the thread
	if (thread_ != 0) {
		thread_->terminate();
		thread_->wait(3000); // wait no more than 3 seconds
		delete thread_;
	}
	delete status_;

	if (gthread_ != 0) {
		// terminate the thread
		gthread_->terminate();
		gthread_->wait(3000);
		delete gthread_;
		// make gdb menu invisible
	}
}


void Editor::cursorPosChanged(int para, int pos) {
	char msg1[500];
	char msg2[500];

	*msg1 = '\0';
	*msg2 = '\0';

	//printf("tabStopWidth %d\n", e_->tabStopWidth());
	// print line and colum position
	QString line = e_->textLine(para);
	QString tmpline = line;
	int newpos;
	if (pos == 0)
		newpos = 0;
	else {
		tmpline.truncate(pos);
		//lucian: tab size hardcoded - if tab is 8 use 7 since it is added 1 by pos
						  // take into account the TAB character
		newpos = (tmpline.contains("\t")? 1: 0)  * 7 + pos;
	}

	sprintf(msg1, "%d:%d", para + 1, newpos + 1);
	if (e_->isRecording()) {
		sprintf(msg2, "Rec");
	}
	else if (e_->isAcc())
		sprintf(msg2, "Acc: %d", e_->getAccLines());
	else if (e_->hasSelectedText()) {
		int para_from;
		int index_from;
		int para_to;
		int index_to;
		e_->getSelection(&para_from, &index_from, &para_to, &index_to);
		int minx = index_to - index_from;
		if (minx < 0)
			minx *= -1;

		int deltax = para_to - para_from + 1;
		if (minx == 0 && deltax == 1);
		else
			sprintf(msg2, "%d:%d",
				para_to - para_from + 1, minx);
	}

	lac_->setText(msg1);
	attention_->setText(msg2);


	// update hex if necessary
	if (tw_current_ == tw_hex_) {
		clickedHex(0, 0);
	}


	// ctags processing
	if (!thread_->isActive() || ctags_locked_)
		return;

	QChar ch = line.at(pos);
	if (ch.isLetterOrNumber() || ch == '_') {
		// extract word
		int xstart = pos;
		while (xstart >= 0 && (line.at(xstart).isLetterOrNumber() || line.at(xstart) == '_'))
			xstart--;
		if (xstart != 0)
			xstart++;

		int xend = pos;
		while (line.at(xend).isLetterOrNumber() || line.at(xend) == '_')
			xend++;

		QString ctagword = line.mid(xstart, xend - xstart);
		if (ctagword.startsWith("\t"))	  // lucian: some problems with the first letter when is at the beginning of the line! Below is just a hack!
			ctagword = ctagword.mid(1);
		if (ctagword.startsWith("\\"))
			ctagword = ctagword.mid(1);
		thread_->startProcessing(ctagword);
	}
}


void Editor::pickBuffer(int id) {
	if (id < 0)
		return;
	BCursor busy;
	storeCurentBuffer();

	int index = id - 1;
	LOG("got alt key, id %d", id);

	// move to the specific buffer
	b_->setActiveIndex(index);

	// restore data
	if (using_aux_)
		changed_aux_ = true;
	else
		changed_main_ = true;

	QString fname = b_->getActiveFName();
	updateDirpath(fname);
	setFname(fname, b_->getActiveFormat(), false);
	int line = b_->getActiveLine();
	int col = b_->getActiveCol();
	e_->setCursorPosition(line, col, false);
	e_->setText(b_->getActiveText());

#if 0	// some 50ms delay here
	QTime t;
	t.start();
	while (t.elapsed() < 50) {
		qApp->processEvents();
	}
#endif
	if (using_aux_)
		changed_aux_ = fname.startsWith("*");
	else
		changed_main_ = fname.startsWith("*");
	//LOG("get line %d col %d", line, col);
	e_->setCursorPosition(line, col, false);
	redoBufMgrList();
}


void Editor::pickBuffer(BufMgr *b, BriefEdit *e, int id) {
	if (id < 0)
		return;
	BCursor busy;
	storeCurentBuffer(b, e);

	int index = id - 1;
	LOG("got alt key, id %d", id);

	// move to the specific buffer
	b->setActiveIndex(index);

	// restore data
	if (using_aux_)
		changed_aux_ = true;
	else
		changed_main_ = true;

	QString fname = b->getActiveFName();
	updateDirpath(fname);
	//	setFname(fname, false);
	int line = b->getActiveLine();
	int col = b->getActiveCol();
	e->setCursorPosition(line, col, false);
	e->setText(b->getActiveText());
#if 0
	// some 50ms delay here
	QTime t;
	t.start();
	while (t.elapsed() < 50) {
		qApp->processEvents();
	}
#endif
	if (using_aux_)
		changed_main_ = fname.startsWith("*");
	else
		changed_aux_ = fname.startsWith("*");
	//LOG("get line %d col %d", line, col);
	e->setCursorPosition(line, col, false);
	//	redoBufMgrList();
}


void Editor::storeCurentBuffer(BufMgr *b, BriefEdit *e) {
	QString fn = b->getActiveFName();
	if (fn.isNull() || fn.isEmpty())	  // it will get here first time we open the editor
		return;

	b->setActiveText(e->text());
	bool isnew = b->setActiveFName(fn);
	if (isnew)
		b->add(caption(), default_format_);
	int line;
	int col;
	e->getCursorPosition(&line, &col);
	//LOG("save line %d col %d", line, col);
	b->setActiveLine(line);
	b->setActiveCol(col);
}


void Editor::storeCurentBuffer() {
	QString fn = caption();
	if (fn.isNull() || fn.isEmpty() || fn == "empty buffer")	  // it will get here first time we open the editor
		return;

	b_->setActiveText(e_->text());
	bool isnew = b_->setActiveFName(caption());
	if (isnew)
		b_->add(caption(), default_format_);
	int line;
	int col;
	e_->getCursorPosition(&line, &col);
	//LOG("save line %d col %d", line, col);
	b_->setActiveLine(line);
	b_->setActiveCol(col);
}


void Editor::nextBuffer() {
	BCursor busy;
	// save curent text first
	storeCurentBuffer();

	// move to the next buffer
	b_->setActiveNext();

	// restore data
	if (using_aux_)
		changed_aux_ = true;
	else
		changed_main_ = true;
	QString fname = b_->getActiveFName();
	updateDirpath(fname);
	setFname(fname, b_->getActiveFormat(), false);

	int line = b_->getActiveLine();
	int col = b_->getActiveCol();
	//LOG("get line %d col %d", line, col);
	e_->setCursorPosition(line, col, false);
	e_->setText(b_->getActiveText());
	if (using_aux_)
		changed_aux_ = fname.startsWith("*");
	else
		changed_main_ = fname.startsWith("*");

	//LOG("get line %d col %d", line, col);
	e_->setCursorPosition(line, col, false);
	e_->syntaxHighlighter()->rehighlight();

	//LOG("starting redo");
	redoBufMgrList();
}


bool Editor::saveBufMgrAll(BufMgr *bm, BriefEdit *e) {
	ASSERT(bm);
	ASSERT(e);
	// save current editing info in buffer mgr
	storeCurentBuffer(bm, e);
	bool retval = true;

	// save content of buffer mgr in files
	for (int i = 0; i < MAXBUFMGR; i++) {
		QString fname = bm->getFName(i);
		if (fname == "*") {
			QMessageBox::warning(this,tr("I/O Info"), QString(tr("Please save all new files first\n")));
			return false;
		}

		if (fname.isNull())
			break;
		if (fname.startsWith("*")) {
			retval = retval && saveCommonText(fname.mid(1), bm->getFormat(i), bm->getText(i));
			bm->setFName(fname.mid(1), i);
		}
	}

	// remove * from current editing name; by now we have no * in bufmgr
	if (bm == b_) {
		QString fn = bm->getActiveFName();
		setFname(fn, bm->getActiveFormat(), false);
	}
	return retval;
}


// this function is called every 5 seconds
void Editor::slotTimeout() {
	// check modifications for dirwindow
	QString cleaner;
	// if dir starts with * it means the file is being modified
	if (dirfiles_path_.startsWith("*"))
		cleaner = dirfiles_path_.mid(1);
	else
		cleaner = dirfiles_path_;

	QFileInfo qf(cleaner);
	QString modif = qf.lastModified().toString();

	if (modif != last_modif_) {
		QString dfiles = getDirfiles();
		setDirfiles(dfiles);
		last_modif_ = modif;
	}


#if 0
	if (!gthread_->isActive() && tw_gdb_ != 0) {
		tw_->removePage(tw_gdb_);
		delete tw_gdb_;
		tw_gdb_ = 0;
		// make the menu invisible
		m_->setItemVisible(mgdbid_, false);
	}
#endif
	if (++timer_cnt_ == 5) {
		autosaveBufMgrAll(b_main_, emain_);
	}
	else if (timer_cnt_ == 6) {
		autosaveBufMgrAll(b_aux_, eaux_);
	}
	else if (timer_cnt_ > 60) {		  // about 5 minutes
		timer_cnt_ = 0;
	}
}


void Editor::autosaveBufMgrAll(BufMgr *bm, BriefEdit *e) {
	ASSERT(bm);
	ASSERT(e);
	// save current editing info in buffer mgr
	storeCurentBuffer(bm, e);
	if (Global::instance()->no_autosave_)
		return;

	// save content of buffer mgr in files
	for (int i = 0; i < MAXBUFMGR; i++) {
		qApp->processEvents();
		QString fname = bm->getFName(i);
		if (fname == "*") {
			fname = "*newfile";
		}

		if (fname.isNull())
			break;
		if (fname.startsWith("*")) {
			QFileInfo qf(fname.mid(1));
			QString fname_only = qf.fileName();
			char *backup_fname = Global::getAutobackupFname((const char *) qf.fileName());
			// generate no warning while saving
			saveCommonText(QString(backup_fname), bm->getFormat(i), bm->getText(i));
			delete[] backup_fname;
		}
	}
}


bool Editor::saveAll() {
	bool retval = true;
	// save main buffer manager
	retval = retval && saveBufMgrAll(b_main_, emain_);
	changed_main_ = false;
	// save aux buffer manager
	retval = retval && saveBufMgrAll(b_aux_, eaux_);
	changed_aux_ = false;

	redoBufMgrList();
	return retval;
}


void Editor::redoBufMgrList() {
	buffers_->clear();
	buffers_->insertItem( tr("&Next"), this, SLOT(nextBuffer()), Qt::ALT+Qt::Key_N);
	buffers_->insertSeparator();
	for (int i = 0; i < MAXBUFMGR; i++) {
		QString fname = b_->getFName(i);
		if (fname.isNull())
			break;
		if (fname.isEmpty())
			fname = "empty buffer";
		buffers_->insertItem(fname, i + 1);
	}
}


void Editor::setFname(QString fname1, FILE_FORMAT format, bool modified) {
	QString fname = fname1;

	if (format != FILE_FORMAT_MAX) {
		ASSERT(ff_ != 0);
		ff_->setText(ff_str(format));
	}

	if (modified) {
		QString new_fname = "*";
		new_fname.append(fname);
		setCaption(new_fname);
	}
	else {
		setCaption(fname);
	}

	//	QSyntaxHighlighter *tmp1 = e_->syntaxHighlighter();
	//printf("old %s syntax %p\n", (const char *) fname, tmp1);
	getSyntax(fname, e_);
	//	e_->update();

	// lucian: leaking memory here
	//	QSyntaxHighlighter *tmp2 = e_->syntaxHighlighter();
	//printf("new %s syntax %p\n", (const char *) fname, tmp2);
	//	if (tmp1 != 0)
	//		delete tmp1;

	// if necessary load latex keywords in dictionary
	if (fname.isNull() || fname.isEmpty())
		return;

	if (!fname.endsWith(".tex") && !fname.endsWith(".sty"))
		return;

	if (pdict_ == 0)
		initDictionary();
	if (Global::instance()->has_latex_ == false) {
		loadLatexKeywords(pdict_);
		Global::instance()->has_latex_ = true;
	}
}


QString Editor::getFname() {
	QString retval = caption();

	if (retval.startsWith("*"))
		retval = retval.mid(1);

	return retval;
}


void Editor::newDoc() {
	LOG("newDoc");
	Editor *ed = new Editor;
	if (qApp->desktop()->size().width() < 450
	|| qApp->desktop()->size().height() < 450) {
		ed->showMaximized();
	}
	else {
		ed->resize(Global::instance()->xsize, Global::instance()->ysize);
		LOG("Editor::newDoc sizes %d %d", Global::instance()->xsize, Global::instance()->ysize);
		ed->show();
	}
}


void Editor::newDocBuffer() {
	LOG("newDocBuffer");
	// save curent text first
	storeCurentBuffer();

	// add new empty buffer
	if (using_aux_)
		changed_aux_ = true;
	else
		changed_main_ = true;

	b_->add("", default_format_);
	setFname("empty buffer", default_format_);
	e_->setText("");
	e_->setCursorPosition(1, 1, false);
	redoBufMgrList();
	if (using_aux_)
		changed_aux_ = false;
	else
		changed_main_ = false;
}




void Editor::msgScreen(QString str, int mstime) {
	status_->message(str, mstime);
	qApp->processEvents();
}


void Editor::setDefaultFormat(FILE_FORMAT format) {
	default_format_ = format;
}




void Editor::print() {
#ifndef ROSPELL_NO_PRINTER
						  // opens printer dialog
	if (printer_.setup(this)) {
						  // we'll set our own margins
		printer_.setFullPage(TRUE);
		QPainter p;
						  // paint on printer
		p.begin( &printer_ );
		p.setFont( e_->font() );
		QFontMetrics fm = p.fontMetrics();
						  // need width/height
		Q3PaintDeviceMetrics metrics( &printer_ );
		// of printer surface
						  // half-inch margin
		const int MARGIN = metrics.logicalDpiX() / 2;
						  // y position for each line
		int yPos        = MARGIN;

		for (int i = 0 ; i < e_->numLines() ; i++) {
			if (printer_.aborted())
				break;
			if (yPos + fm.lineSpacing() > metrics.height() - MARGIN) {
						  // no more room on this page
						  // start new page
				if (!printer_.newPage())
						  // some error
					break;
						  // back to top of page
				yPos = MARGIN;
			}
			p.drawText( MARGIN, yPos, metrics.width() - 2*MARGIN,
				fm.lineSpacing(), ExpandTabs, e_->textLine( i ) );
			yPos += fm.lineSpacing();
		}
		p.end();			  // send job to printer
	}
	cleanMWords();
#endif
}


void Editor::resizeEvent( QResizeEvent * ) {
	LOG("resizeEvent sizes %d %d", width(), height());
	Global::instance()->xsize = width();
	Global::instance()->ysize = height();
}


void Editor::closeEvent( QCloseEvent *event ) {
	LOG("closeEvent");
	event->accept();
	bool torelease = true;

	// save curent text first
	storeCurentBuffer();
	b_main_->saveHistory();
	b_aux_->saveHistory();

	if (b_main_->anyBufChanged() || b_aux_->anyBufChanged()) {
		QString r1 = b_main_->printUnsaved();
		QString r2 = b_aux_->printUnsaved();
		switch (QMessageBox::warning( this, tr("Question"),
			tr("One or more buffers have not been saved:\n\n") + r1 + r2 + tr("Save all?"),
			tr("&Yes"),
			tr("&No"),
			tr("&Cancel Exit"),
		0, 2)) {
						  // yes
			case 0:
				if (saveAll())
					event->accept();
				else {
					event->ignore();
					torelease = false;
				}
				break;
						  // no
			case 1:
				event->accept();
				break;
						  // cancel
			default:
				event->ignore();
				torelease = false;
				break;
		}
	}

	if (torelease) {
		releaseAll();
	}
}


void Editor::closeBuffer() {
	LOG("closeBuffer");
	// save curent buffer first
	storeCurentBuffer();

	b_main_->saveHistory();
	b_aux_->saveHistory();

						  // the text has been changed
	if (using_aux_ && changed_aux_ || (!using_aux_) && changed_main_) {
		switch (QMessageBox::warning( this, tr("Question"),
			tr("Save changes to Document?"),
			tr("&Yes"),
			tr("&No"),
			tr("Cancel"),
		0, 2)) {
						  // yes
			case 0:
				save();
				break;
						  // no
			case 1:
				break;
						  // cancel
			default:
				return;
		}
	}

	// first move some active pointers
//	e_->setText("");
//	setFname("noname", default_format_, false);

	// delete buffer
	int aindex = b_->getActiveIndex();
	b_->del(aindex);

	// restore data
	if (using_aux_) // we will get hit with a textChanged call because of  e_->setText()
		changed_aux_ = true;
	else
		changed_main_ = true;
	e_->setText(b_->getActiveText());
	QString fname = b_->getActiveFName();
	if (fname.isEmpty()) {
		setFname("empty buffer", default_format_, false);
	}
	else
		setFname(fname, b_->getActiveFormat(), false);

	redoBufMgrList();
	int line = b_->getActiveLine();
	int col = b_->getActiveCol();
	e_->setCursorPosition(line, col, false);
	e_->syntaxHighlighter()->rehighlight();
	if (using_aux_)
		changed_aux_ = fname.startsWith("*");
	else
		changed_main_ = fname.startsWith("*");
//	if (fname.isEmpty()) {
//		setCaption("");
//	}
}


void Editor::textChanged() {
	//is coming here for each key press;
	//LOG("Text changed");
	if (using_aux_) {
		if (changed_aux_ == false) {
			QString fname = getFname();
			if (fname == "empty buffer")
				fname = "";

			// setFname will not change the format
			setFname(fname, FILE_FORMAT_MAX, true);
			fname = caption();
			bool isnew = b_->setActiveFName(fname);
			if (isnew)
				b_->add(caption(), default_format_);
			redoBufMgrList();
		}
		changed_aux_ = true;
	}
	else {
		if (changed_main_ == false) {
			QString fname = getFname();
			// setFname will not change the format
			if (fname == "empty buffer")
				fname = "";
			setFname(fname, FILE_FORMAT_MAX, true);
			fname = caption();
			bool isnew = b_->setActiveFName(fname);
			if (isnew) {
				b_->add(caption(), default_format_);
			}
			redoBufMgrList();
		}
		changed_main_ = true;
	}

	cleanMWords();
}


void Editor::clicked(int para, int pos) {
	// coming here for each click in e_
	bool changing = false;
	LOG("clicked %d/%d", para, pos);
	if (using_aux_ == true) {
		storeCurentBuffer();
		changing = true;
	}

	using_aux_ = false;
	e_ = emain_;
	b_ = b_main_;
	book_ = book_main_;
	cleanMWords();

	if (changing) {
		redoBufMgrList();
		QString fname = b_->getActiveFName();
		setFname(fname, b_->getActiveFormat(), false);
		if (tw_bookmarks_->isVisible())
			regenBookmarks();
	}
}


void Editor::clickedAux(int para, int pos) {
	// coming here for each click in eaux_
	bool changing = false;
	LOG("clicked %d/%d", para, pos);
	if (using_aux_ == false) {
		storeCurentBuffer();
		changing = true;
	}

	using_aux_ = true;
	e_ = eaux_;
	b_ = b_aux_;
	book_ = book_aux_;
	cleanMWords();
	if (changing) {
		redoBufMgrList();
		if (b_->isEmpty()) {
			setFname("", default_format_, false);
		}
		else {
			QString fname = b_->getActiveFName();
			setFname(fname, b_->getActiveFormat(), false);
		}

		if (tw_bookmarks_->isVisible())
			regenBookmarks();
	}
}


bool Editor::extractFileLineColumn(const QString &in, QString &filename, int &line, int &column) {
	filename = in.section(":", 0, 0);
	if (filename.isEmpty() || filename.isNull()) {
		LOG("filename section empty or null");
		return false;
	}
	LOG("extracted file: %s", (const char *) filename);

	QString linenum = in.section(":", 1, 1);
	LOG("linenum: %s", (const char *) linenum);
	if (linenum.isEmpty() || linenum.isNull()) {
		LOG("line section empty or null");
		return false;
	}
	line = atoi((const char *) linenum);
	if (line <= 0)
		return false;
	LOG("extracted line: %d", line);

	QString colnum = in.section(":", 2, 2); // from now on we only return true
	if (colnum.isEmpty() || colnum.isNull()) {
		column = 0;
		return true;
	}
	column = atoi((const char *) colnum);
	if (column <= 0)
		column = 0;
	return true;
}


void Editor::clickedTags(int para, int pos) {
	LOG("clickedTw %d/%d", para, pos);
	if (ctags_locked_)
		return;

	QString sline = tw_tags_->textLine(para);
	QString filename;
	int line = 0;
	int column = 0;
	if (extractFileLineColumn(sline, filename, line, column) ||
	extractFileLineColumn(tw_tags_->textLine(0), filename, line, column)) {
		// we have a file and a line number
		// save curent text first
		storeCurentBuffer();
		// open the file in the editor window
		LOG("loading file from ctags %s", (const char *) filename);
		// load the file in current edit window; it is protected not to load already loaded files
		load(filename);
		// move to an absolute file name
		QFileInfo qf(filename);
		filename = qf.absFilePath();
		int index;

		bool foundit = b_->hasFile(filename, &index);
		if (foundit) {
			e_->setCursorPosition(line - 1, 0, false);
			e_->setFocus();
		}
		else {
			BufMgr *b = (b_ == b_main_)? b_aux_: b_main_;
			BriefEdit *e = (e_ == emain_)? eaux_: emain_;
			foundit = b->hasFile(filename, &index);
			if (foundit) {
				e->setCursorPosition(line - 1, 0, false);
				e->setFocus();
			}
		}
	}
}


void Editor::customEvent( QCustomEvent * e ) {
	LOG("Editor::customEvent");
	if (e->type() == 65432)
		ctagsDone();
	else if (e->type() == 65431)
		gdbDisplayLine();
	else if (e->type() == 65430)
		gdbDisplayText((char *) e->data());
}

void Editor::gdbDisplayText(char *txt) {
	if (tw_gdb_ == 0) {
		ASSERT(0);
		return;
	}

	char *txt1 = UTILS::buffer_sed(txt, "\r", " ");

	tw_gdb_->insert(txt1);
	tw_gdb_->update();
	delete [] txt;
	delete [] txt1;
}

void Editor::gdbDisplayLine() {
	LOG("Editor::gdbDisplayLine\n");
	QString dl = gthread_->getDisplayLine();
	clickedUserSearch(dl, 0, 0);
	// assuming the file was loaded in the active editor window,
	// we need to select the running line in this moment
	int line;
	int col;
	e_->getCursorPosition(&line, &col);
	e_->setSelection(line, 0, line + 1, 0);
//	tw_->setCurrentPage(ID_GDB);
	tw_gdb_->setFocus();
}

void Editor::ctagsDone() {
	LOG("Editor::ctagsDone\n");
	if (ctags_locked_)
		return;
	QString msg = thread_->getMsg();
	tw_tags_->setText(msg);
	QString sline = tw_tags_->textLine(0);
	if (sline.endsWith(" single ctag definition, see below...")) {
		QString filename;
		int line = 0;
		int column = 0;
		if (extractFileLineColumn(sline, filename, line, column))
			tw_tags_->setCursorPosition(line, 0, false);
	}
}


void Editor::roDebug() {
	QString result = "";
	if (changed_main_)
		result += "main changed, ";
	if (changed_aux_)
		result += "aux changed";
	result += "\n";

	result += b_main_->roDebug();
	result += b_aux_->roDebug();
	result += thread_->roDebug();
	result += (ctags_locked_)? "ctags locked\n": "";
	QMessageBox::warning(this,"roDebug", result);
}


void Editor::roDevel() {
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString line = e_->textLine(para);
	int lines = e_->numLines();
	int len = line.length();
	QString line1 = line.mid(0, index);
	if (line1.isNull() || line1.isEmpty())
		line1 = "";
	QString line2 = line.mid(index);
	if (line2.isNull() || line2.isEmpty())
		return;

	const char *ptr = (const char *) line2;
	int i = 0;
	while (isalnum(ptr[i]) || ptr[i] == '_')
		i++;

	QString line3 = line2.mid(0, i);
	QString line4 = line2.mid(i);

	line = line1 + "'" + line3 + "'" + line4 + "\n";

	// select the line
	if (para == (lines - 1)) {
		e_->setSelection(para, 0, para, len);
	}
	else {
		e_->setSelection(para, 0, para + 1, 0);
	}

	e_->cut();
	e_->insert(line);
	e_->setCursorPosition(para, index, false);
}


void Editor::userF1() {
	tw_user_->setText("");
	msgScreen(tr("F1, please wait..."));
	BCursor busy;

	QTUTILS::run_program(QString("~/.rospell/userF1.sh"), tw_user_);

	tw_->setCurrentPage(ID_USER);
	msgScreen(tr("done"), 2000);
}


void Editor::userF2() {
	tw_user_->setText("");
	msgScreen(tr("F2, please wait..."));
	BCursor busy;
	QTUTILS::run_program(QString("~/.rospell/userF2.sh"), tw_user_);

	tw_->setCurrentPage(ID_USER);
	msgScreen(tr("done"), 2000);
}


void Editor::userF3() {
	tw_user_->setText("");
	msgScreen(tr("F3, please wait..."));
	BCursor busy;
	QTUTILS::run_program(QString("~/.rospell/userF3.sh"), tw_user_);

	tw_->setCurrentPage(ID_USER);
	msgScreen(tr("done"), 2000);
}


void Editor::userF4() {
	tw_user_->setText("");
	msgScreen(tr("F4, please wait..."));
	BCursor busy;
	QTUTILS::run_program(QString("~/.rospell/userF4.sh"), tw_user_);

	tw_->setCurrentPage(ID_USER);
	msgScreen("done", 2000);
}


bool Editor::extractLineColumn(const QString &in, int &line, int &column) {
	QString linenum = in.section(":", 0, 0);
	LOG("linenum: %s", (const char *) linenum);
	if (linenum.isEmpty() || linenum.isNull()) {
		LOG("line section empty or null");
		return false;
	}
	line = atoi((const char *) linenum);
	if (line <= 0) {
		LOG("failed extractLine: %s, line=%d", (const char *) linenum, line);
		return false;
	}
	LOG("extracted line: %d", line);

	QString colnum = in.section(":", 1, 1); // from now on we only return true
	if (colnum.isEmpty() || colnum.isNull()) {
		column = 0;
		return true;
	}
	column = atoi((const char *) colnum);
	if (column <= 0)
		column = 0;

	return true;
}


void Editor::clickedUserSearch(QString sline, int para, int pos) {
	(void) para;
	(void) pos;
	QString filename;
	int line = 0;
	int column = 0;

	// only in the active window!!!
	if (extractLineColumn(sline, line, column)) {
		// we have a line number in the current file
		e_->setCursorPosition(line - 1, column, false);
		e_->setFocus();
	}
	else if (extractFileLineColumn(sline, filename, line, column)) {
		// we have a file and a line number
		// save curent text first
		storeCurentBuffer();
		// open the file in the editor window
		LOG("loading file from ctags %s", (const char *) filename);
		// load the file in current edit window; it is protected not to load already loaded files
		load(filename);
		// move to an absolute file name
		QFileInfo qf(filename);
		filename = qf.absFilePath();
		int index;

		bool foundit = b_->hasFile(filename, &index);
		if (foundit) {
			e_->setCursorPosition(line - 1, column, false);
			e_->setFocus();
		}
		else {
			BufMgr *b = (b_ == b_main_)? b_aux_: b_main_;
			BriefEdit *e = (e_ == emain_)? eaux_: emain_;
			foundit = b->hasFile(filename, &index);
			if (foundit) {
				e->setCursorPosition(line - 1, column, false);
				e->setFocus();
			}
		}
	}

}


void Editor::clickedSearch(int para, int pos) {
	LOG("clickedSearch %d/%d", para, pos);
	QString sline = tw_search_->textLine(para);
	clickedUserSearch(sline, para, pos);
}


void Editor::clickedUser(int para, int pos) {
	(void) pos;

	LOG("clickedUser %d/%d", para, pos);
	QString sline = tw_user_->textLine(para);
	clickedUserSearch(sline, para, pos);
}


void Editor::clickedBookmarks(int para, int pos) {
	(void) pos;
	int line = para / 2 + 1;
	bool found = false;
	int bookmark;
	for (bookmark = 0; bookmark < MAXBOOKMARKS; bookmark++) {
		if (book_[bookmark].fname_.isEmpty())
			continue;
		line--;
		if (line == 0) {
			found = true;
			break;
		}
	}
	if (found)
		gotoBookmarkCommon(bookmark);
}


void Editor::clickedHex(int para, int pos) {
	(void) para;
	(void) pos;

	QString hex = "";
	int lineno;
	int col;
	e_->getCursorPosition(&lineno, &col);
	QString line = e_->textLine(lineno);
	Q3CString uline = line.utf8();
	const char *ptr1 = uline;
	int len = strlen(ptr1);
	unsigned char *ptr = (unsigned char *) ptr1;

	char start[10000];
	char *hexline = start;
	int i;
	char str[17];
	memset(str, '\0', 17);

	for (i = 0; i < len; i++, ptr++) {
		if (i % 16 == 0) {
			sprintf(hexline, "%s\n", str);
			hex += start;
			hexline = start;
		}

		sprintf(hexline, "%02x ", *ptr);
		hexline += strlen(hexline);
		if (isprint(*ptr))
			str[i % 16] = *ptr;
		else
			str[i % 16] = '.';
	}
	str[i % 16] = '\0';
	for (i = 15 - (len % 16); i >= 0; i--) {
		sprintf(hexline, "   ");
		hexline += strlen(hexline);
	}
	sprintf(hexline, "%s\n", str);
	hex += start;

	tw_hex_->setText(hex);
}


QString Editor::getDirfiles() {
	QString cleaner;
	// if dir starts with * it means the file is being modified
//	if (dirfiles_path_.startsWith("*"))
//		cleaner = dirfiles_path_.mid(1);
//	else
//		cleaner = dirfiles_path_;

	FVBase fvb(dirfiles_path_, dircombo_->currentText(), "", false);
	FVisitorDirfiles visitor;
	return fvb.walk(&visitor);
}


void Editor::clickedDirfiles(int para, int pos) {
	LOG("clickedDirfiles %d/%d", para, pos);
	BCursor busy;
	QString file = dirfiles_->textLine(para);
	QFileInfo qf(dirfiles_path_ + file);
	if (file == "..") {
		dirfiles_path_ += "../";
		QString dfiles = getDirfiles();
		setDirfiles(dfiles);
	}
	else if (qf.isFile()) {
		load(dirfiles_path_ + file);
	}
	else if (qf.isDir()) {
		dirfiles_path_ += file + "/";
		QString dfiles = getDirfiles();
		setDirfiles(dfiles);
	}
	else
		printf("??????\n");

						  //clean any selection!!!
	dirfiles_->setCursorPosition(para, pos, false);
	e_->setFocus();
}


void Editor::twChanged(QWidget* qm) {
	tw_current_ = (Q3MultiLineEdit *) qm;
	if (qm == tw_hex_) {
		//		sleep(2);
		//		emit tw_hex_->clicked(0,0);
		clickedHex(0, 0);
	}
}


void Editor::dirboxChanged(int index) {
	(void) index;
	QString dfiles = getDirfiles();
	setDirfiles(dfiles);
}


void Editor::initDictionary() {
	BCursor busy;
	Global *glb = Global::instance();
	if (pdict_ != 0)
		delete pdict_;
	pdict_ = new Dictionaries(this); // read master, read user, save user
	if (glb->has_latex_ == true) {
		loadLatexKeywords(pdict_);
	}


	enableDisableMenus();
	ASSERT(pdict_ != 0);
}

void Editor::enableDisableMenus() {
	Global *glb = Global::instance();

	if (glb->lang == LANG_EN_US ||
	    glb->lang == LANG_RO_CEDILLA ||
	    glb->lang == LANG_RO_COMMABELOW)

		tools_->setItemEnabled(dexid_, true);
	else
		tools_->setItemEnabled(dexid_, false);

	if (glb->lang == LANG_RO_CEDILLA ||
	    glb->lang == LANG_RO_COMMABELOW) {
		cleaners_->setItemEnabled(ced2comaid_, true);
		cleaners_->setItemEnabled(coma2cedid_, true);
		cleaners_->setItemEnabled(addroid_, true);
		cleaners_->setItemEnabled(remroid_, true);
	}
	else {
		cleaners_->setItemEnabled(ced2comaid_, false);
		cleaners_->setItemEnabled(coma2cedid_, false);
		cleaners_->setItemEnabled(addroid_, false);
		cleaners_->setItemEnabled(remroid_, false);
	}
}

void Editor::changeFormat() {
	LOG("Editor::changeFormat");
	cleanMWords();
	ChangeFormatDialog *cf = new ChangeFormatDialog();
	cf->setCaption(tr("Format"));
	if (!cf->exec()) {
		delete cf;
		return;
	}
	FILE_FORMAT format = ff_format((const char *) cf->getFormatString());
	if (cf->getChangeGlobal())
		default_format_ = format;
	else {
		if (b_ == 0 || ff_ == 0) {
			ASSERT(0);
			return;
		}
		ff_->setText(ff_str(format));

		b_->setActiveFormat(format);
	}
}


