// Copyright (C) 2005,  Lucian Constantin <struct_bylighting@yahoo.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//****************************************************************************
//** $Id: qt/qwerty.h   3.3.4   edited May 27 2003 $
//**
//** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
//**
//** This file is part of an example program for Qt.  This example
//** program may be used, distributed and modified without limitation.
//**
//*****************************************************************************/

#ifndef QWERTY_H
#define QWERTY_H

#include "global.h"

#ifdef USING_QT4
//Added by qt3to4:
#include <QCloseEvent>
#include <QResizeEvent>
#include <Q3PopupMenu>
#endif


#include <qwidget.h>
#include <qmenubar.h>
#include <qmainwindow.h>



#ifdef USING_QT3
#include <qlayout.h>
#include <qmultilineedit.h>
#include <qhbox.h>
#include <qvbox.h>
#include <qaccel.h>
#include <qsyntaxhighlighter.h>
#endif
#ifdef USING_QT4
#include <Q3VBoxLayout>
#include <q3multilineedit.h>
#include <q3hbox.h>
#include <q3vbox.h>
#include <q3accel.h>
#include <q3syntaxhighlighter.h>
#endif

#include <qprinter.h>
#include <qdialog.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qfont.h>
#include <qlayout.h>
#include <qstatusbar.h>
#include <qtabwidget.h>
#include <qtimer.h>

// forward declarations
class BufMgr;
class CtagsThread;
class GdbThread;
class GdbEdit;
class Dictionaries;
class BriefEdit;
class LineAndColumn;
class InsertOverwrite;
class StatusField;
class QComboBox;
class Q3VBox;
class QToolBar;

class Editor : public QMainWindow {
//public Q3VBox {
	Q_OBJECT

	// id for tag windows
	static const int ID_TAGS = 0;
	static const int ID_SEARCH = 1;
	static const int ID_USER = 2;
	static const int ID_BOOKMARKS = 3;
	static const int ID_HEX = 4;
	static const int ID_LAST_STATIC = 4; // this is the last static one
	// id for dinamic gdb tags
	static const int ID_GDB = ID_LAST_STATIC + 1;
public:
	Editor( QWidget *parent=0, const char *name="qwerty" );
	~Editor();
	void load(const QString& fileName, bool absolute = false);
	void setDefaultFormat(FILE_FORMAT format);
	void setFname(QString fname, FILE_FORMAT format, bool modified = false);
	QString getFname();
	bool advanceSearch(QString tofind, bool ignore_case = false, bool whole = false);
	bool advanceSearchList(QString tofind, bool ignore_case = false, bool whole = false);
	bool advanceSearchRev(QString tofind, bool ignore_case = false, bool whole = false);
	bool advanceSearchListRev(QString tofind, bool ignore_case = false, bool whole = false);
	void msgScreen(QString str, int mstime = 0);
	GdbEdit *getGdbTab() {
		return tw_gdb_;
	}
	Q3VBox *qvb_;

public slots:
	void newDoc();
	void newDocBuffer();
	void load();
	void openWwwLocation();
	void openRecent0();
	void openRecent1();
	void openRecent2();
	void openRecent3();
	void openRecent4();
	void openRecent5();
	bool save();
	bool saveAs();
	bool saveAll();
	void changeFormat();
	void print();
	void font();
	void spellAdvance();
	void spellIgnore();
	void spellAdd();
	void undo();
	void redo();
	void cut();
	void copy();
	void paste();
	void selectBlock();
	void e_delete();
	void search();
	void searchAgain();
	void replace();
	void toc();
	void gotoLine();
	void setBookmark();
	void gotoBookmark();
	void insertHex();
	void status();
	void customize();
	void uppercase();
	void lowercase();
	void dexonline();
	void unixCleanup();
	void stripTrailingSpaces();
	void tabToSpace();
	void nextBuffer();
	void closeBuffer();
	void diff();
	void dirMerge();
	void sessionDiff();
	void merge();
	void beautify();
	void htmltidy();
	void kloc();
	void make();
	void wordCount();
	void toggleSpeller();
	void cedilla2commabelow();
	void commabelow2cedilla();
	void autospell();
	void removeDiacritics();
	void colors();
	void configCtags();
	void lockCtags();
	void unlockCtags();
	void configLanguage();
	void configWWW();
	void grep();

	// events coming here from ctag or gdb thread
	void customEvent( QCustomEvent * e );
	// are routed as follows:
	void ctagsDone();
	void gdbDisplayLine();
	void gdbDisplayText(char *txt);

	void selectAll();
	void toggleOverwrite();
	void recordAction();
	void playbackAction();
	void editAlias();

	void editF1();
	void editF2();
	void editF3();
	void editF4();

	// gdb
	void gdbEnable();
	void gdbRunRestart();
	void gdbStepOver();
	void gdbStepInto();
	void gdbStepOut();
	void gdbRunCursor();
	void gdbStop();
	void gdbExit();
	void gdbContinue();


protected:
	void setSpellerBar();
	void resizeEvent(QResizeEvent *);
	void closeEvent(QCloseEvent *);

private slots:
	void pickBuffer(int id);
	void pickBuffer(BufMgr *b, BriefEdit *e, int id);
	void textChanged();
	void clicked(int para, int pos);
	void clickedAux(int para, int pos);
	void clickedDirfiles(int para, int pos);
	void clickedTags(int para, int pos);
	void clickedSearch(int para, int pos);
	void clickedUser(int para, int pos);
	void clickedBookmarks(int para, int pos);
	void clickedHex(int para, int pos);
	void twChanged(QWidget* qm);
	void dirboxChanged(int index);
	// sepeller-sugested words
	void replaceWord1();
	void replaceWord2();
	void replaceWord3();
	void replaceWord4();
	void replaceWord5();
	void cursorPosChanged(int para, int pos);
	// bookmarks
	void setBookmark1();
	void setBookmark2();
	void setBookmark3();
	void setBookmark4();
	void setBookmark5();
	void setBookmark6();
	void setBookmark7();
	void setBookmark8();
	void setBookmark9();
	void setBookmark0();
	// user progamable keys
	void userF1();
	void userF2();
	void userF3();
	void userF4();
	// file options
	void fileOptions();
	// start/end of buffer
	void startOfBuffer();
	void endOfBuffer();
	// debug
	void roDebug();
	void roDevel();
	// latex templates
	void latexT1();
	void latexT2();
	void latexT3();
	void latexT4();
	// latex features
	bool isLatex();
	void latexF1();
	void latexF2();
	void latexF3();
	void latexF4();
	void latexF5();
	void latexF6();
	void latexF7();
	void latexF8();
	void latexF9();
	void latexF10();
	void latexF11();
	void latexF12();
	// latex font sizes
	void latexFS1();
	void latexFS2();
	void latexFS3();
	void latexFS4();
	void latexFS5();
	void latexFS6();
	void latexFS7();
	void latexFS8();
	void latexFS9();
	void latexFS10();
	// latex font type
	void latexFT1();
	void latexFT2();
	void latexFT3();
	void latexFT4();
	void latexFT5();
	// help support
	void helpAbout();
	void helpAboutQt();
	void helpGettingStarted();
	void helpBasicOperation();
	void helpProgramming();
	void helpKeyboard();
	void slotTimeout();


private:
	QTimer timer_;
	int timer_cnt_;
	void updateRecent();
	void initDictionary();
	bool saveBufMgrAll(BufMgr *bm, BriefEdit *e);
	void autosaveBufMgrAll(BufMgr *bm, BriefEdit *e);
	void clickedUserSearch(QString sline, int para, int pos);
	void regenBookmarks();
	void setBookmarkCommon(int bookmark);
	void gotoBookmarkCommon(int bookmark);
	void addLatexMenu();
	void addGdbMenu();
	void addHelpMenu();
	void releaseAll();
	bool saveCommon( const QString& fileName, FILE_FORMAT format);
	bool saveCommonText(const QString& fileName, FILE_FORMAT format, QString text);
	FILE_FORMAT detectWwwFormat(const char *str);
	void advanceStartWord();
	QString advanceEndWord();
	bool skeepXML();

	void cleanMWords();
	// lookalike words entry point
	void doLookalike(char *active_word);
	// insert lookalike in empty storage word
	void insertLookalike(const char *str);
	// save curent buffer in bufmgr
	void storeCurentBuffer();
	void storeCurentBuffer(BufMgr *b, BriefEdit *e);
	// get the names of the files to put in dirfiles window
	QString getDirfiles();
	void setDirfiles(QString dfiles);

	void redoBufMgrList();
	bool extractFileLineColumn(const QString &in, QString &filename, int &line, int &column);
	bool extractLineColumn(const QString &in, int &line, int &column);
	void enableDisableMenus();
	void updateDirpath(QString filename);

	Q3Accel *a_;
	Q3PopupMenu *file_;
	Q3PopupMenu *edit_;
	Q3PopupMenu *tools_;
	int dexid_; // id for tools/dexonline
	Q3PopupMenu *cleaners_;
	Q3PopupMenu *code_;
	int ced2comaid_;
	int coma2cedid_;
	int addroid_;
	int remroid_;
	QMenuBar       *m_;
	QMenuBar       *mwords_;
	int mw1_;	//id for the first word in mwords_ menu bar
	int mw2_;	//id for the second word in mwords_ menu bar
	int mw3_;	//id for the third word in mwords_ menu bar
	int mw4_;	//id for the fourth word in mwords_ menu bar
	int mw5_;	//id for the fifth word in mwords_ menu bar
	char word1_[100];
	char word2_[100];
	char word3_[100];
	char word4_[100];
	char word5_[100];
	Q3PopupMenu *mgdb_;
	int mgdbid_; // id for mgdb_ menu in menubar
	int rc0_;
	int rc1_;
	int rc2_;
	int rc3_;
	int rc4_;
	int rc5_;

	// bookmarks
	struct Bookmark {
		int x_;
		int y_;
		QString fname_;
		QString line_;
		Bookmark(): x_(0), y_(0), fname_(""), line_("") {}
	};

	// active edit window
	BriefEdit *e_;
	bool using_aux_;
	BufMgr *b_;
	Bookmark* book_;

	// main edit window
	BriefEdit *emain_;
	bool changed_main_;
	BufMgr *b_main_;
	Bookmark book_main_[MAXBOOKMARKS];

	// aux edit window
	BriefEdit *eaux_;
	bool changed_aux_;
	BufMgr *b_aux_;
	Bookmark book_aux_[MAXBOOKMARKS];

	// dirfiles
	QComboBox *dircombo_;
	Q3MultiLineEdit *dirfiles_;
	QString dirfiles_path_;
	QString last_modif_;
	int dirfiles_size_;
#ifndef ROSPELL_NO_PRINTER
	QPrinter        printer_;
#endif
	Dictionaries *pdict_;
	QString search_;		// copy of the last search string
	bool ignore_case_;	// ignore case flag for the last search
	bool whole_;		// whole word flag for the last search
	bool backwards_;	// backwards search flag
	FILE_FORMAT default_format_;

	QString line_;
	int x_;
	int y_;
	int xword_;
	int yword_;

	// buffer manager
	Q3PopupMenu *buffers_;

	// user area - containing exclusively only Q3MultiLineEdit
	QTabWidget *tw_;
	int tw_size_;
	Q3MultiLineEdit *tw_current_;
	Q3MultiLineEdit *tw_user_;
	Q3MultiLineEdit *tw_search_;
	Q3MultiLineEdit *tw_tags_;
	Q3MultiLineEdit *tw_bookmarks_;
	Q3MultiLineEdit *tw_hex_;
	GdbEdit *tw_gdb_;
	// status bar
	QStatusBar *status_;
	StatusField *lac_; // line and column
	StatusField *attention_; // attention: selections, record, etd
	StatusField *iow_; // insert overwrite
	StatusField *ff_; // format
	StatusField *lang_; // language

	// ctags, gdb
	CtagsThread *thread_;
	GdbThread *gthread_;

	// mouse press/release y location to resize tags/search/usr window or dirfiles window
	int mouse_press_y_;
	int mouse_press_x_;
	bool resize_dirfiles_;

	// ctags locking/unlocking
	bool ctags_locked_;

	// show/hide toggle for speller menu
	bool show_speller_;

	// toolbars
	QToolBar *tool1_;
	QToolBar *tool2_;
	QToolBar *tool3_;
	QToolBar *tool4_;

};

#endif // QWERTY_H








