// Copyright (C) 2005,  Lucian Constantin <struct_bylighting@yahoo.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#ifndef UTF8_H
#define UTF8_H
//#include <string>
#include "debug.h"

class UTF8 {
public:
	static inline unsigned utf8CharLen(char *str) {
		ASSERT(str != 0);
		unsigned char *str1 = (unsigned char *) str;
		if (!(str1[0] & 0x80))
			return 1;
		else if (((str1[0] & 0xe0) == 0xc0) &&
			    ((str1[1] & 0xc0) == 0x80) &&
			    ((str1[2] & 0xc0) != 0x80))
			return 2;
		else if (((str1[0] & 0xf0) == 0xe0) &&
			    ((str1[1] & 0xc0) == 0x80) &&
			    ((str1[2] & 0xc0) == 0x80) &&
			    ((str1[3] & 0xc0) != 0x80))
			return 3;
		else if (((str1[0] & 0xf8) == 0xf0) &&
			    ((str1[1] & 0xc0) == 0x80) &&
			    ((str1[2] & 0xc0) == 0x80) &&
			    ((str1[3] & 0xc0) == 0x80) &&
			    ((str1[4] & 0xc0) != 0x80))
			return 4;
		return 0;
	}
#if 0
	static bool isutf8RomanianChar(char *str) {
		ASSERT(str != 0);
		unsigned len = utf8CharLen(str);
		if (len != 2)
			return false;
		unsigned char *str1 = (unsigned char *) str;
		// ăÂ
		if (str1[0] == 0xc4 && (str1[1] == 0x83 || str1[1] == 0x82))
			return true;
		// şŞţŢ
		else if (str1[0] == 0xc5 && (str1[1] == 0x9f || str1[1] == 0x9e || str1[1] == 0xa3 || str1[1] == 0xa2))
			return true;
		// âÂîÎ
		else if (str1[0] == 0xc3 && (str1[1] == 0xa2 || str1[1] == 0x82 || str1[1] == 0xae || str1[1] == 0x8e))
			return true;
		// ȘșȚț
		else if (str1[0] == 0xc8 && (str1[1] == 0x98 || str1[1] == 0x99 || str1[1] == 0x9a || str1[1] == 0x9b))
			return true;

		return false;
	}
#endif

	static inline char *strtoascii(const char *str) {
		ASSERT(str != 0);
		char *retval = new char[strlen(str) + 1];
		char *ptr1 = (char *) str;
		char *ptr2 = retval;
		while (*ptr1 != '\0') {
			int rv = UTF8::chartoascii(ptr1, *ptr2);
			// look for skipped chars
			if (*ptr2 != 0) {
				ptr2++;
			}
			else {
				memcpy(ptr2, ptr1, rv);
				ptr2 += rv;
			}
			ptr1 += rv;
		}
		*ptr2 = '\0';
		ASSERT(std::strlen(retval) <= std::strlen(str));
		return retval;
	}

	static inline char *strtolower(const char *str) {
		ASSERT(str != 0);
		char *retval = new char[strlen(str) + 1];
		strcpy(retval, str);
		char *ptr = retval;
		while (*ptr != '\0') {
			unsigned len = UTF8::tolower(ptr);
			ptr += len;
		}
		*ptr = '\0';
		return retval;
	}

	static inline char *strtoupper(const char *str) {
		ASSERT(str != 0);
		char *retval = new char[strlen(str) + 1];
		strcpy(retval, str);
		char *ptr = retval;
		while (*ptr != '\0') {
			unsigned len = UTF8::toupper(ptr);
			ptr += len;
		}
		*ptr = '\0';
		return retval;
	}

	static inline char *strcommabelow2cedilla(const char *str) {
		ASSERT(str != 0);
		char *retval = new char[strlen(str) + 1];
		strcpy(retval, str);
		char *ptr = retval;
		while (*ptr != '\0') {
			unsigned len = UTF8::commabelow2cedilla(ptr);
			ptr += len;
		}
		*ptr = '\0';
		return retval;
	}

	static inline char *strcedilla2commabelow(const char *str) {
		ASSERT(str != 0);
		char *retval = new char[strlen(str) + 1];
		strcpy(retval, str);
		char *ptr = retval;
		while (*ptr != '\0') {
			unsigned len = UTF8::cedilla2commabelow(ptr);
			ptr += len;
		}
		*ptr = '\0';
		return retval;
	}

	static bool isupper(char *str) {
		unsigned char *str1 = (unsigned char *) str;
		if (*str1 != 0x80)
			return std::isupper(*str1);


		if (str1[0] == 0xc4 && str1[1] == 0x82) { // Ă
			return true;
		} else if (str1[0] == 0xc5 && str1[1] == 0x9e) { // Ş
			return true;
		} else if (str1[0] == 0xc3 && str1[1] == 0x82) { // Â
			return true;
		} else if (str1[0] == 0xc5 && str1[1] == 0xa2) { // Ţ
			return true;
		} else if (str1[0] == 0xc3 && str1[1] == 0x8e) { // Î
			return true;
		} else if (str1[0] == 0xc8 && str1[1] == 0x98) { // Ș
			return true;
		} else if (str1[0] == 0xc8 && str1[1] == 0x9a) { // Ț
			return true;
		}

		return false;
	}

	static unsigned tolower(char *str) {
		ASSERT(str != 0);
		if (!(str[0] & 0x80)) {
			str[0] = std::tolower(str[0]);
			return 1;
		}
		unsigned char *str1 = (unsigned char *) str;
		if (str1[0] == 0xc4 && str1[1] == 0x82) {
			str1[1] = 0x83;
			return 2;
		} else if (str1[0] == 0xc5 && str1[1] == 0x9e) {
			str1[1] = 0x9f;
			return 2;
		} else if (str1[0] == 0xc3 && str1[1] == 0x82) {
			str1[1] = 0xa2;
			return 2;
		} else if (str1[0] == 0xc5 && str1[1] == 0xa2) {
			str1[1] = 0xa3;
			return 2;
		} else if (str1[0] == 0xc3 && str1[1] == 0x8e) {
			str1[1] = 0xae;
			return 2;
		} else if (str1[0] == 0xc8 && str1[1] == 0x98) { // Ș
			str1[1] = 0x99;
			return 2;
		} else if (str1[0] == 0xc8 && str1[1] == 0x9a) { // Ț
			str1[1] = 0x9b;
			return 2;
		}

		// chars that are not pure ascii or are not recognized as romanian characters
		// will be left unchanged
		unsigned retval = utf8CharLen((char *) str1);
		return retval;
	}

	static unsigned toupper(char *str) {
		ASSERT(str != 0);
		if (!(str[0] & 0x80)) {
			str[0] = std::toupper(str[0]);
			return 1;
		}
		unsigned char *str1 = (unsigned char *) str;
		if (str1[0] == 0xc4 && str1[1] == 0x83) {
			str1[1] = 0x82;
			return 2;
		} else if (str1[0] == 0xc5 && str1[1] == 0x9f) {
			str1[1] = 0x9e;
			return 2;
		} else if (str1[0] == 0xc3 && str1[1] == 0xa2) {
			str1[1] = 0x82;
			return 2;
		} else if (str1[0] == 0xc5 && str1[1] == 0xa3) {
			str1[1] = 0xa2;
			return 2;
		} else if (str1[0] == 0xc3 && str1[1] == 0xae) {
			str1[1] = 0x8e;
			return 2;
		} else if (str1[0] == 0xc8 && str1[1] == 0x99) { // ș
			str1[1] = 0x98;
			return 2;
		} else if (str1[0] == 0xc8 && str1[1] == 0x9b) { // ț
			str1[1] = 0x9a;
			return 2;
		}

		// chars that are not pure ascii or are not recognized as romanian characters
		// will be left unchanged
		unsigned retval = utf8CharLen((char *) str1);
		return retval;
	}

	static unsigned chartoascii(char *str, char &c) {
		ASSERT(str != 0);
		if (!(str[0] & 0x80)) {
			c = str[0];
			return 1;
		}
		unsigned char *str1 = (unsigned char *) str;
		if (str1[0] == 0xc4 && str1[1] == 0x83) {
			c = 'a';
			return 2;
		} else if (str1[0] == 0xc4 && str1[1] == 0x82) {
			c = 'A';
			return 2;
		} else if (str1[0] == 0xc5 && str1[1] == 0x9f) {
			c = 's';
			return 2;
		} else if (str1[0] == 0xc5 && str1[1] == 0x9e) {
			c = 'S';
			return 2;
		} else if (str1[0] == 0xc3 && str1[1] == 0xa2) {
			c = 'a';
			return 2;
		} else if (str1[0] == 0xc3 && str1[1] == 0x82) {
			c = 'A';
			return 2;
		} else if (str1[0] == 0xc5 && str1[1] == 0xa3) {
			c = 't';
			return 2;
		} else if (str1[0] == 0xc5 && str1[1] == 0xa2) {
			c = 'T';
			return 2;
		} else if (str1[0] == 0xc3 && str1[1] == 0xae) {
			c = 'i';
			return 2;
		} else if (str1[0] == 0xc3 && str1[1] == 0x8e) {
			c = 'I';
			return 2;
		} else if (str1[0] == 0xc8 && str1[1] == 0x99) { // ș
			c = 's';
			return 2;
		} else if (str1[0] == 0xc8 && str1[1] == 0x9b) { // ț
			c = 't';
			return 2;
		} else if (str1[0] == 0xc8 && str1[1] == 0x98) { // Ș
			c = 'S';
			return 2;
		} else if (str1[0] == 0xc8 && str1[1] == 0x9a) { // Ț
			c = 'T';
			return 2;
		}
		c = 0;
		// chars that are not pure ascii or are not recognized as romanian characters
		// will be left unchanged
		unsigned retval = utf8CharLen((char *) str1);
		return retval;
	}

private:
	static unsigned commabelow2cedilla(char *str) {
		ASSERT(str != 0);
		if (!(str[0] & 0x80)) {
			return 1;
		}
		unsigned char *str1 = (unsigned char *) str;
		if (str1[0] == 0xc8 && str1[1] == 0x99) { // ș
			str1[0] = 0xc5;
			str1[1] = 0x9f;
			return 2;
		} else if (str1[0] == 0xc8 && str1[1] == 0x9b) { // ț
			str1[0] = 0xc5;
			str1[1] = 0xa3;
			return 2;
		} else if (str1[0] == 0xc8 && str1[1] == 0x98) { // Ș
			str1[0] = 0xc5;
			str1[1] = 0x9e;
			return 2;
		} else if (str1[0] == 0xc8 && str1[1] == 0x9a) { // Ț
			str1[0] = 0xc5;
			str1[1] = 0xa2;
			return 2;
		}

		// chars that are not pure ascii or are not recognized as romanian characters
		// will be left unchanged
		unsigned retval = utf8CharLen((char *) str1);
		return retval;
	}

	static unsigned cedilla2commabelow(char *str) {
		ASSERT(str != 0);
		if (!(str[0] & 0x80)) {
			return 1;
		}
		unsigned char *str1 = (unsigned char *) str;
		if (str1[0] == 0xc5 && str1[1] == 0x9f) { // ş
			str1[0] = 0xc8;
			str1[1] = 0x99;
			return 2;
		} else if (str1[0] == 0xc5 && str1[1] == 0xa3) { // ţ
			str1[0] = 0xc8;
			str1[1] = 0x9b;
			return 2;
		} else if (str1[0] == 0xc5 && str1[1] == 0x9e) { // Ş
			str1[0] = 0xc8;
			str1[1] = 0x98;
			return 2;
		} else if (str1[0] == 0xc5 && str1[1] == 0xa2) { // Ţ
			str1[0] = 0xc8;
			str1[1] = 0x9a;
			return 2;
		}

		// chars that are not pure ascii or are not recognized as romanian characters
		// will be left unchanged
		unsigned retval = utf8CharLen((char *) str1);
		return retval;
	}

};

#endif
